# jedit_cmds.tcl - most user-visible commands for jedit
#   (preference commands are in jedit_prefs.tcl)
#
######################################################################
#
# NOTE: these mostly take the arguments "t args", where t is the text
# widget they apply to.  That way they can be used in a j:tkb:mkmap
# table (where %W %K %A will be appended to the command before 
# execution) as well as with just %W in bindings.  In a few cases
# where t would be ignored, they just take "args".
#
######################################################################

j:ldb:set_defaults {
  {JEconfirm:close_window {Are you sure you want to close this window?}}
  {JEconfirm:quit {Are you sure you want to quit?}}
  {JEprompt:saveas {Save as:}}
  {JEconfirm:...exists_replace {File "$filename" exists; replace it?}}
  {JEconfirm:print_to...
    {Print using `lpr' to printer `$J_PREFS(printer)'?}}
  {JEconfirm:print_ps_to...
    {Print as PostScript using `lpr' to printer `$J_PREFS(printer)'?}}
  {JEprompt:insert {Insert:}}
  {JEtitle:note {Note}}
  {JEprompt:go_to_line {Go to line number:}}
  {JEmsg:at_line...column...
    {The insertion point is at line $line, column $column.}}
  {JEbutton:done {Done}}

}
######################################################################
# prompt user for mode
######################################################################

j:command:register jedit:cmd:ask_mode {Mode...}
proc jedit:cmd:ask_mode { t args } {
  global MODE

  set window [jedit:text_to_top $t]
  set prompt_result [j:prompt -text JEprompt:mode]
  if {$prompt_result != {}} then {
    jedit:set_mode $t $prompt_result
    jedit:apply_mode $window
  }
}



######################################################################
# close a window
######################################################################

j:command:register jedit:cmd:close {Close Window}
proc jedit:cmd:close { t args } {
  global JEDIT_WINDOW_COUNT

  if {$JEDIT_WINDOW_COUNT == 1} {
    jedit:cmd:quit $t
  } else {
    set mode [jedit:get_mode $t]
    if {[info procs mode:$mode:pre_close_hook] != {}} {
      mode:$mode:pre_close_hook $t
    }

    if {[info procs mode:$mode:close] == {}} {
      if [j:confirm -text JEconfirm:close_window] {
        incr JEDIT_WINDOW_COUNT -1      ;# one fewer window
        destroy [jedit:text_to_top $t]
      }
    } else {
      mode:$mode:close $t
    }
  }
}

######################################################################
# quit the editor
######################################################################

j:command:register jedit:cmd:quit {Quit}
proc jedit:cmd:quit { t args } {
  set mode [jedit:get_mode $t]
  
  if {[info procs mode:$mode:pre_quit_hook] != {}} {
    mode:$mode:pre_quit_hook $t
  }
  
  if {[info procs mode:$mode:quit] == {}} {
    if [j:confirm -text JEconfirm:quit] {
      exit 0
    }
  } else {
    mode:$mode:quit $t
  }
}

######################################################################
# read in a file
######################################################################

j:command:register jedit:cmd:load {Load...}
proc jedit:cmd:load { t args } {
  jedit:save_checkpoint $t	;# save undo information

  set filename [j:fs -prompt "Load:"]
  if {"x$filename" != "x"} then {
    jedit:set_filename $t $filename
    
    # if new filename should have a different mode, set it:
    set old_mode [jedit:get_mode $t]
    set new_mode [jedit:guess_mode $filename]
    if {[string compare $old_mode $new_mode] != 0} {
      jedit:set_mode $t $new_mode
      jedit:apply_mode $t
    }
    
    jedit:read $filename $t
  }
}
  
######################################################################
# write out a file, using window's filename if defined
######################################################################

j:command:register jedit:cmd:save {Save}
proc jedit:cmd:save { t args } {
  set filename [jedit:get_filename $t]
  if {"x$filename" != "x"} then {
    jedit:write $filename $t
  } else {
    set filename [j:fs -prompt "Save as:"]
    if {"x$filename" != "x"} then {
      jedit:set_filename $t $filename
      jedit:set_label [jedit:text_to_top $t]
      jedit:write $filename $t
    }
  }
}

######################################################################
# write out a file, prompting for a filename
######################################################################

j:command:register jedit:cmd:saveas {Save As...}
proc jedit:cmd:saveas { t args } {
  set filename [j:fs -prompt JEprompt:saveas]
  if {"x$filename" != "x" && \
     ( ! [file exists $filename] || \
      [j:confirm -text JEconfirm:...exists_replace] )} then {
    jedit:set_filename $t $filename
    jedit:set_label [jedit:text_to_top $t]
    jedit:write $filename $t
  }
}

######################################################################
# print the file using lpr
######################################################################

j:command:register jedit:cmd:print {Print...}
proc jedit:cmd:print { t args } {
  global J_PREFS
  if [j:confirm -priority 24 \
    -text JEconfirm:print_to...] {
    exec lpr -P$J_PREFS(printer) << [$t get 1.0 end]
  }
}

######################################################################
# print rich-text as postscript using lpr
######################################################################

j:command:register jedit:cmd:print_postscript {Print PostScript}
proc jedit:cmd:print_postscript { t args } {
  global J_PREFS
  if [j:confirm -priority 24 \
    -text JEconfirm:print_ps_to...] {
    exec lpr -P$J_PREFS(printer) << [j:tc:ps:convert_text $t]
  }
}

######################################################################
# read in a file and insert it at the insert mark
######################################################################

j:command:register jedit:cmd:insfile {Insert File...}
proc jedit:cmd:insfile { t args } {
  jedit:save_checkpoint $t			;# save undo information
  set prompt_result [j:fs -prompt JEprompt:insert]
  if {$prompt_result != {}} then {
    j:text:insert_string $t [exec cat $prompt_result]
    j:text:insert_string $t "\n"
  }
}

######################################################################
# delete the selection and copy it to CUTBUFFER
######################################################################

j:command:register jedit:cmd:cut {Cut}
proc jedit:cmd:cut { t args } {
  global CUTBUFFER

  jedit:save_checkpoint $t			;# save undo information

  set CUTBUFFER [$t get sel.first sel.last]
  j:text:delete $t sel.first sel.last
}

######################################################################
# copy the selection into CUTBUFFER
######################################################################

j:command:register jedit:cmd:copy {Copy}
proc jedit:cmd:copy { t args } {
  global CUTBUFFER

  set CUTBUFFER [$t get sel.first sel.last]
}

######################################################################
# insert CUTBUFFER
######################################################################

j:command:register jedit:cmd:paste {Paste}
proc jedit:cmd:paste { t args } {
  global CUTBUFFER
  
  set mode [jedit:get_mode $t]
  
  jedit:save_checkpoint $t			;# save undo information

  if {[info procs mode:$mode:pre_paste_hook] != {}} {
    mode:$mode:pre_paste_hook $t
  }

  j:text:insert_string $t $CUTBUFFER

  if {[info procs mode:$mode:post_paste_hook] != {}} {
    mode:$mode:post_paste_hook $t
  }
}

######################################################################
# mark the entire text as selected
######################################################################

j:command:register jedit:cmd:select_all {Select All}
proc jedit:cmd:select_all { t args } {
  $t tag add sel 1.0 end
}

######################################################################
# expand dynamic abbreviation before insert
######################################################################

j:command:register jedit:cmd:dabbrev {Expand Dynamic Abbreviation}
proc jedit:cmd:dabbrev { t args } {
  # THIS COULD BE SIMPLIFIED: do i need both match... and abbrev... vars?
  # PROBLEM: this depends on the Text widget's notion of words.
  # it would be nice to be able to expand, say, $tk_l to $tk_library.

  global ABBREV ABBREV_POS MATCH MATCH_POS

  $t mark set abbrevend insert
  $t mark set abbrevstart insert
  while {[$t compare abbrevstart != 1.0] &&
         [string match {[a-zA-Z0-9']} [$t get {abbrevstart - 1 char}]]} {
    $t mark set abbrevstart {abbrevstart -1char}
  }

  set ABBREV_POS [$t index abbrevstart]	;# for dabbrev_again

  set ABBREV [$t get abbrevstart insert]

  set context [$t get 0.0 abbrevstart]

  while {1} {
    set matchpos [string last $ABBREV $context]
  
    if {$matchpos == -1} {return 0}	;# not found

    $t mark set matchstart [$t index "0.0 +$matchpos chars"]
    if {[$t compare matchstart == {matchstart wordstart}]} {
      $t mark set matchend [$t index {matchstart wordend}]
      break				;# sort of an `until'
    }
    set context [$t get 0.0 matchstart]
  }

  set MATCH [$t get matchstart matchend]

  set MATCH_POS [$t index matchstart]

  j:text:replace $t abbrevstart abbrevend $MATCH
  return 1
}

######################################################################
# look up and expand static abbrev before insert
######################################################################

j:command:register jedit:cmd:sabbrev {Expand Static Abbreviation}
proc jedit:cmd:sabbrev { t args } {
  $t mark set abbrevend insert
  # following don't really need to be global (shared with dabbrev):
  global ABBREV ABBREV_POS ABBREVS

  $t mark set abbrevend insert
  $t mark set abbrevstart insert
  while {[$t compare abbrevstart != 1.0] &&
         [string match {[a-zA-Z0-9_']} [$t get {abbrevstart - 1 char}]]} {
    $t mark set abbrevstart {abbrevstart -1char}
  }

  # avoid expanding things like \def, .PP, file.c, etc.:
  set prefix [$t get {abbrevstart -2chars} {abbrevstart}]
  if {[string length $prefix] > 0} {
    if {[string match {?[@$%&+=\:~.]} $prefix]} {
      return 0
    }
    # don't expand "l" in "ls -l", but do expand "this---l"
    if {[string match "\[ \t\n\]-" $prefix]} {	;# don't expand "ls -l"
      return 0
    }
    # don't expand "s" in "house(s)", but do expand "so (s) of"
    if {[string match "\[a-zA-Z](" $prefix]} {	;# don't expand "house(s)"
      return 0
    }
  }

  set ABBREV_POS [$t index abbrevstart]	;# for dabbrev_again

  # first try regular version:
  set ABBREV [$t get abbrevstart insert]
  if {[info exists ABBREVS($ABBREV)]} {
    j:text:replace $t $ABBREV_POS abbrevend $ABBREVS($ABBREV)
    return 1
  }
  # else try capitalised version
  if {[string match {[A-Z][a-z]*} $ABBREV]} {
    set lcabbrev [jedit:uncapitalise $ABBREV]
    if {[info exists ABBREVS($lcabbrev)]} {
      j:text:replace $t $ABBREV_POS abbrevend \
        [jedit:capitalise $ABBREVS($lcabbrev)]
      return 1
    }
  }
  return 0
}

######################################################################
# edit your abbrevs file
######################################################################

j:command:register jedit:cmd:edit_abbrevs {Edit Static Abbreviations}
proc jedit:cmd:edit_abbrevs { args } {
  global HOME
  if {! [file isdirectory "$HOME/.tk"]} then {
    exec mkdir "$HOME/.tk"
    # above should have error-checking
  }
  exec jabbrevs "$HOME/.tk/abbrevs.tcl" &	;# doesn't currently use arg
}

######################################################################
# read abbrevs file
######################################################################

j:command:register jedit:cmd:read_abbrevs {Reread Static Abbreviations}
proc jedit:cmd:read_abbrevs { args } {
  j:source_config abbrevs.tcl
}

######################################################################
# toggle static abbrevs
######################################################################

proc jedit:cmd:toggle_sabbrev { t args } {
  global JEDIT_MODEPREFS
  
  set mode [jedit:get_mode $t]
  
  set JEDIT_MODEPREFS($mode,sabbrev) \
    [expr {! $JEDIT_MODEPREFS($mode,sabbrev)}]
}

######################################################################
# toggle dynamic abbrevs
######################################################################

proc jedit:cmd:toggle_dabbrev { t args } {
  global JEDIT_MODEPREFS
  
  set mode [jedit:get_mode $t]
  
  set JEDIT_MODEPREFS($mode,dabbrev) \
    [expr {! $JEDIT_MODEPREFS($mode,dabbrev)}]
}

######################################################################
# go to a particular line
######## NEED TO CHECK THAT AN INDEX WAS TYPED!
######################################################################

j:command:register jedit:cmd:go_to_line {Go to Line...}
proc jedit:cmd:go_to_line { t args } {
  set prompt_result [j:prompt -text JEprompt:go_to_line]
  if {$prompt_result != {}} then {
    jedit:go_to_line $t $prompt_result
  }
}

######################################################################
# display which line the cursor is on
######################################################################

j:command:register jedit:cmd:current_line {Show Current Position}
proc jedit:cmd:current_line { t args } {
  set insertindex [split [$t index insert] {.}]
  set line [lindex $insertindex 0]
  set column [lindex $insertindex 1]
  j:alert -title "Notice" \
    -text JEmsg:at_line...column...
}

######################################################################
# insert X selection
######################################################################

j:command:register jedit:cmd:xpaste {Insert X Selection}
proc jedit:cmd:xpaste { t args } {
  set mode [jedit:get_mode $t]
  
  jedit:save_checkpoint $t			;# save undo information
  
  if {[info procs mode:$mode:pre_xpaste_hook] != {}} {
    mode:$mode:pre_xpaste_hook $t
  }

  j:text:insert_string $t [j:selection_if_any]

  if {[info procs mode:$mode:post_xpaste_hook] != {}} {
    mode:$mode:post_xpaste_hook $t
  }
}

######################################################################
# front end for j:find to match jedit:cmd argument convention
######################################################################

j:command:register jedit:cmd:find {Find...}
proc jedit:cmd:find { t args } {
  jedit:save_checkpoint $t
  j:find $t
}

######################################################################
# find same string again (same kind of search)
######################################################################

j:command:register jedit:cmd:find_again {Find Again}
proc jedit:cmd:find_again { t args } {
  jedit:save_checkpoint $t
  j:find:again $t
}

######################################################################
# hacks for more-specific kinds of finds (for vi/emacs bindings)
### BOGUS!  jedit should not need to know about the internals of j:find!
######################################################################
proc jedit:cmd:find_forward { t args } {
  global j_find
  set j_find(backwards) 0
  j:find $t
}

proc jedit:cmd:find_backward { t args } {
  global j_find
  set j_find(backwards) 1
  j:find $t
}

######################################################################
# save all windows and quit
######################################################################

# we need to make sure there's a filename before calling save, because
# a cancel in the saveas file selector box will cancel the save, but
# not the close!

j:command:register jedit:cmd:done {Done}
proc jedit:cmd:done { t args } {
  set mode [jedit:get_mode $t]
  
  if {[info procs mode:$mode:done] == {}} {
    set filename [jedit:get_filename $t]
    if {"x$filename" == "x"} then {
      set filename [j:fs -prompt JEprompt:saveas]
      if {"x$filename" == "x"} {			;# user clicked cancel
        return
      } else {
        jedit:set_filename $t $filename
      }
    }

    jedit:cmd:save $t
    jedit:cmd:close $t
  } else {
    mode:$mode:done $t
  }
}

######################################################################
# commands defined elsewhere:

j:command:register jedit:cmd:save_checkpoint {Checkpoint}
j:command:register jedit:cmd:undo {Undo}
j:command:register jedit:cmd:redo {Redo}

######################################################################
# dummy to force autoloading of this file (for registration of cmds):

proc jedit:register_commands {} {}

