/*********************************************************************
* LoLA (Library of Location Algorithms)                              *
*                                                                    *
* CLASS FACILITIES                                                   *
**********************************************************************/

#ifndef LOLA_FACILITIES_H
#define LOLA_FACILITIES_H

#include <LOLA/polygauge.h>
#include <fstream.h>


/*{\Manpage {facilities} {} {Facilities}}*/

class facilities : virtual public list<location>
/*{\Mdefinition An instance of the data type $facilities$ consists of a list of
locations.}*/
{
 protected:
  list<location>  solution;
  list<string>    LocTxt;
  facilities      collect(int,int);
  int             weiszfeld_test(location&);
  int             weiszfeld_test(location&,int);
  int             generalized_weiszfeld_test(list_item,int,int);
  location        generalized_weiszfeld_iteration(location,int,double,int);

  string&          refname(list_item location);
  string&          refname(int location);
  
 public: 
               
  inline int      _d() { return head().dim(); }
  inline int      _Q() { return head().wht_dim(); }

/*{\Mcreation EX }*/
/*{\Mtext {\it facilities EX}; \hspace{1cm}
introduces a variable $EX$ of type $facilities$.}*/
/*{\Moperations 2.6 4.9 }*/
/*{\Mtext
\medskip
{\bf 3.1 Input and Output}
\medskip }*/
  list<location>  alg_solution() {return solution;}
  list<string>	  loc_text() {return LocTxt;}
  Loc_Vector      total_weight();
  string name(list_item location);
  string name(int location);
/*{\Mop provides a name for $location$.}*/
  void set_name(list_item location, const string &name);
  void set_name(int location, const string &name);
/*{\Mopl specifies a  name for $location$.}*/
  void            Write();
  void            Write(ostream &out);
/*{\Mop prints all facilities.}*/
  double          median_objective(int p,int weight);
  double          n_median_objective(int p,int weight);
  double		  median_objective_comp(int p,location optimum,int weight);
  void            WriteOpt();
  void            WriteOpt(ostream &out);
/*{\Mop prints the set of optimal solutions.}*/
/*{\Mtext
\medskip
{\bf 3.2 Operations}
\medskip }*/
  bool            element(location &Loc);
/*{\Mop returns true if $Loc$ is an element of $EX$, else false.}*/
  location        get_element(int idx);
/*{\Mop returns the $idx$-element of the $EX$.}*/
  int             get_idx(location &L);
/*{\Mop returns the index of $L$ otherwise the total number of locations.}*/
  void            remove(location &L);
/*{\Mop removes $L$ from $EX$ if possible.}*/
  void            remove(int idx);
/*{\Mop removes the $idx$-element of $EX$ if possible.}*/
  double          max_direction_value(int i);
/*{\Mop returns the maximum value of coordinate $i$ of all locations of $EX$.}*/
  double          min_direction_value(int i);
/*{\Mop returns the minimum value of coordinate $i$ of all locations of $EX$.}*/
  double          diff_value(int i);
/*{\Mop returns the maximal difference between coordinates $i$ of any two
 locations of $EX$.}*/
/*{\Mtext
\medskip
{\bf 3.3 Algorithms}
\medskip }*/
/*{\Mtext
\medskip
{\bf 3.3.1 Median-problems}
\medskip }*/

  double          l1_sum(int weight,bool erase);
/*{\Mopl computes the optimum for problem class $1/P/./l_1/\sum$
and returns the objective value.}*/
  double          l2sqr_sum(int weight,bool erase);
/*{\Mopl computes the optimum for problem class $1/P/./l_2^2/\sum$ and returns the objective value.}*/
  double          linf_sum(int weight,bool erase);
/*{\Mopl computes the optimum for problem class $1/P/./l_{\infty}/\sum$
and returns the objective value.}*/
  double          l2_sum(int weight,double epsilon,bool erase);
/*{\Mopl computes the optimum for problem class $1/P/./l_2/\sum$ (Weiszfeld-Algorithm) and returns the objective value.}*/
  double          Weiszfeld(double epsilon);
/*{\Mop calls l2\_sum(0, epsilon, true).}*/
  double          lp_sum(int p,double epsilon,double delta,int iter_max,int weight,bool erase);
/*{\Mopl computes the optimum for problem class $1/P/./l_p/\sum$ 
(Generalized Weiszfeld-Algorithm) and returns the objective value.}*/
  double          l2sqr_qsum(bool erase);
/*{\Mopl computes the optimum for problem class $1/P/./l_2^2/Q-\sum$
(Q-criterial problem) and returns the objective value.}*/

  double          l1_2sum(bool erase);
/*{\Mopl computes the optimum for problem class $1/P/./l_1/2-\sum$
(bi-criterial problem) and returns the objective value.}*/

  double          linf_2sum(bool erase);
/*{\Mopl computes the optimum for problem class $1/P/./l_{\infty}/2-\sum$
(bi-criterial problem) and returns the objective value.}*/

  double          N_l1_sum(int N,matrix& w,bool erase);
/*{\Mopl computes the optimum for problem class $N/P/./l_1/\sum$
with matrix of interdependencies $w$ and returns the objective value.}*/

  double          N_linf_sum(int N,matrix& w,bool erase);
/*{\Mopl computes the optimum for problem class $N/P/./l_\infty/\sum$
with matrix of interdependencies $w$ and returns the objective value.}*/

  double          N_l2sqr_sum(int N,matrix& w,bool erase);
/*{\Mopl computes the optimum for problem class $N/P/./l_2^2/\sum$
with matrix of interdependencies $w$ and returns the objective value.}*/

  double*         N_lp_sum_v1(int N,int p,matrix& w,bool erase);  
/*{\Mopl computes the optimum for problem class $N/P/./l_p/\sum$ (Version 1) and returns the objective value. }*/
/*{%\Mtext
%\medskip
%{\it double*} \hspace{0.3cm} EX.N\_lp\_sum\_v1({\it int n,int p,matrix w});
%\medskip }*/
/*{%\Mtext
%\begin{tabular}{p{6.5cm}l}
%& computes the optimum for problem class \\
%& $N/P/./l_p/\sum$ (Version 1) and returns the objective value.
%\end{tabular}
%\medskip }*/

  double*	  N_lp_sum_v2(int N,int p,matrix& w,double epsilon,bool erase);
/*{\Mopl computes the optimum for problem class $N/P/./l_p/\sum$ (Version 2) and
 returns the objective value. }*/
/*{%\Mtext
%\medskip
%{\it double*} \hspace{0.3cm} EX.N\_lp\_sum\_v2({\it int n,int p,matrix w});
%\medskip }*/
/*{%\Mtext
%\begin{tabular}{p{6.5cm}l}
%& computes the optimum for problem class \\
%& $N/P/./l_p/\sum$ (Version 2) and returns the objective value.
%\end{tabular}
%\medskip }*/
  double          fn_sum(int n,int p);
/*{\Mop computes objective value of $X_n$ regarding all locations of $EX$.}*/
  double          fex_sum(int p);
/*{\Mop computes objective value of all solutions regarding all facilities.}*/
  double          fnew_sum(matrix& w,int p);
/*{\Mopl computes objective value of $X_n$'s to each other.}*/
  double	  S(int N,int n,matrix& w);
  void            N_l1_sum_gen_LP(int N,matrix& w,char* lp1,char* lp2); 
/*{\Mopl generates LP for problem class $N/P/./l_1/\sum$.}*/
  void            write_mps(vector& OBJ,matrix& CONSTRAINTS,vector& RHS,list<char>& EQU,char* filename);
/*{\Mopl writes down a given objective function vector $OBJ$, matrix of constraints $CONSTRAINTS$, right hand side vector $RHS$, (in)equality sign list $EQU$ to file $filename$ in $mps$--format. For each row of $CONSTRAINTS$ there must be a corresponding entry in $EQU$ of the set E(for equal to), L(for less than or equal to), G(for greater than or equal to), and N(for neutral). Remember that the objective row must not be mentioned in $EQU$. $EQU$ is only for use with the $CONSTRAINT$--rows.}*/
  double          read_sol_mps(char* filename1,char* filename2,int N);
/*\Mopl reads the solution of a CPLEX solved mps format problem from file $filename$, returns the objective value and collects the the solutions into the $solution$--structure of class $facilities$.}*/
  location        A_plus(double &z);
  location        A_minus(double &z); 


  double          gauge_sum(list<polygauge>& LG, list<int>& Ln, bool erase);
/*{\Mopl computes the optimum for problem class $1/P/./\gamma/\sum$.} */
  double          gauge_bi_crit_sum(list<polygauge>& LG, list<int>& Ln, bool erase);
/*{\Mopl computes the optimum for problem class $1/P/./\gamma/2-\sum_{par}$ and returns the number of situations (line segments and cells) the pareto set consists of.} */



/*{\Mtext
\medskip
{\bf 3.3.2 Center-problems}
\medskip }*/
  double          linf_max(bool erase=true);
/*{\Mopl computes the optimum for problem class $1/P/./l_{\infty}/max$
and returns the objective value.}*/
  double          l1_max(bool erase=true);
/*{\Mopl computes the optimum for problem class $1/P/./l_1/max$
and returns the objective value.}*/
  double          l1_v1_max(bool erase=true);  
/*{\Mopl computes the optimum for problem class $1/P/w_m=1/l_1/max$ and
returns the objective value.}*/
  double          l2_max(bool erase=true);
/*{\Mopl computes the optimum for problem class $1/P/./l_2/max$
and returns the objective value.}*/
  double          linf_v1_max(bool erase=true); 
/*{\Mopl computes the optimum for problem class $1/P/./l_{\infty}/max$ and
returns the objective value.}*/
  double          elz_hearn(bool erase=true);
/*{\Mopl computes the optimum for Elzinga-Hearn-Algorithm
and returns the objective value.}*/
  double          N_linf_max(int N, matrix& W, bool erase = true);
/*{\Mopl computes the optimum for problem class $N/P/./l_{\infty}/max$
and returns the objective value.}*/
  double          N_l1_max(int N, matrix& W, bool erase = true);
/*{\Mopl computes the optimum for problem class $N/P/./l_1/max$
and returns the objective value.}*/
  double          N_linf_max_mps(int N, matrix& W, bool erase = true);
/*{\Mopl computes the optimum for problem class $N/P/./l_{\infty}/max$
using $mps$-files and CPLEX and returns the objective value. CPLEX generates also two
files named "sol1.txt" and "sol2.txt".}*/
  double          N_l1_max_mps(int N, matrix& W, bool erase = true);
/*{\Mopl computes the optimum for problem class $N/P/./l_1/max$
using $mps$-files and CPLEX and returns the objective value. CPLEX generates also two
files named "sol1.txt" and "sol2.txt".}*/



/*{\Mtext
\medskip
{\bf 3.4 Useful Functions}
\medskip }*/

  double          objective_value_computation(list<polygauge>& Lg,list<int>& Ln,point X, int q);
/*{\Mopl computes the objective value for a gauge problem.}*/
  vector          subgradient_computation (list<polygauge>& Lg,list<int>& Ln,point X,int q);
/*{\Mopl computes the subgradient value for a gauge problem.}*/
  list<segment>   all_segments_computation(list<polygauge>& Lg,list<int>& Ln,point Y,double f_X,int q);
/*{\Mopl computes all segments for all locations for 1-criterial.}*/
  list<point>   choice_of_optimal_points_1 (list<polygauge>& Lg,list<int>& Ln,list<point>& Liste_der_Gitterpunkte,point X,double f_X,int q);
/*{\Mopl computes the function values for the gridpoints.}*/
  vector niveauline_vector_computation(list<polygauge>& Lg,list<int>& Ln,point X,int q);
/*{\Mopl computes the direction vector for level curve.}*/
  list<point> computation_of_lex_solutions(list<polygauge>& Lg,list<int>& Ln,point X,int q1, int q2, GRAPH<point,segment>&  planarer_Graph);
/*{\Mopl computes the lexicographical solution.}*/
  list<location> non_trivial_case(list<polygauge>& Lg,list<int>& Ln,list<point>& Lex_sol_12, list<point>& Lex_sol_21, GRAPH<point,segment>& planarer_Graph);
/*{\Mopl computes the pareto solution for a non trivial case.}*/

  double gauge_max(list<polygauge>& LG, list<int>& Ln,bool erase);
  double opt1(double& z, list<int>& Ln, list<polygauge>& Lg, int & hilf3,list<point>& OPTLIST); 
  polygon opt11(double& z, list<int>& Ln, list<polygauge>& Lg); 
  void doppel(list<point>& OPTLIST);
  int test1(double& ztest, list<int>& Ln, list<polygauge>& Lg, double& zoben);
  int test1(double& ztest, list<int>& Ln, list<polygauge>& Lg)
  {
    double zoben=1.5e8;
    int i = test1(ztest,Ln,Lg,zoben); return i;
  } 
  double intervallschacht(list<double>& niveauliste,list<int>& Ln, list<polygauge>& Lg, double zmax, double zoben);
  list<polygon> polylist(double& z, list<int>& Ln, list<polygauge>& Lg);
};


// ------------------------- global functions -----------------


line mid_perpend(location &l1, location &l2);
int _COMPARE(const location& generic_pointer1, const location& generic_pointer2);
void _TRANSFORM_LOCATIONS(location& generic_pointer);
void _RE_TRANSFORM_LOCATIONS(location& generic_pointer);
void trans_all(list<location> &facs);
void retrans_all(list<location> &facs);
void trans_all(facilities &facs);
void retrans_all(facilities &facs);
double gauge_median(list<double>&);
polygon poly_intersection(const polygon&, polygon&);
void sort_points(list<point>&);

#endif





