/*********************************************************************
* LoLA (Library of Location Algorithms)                              *
*                                                                    *
* CLASS LGRAPHALG 
**********************************************************************/

#ifndef LOLA_GRAPHALG_H
#define LOLA_GRAPHALG_H

#include <LOLA/graph_restriction.h>
/*{\Manpage {lgraphalg} {} {Algorithm}}*/

class lgraphalg 
/*{\Mdefinition An instance of the data type $lgraphalg$ consists of the algorithms to solve location problems with a network or graph.}*/
{
 protected: 
 list<sol_typ> solution;

 public: 
/*{\Mcreation GrA }*/
/*{\Mtext {\it lgraphalg} GrA; \hspace{1cm}
creates a variable $GrA$ of type $lgraphalg$.}*/
/*{\Moperations 2.7 4.7 }*/

  list<sol_typ>        alg_solution() {return solution;}
/*{\Mop returns the solution for the current problem.}*/

  void            WriteOpt();
  void            WriteOpt(ostream& out);
/*{\Mop writes the solution for the current problem.}*/

/*{\Mtext
\medskip
{\bf 3.1 Algorithms}
\medskip }*/

  double          inmedian(loladirected& G); 
/*{\Mopl computes the In-Median for problem class $1/G_D/./d(V,V)/\sum$
and returns the objective value.}*/

  double          outmedian(loladirected& G); 
/*{\Mopl computes the Out-Median for problem class $1/G_D/./d(V,V)/\sum$
and returns the objective value.}*/

  double          median(loladirected& G); 
/*{\Mopl computes the median for problem class $1/G_D/./d(V,V)/\sum$
and returns the objective value.}*/

  double          median(lolaundirected& U); 
/*{\Mopl computes the optimum for problem class $1/G/./d(V,V)/\sum$
and returns the objective value.}*/

  double          abs_inmedian(loladirected& G); 
/*{\Mopl computes the In-Median for problem class $1/G_D/./d(V,G)/\sum$
and returns the objective value.}*/

  double          abs_outmedian(loladirected& G); 
/*{\Mopl computes the Out-Median for problem class $1/G_D/./d(V,G)/\sum$
and returns the objective value.}*/

  double          abs_median(loladirected& G); 
/*{\Mopl computes the median for problem class $1/G_D/./d(V,G)/\sum$
and returns the objective value.}*/

  double          abs_median(lolaundirected& U); 
/*{\Mopl computes the optimum for problem class $1/G/./d(V,G)/\sum$
and returns the objective value.}*/

  double          center(lolaundirected& G);
/*{\Mopl computes the optimum for problem class $1/G/./d(V,V)/\max$
and returns the objective value.}*/

  double          center(loladirected& G);
/*{\Mopl computes the optimum for problem class $1/G_D/./d(V,V)/\max$
and returns the objective value.}*/

  double          abs_center(lolaundirected& U);
/*{\Mopl computes the optimum for problem class $1/G/./d(V,G)/\max$
and returns the objective value.}*/

  double          abs_center(loladirected& G);
/*{\Mopl computes the optimum for problem class $1/G_D/./d(V,G)/\max$
and returns the objective value.}*/

  double          loc_tree_median(lolatree& U);
/*{\Mopl computes the optimum for problem class $1/T/./d(V,V)/\sum$
and returns the objective value.}*/

  double          abs_tree_median(lolatree& U);
/*{\Mopl computes the optimum for problem class $1/T/./d(V,T)/\sum$
and returns the objective value.}*/

  double          loc_tree_center(lolatree& U);
/*{\Mopl computes the optimum for problem class $1/T/./d(V,V)/\max$
and returns the objective value.}*/

  double          abs_tree_center(lolatree& U);
/*{\Mopl computes the optimum for problem class $1/T/./d(V,T)/\max$
and returns the objective value.}*/

  double          tree_two_center(lolatree& U);
/*{\Mopl computes the optimum for problem class $2/T/./d(V,T)/\max$
and returns the objective value.}*/

/*  double          median(loladirected& G, graph_restriction& EL);

  double          median(lolaundirected& G, graph_restriction& EL);


  double          abs_median(lolaundirected& U, graph_restriction& EL);*/

  double     N_median_cplex(lolaundirected& U,int n,list<int>& belongto);
/*{\Mopl computes the optimum for problem class $N/G/./d(V,V)/\sum$
using cplex and returns the objective value.}*/

  double     N_median_partitioning(lolaundirected& U,int n,list<int>& belongto);
/*{\Mopl computes the optimum for problem class $N/G/./d(V,V)/\sum$
using the node-partitioning-heuristic and returns the objective value, in the list 'belongto' the membership-information for every node is contained .}*/

  double      N_median_exchange(lolaundirected& U,int n,list<int>& belongto);
/*{\Mopl computes the optimum for problem class $N/G/./d(V,V)/\sum$
using the exchange-heuristic and returns the objective value, in the list 'belongto' the membership-information for every node is contained .}*/

  double       N_median_greedy(lolaundirected& U,int n,list<int>& belongto);
/*{\Mopl computes the optimum for problem class $N/G/./d(V,V)/\sum$
using the greedy-heuristic and returns the objective value, in the list 'belongto' the membership-information for every node is contained .}*/

  double       N_center_partitioning(lolaundirected& U,int n,list<int>& belongto);
/*{\Mopl computes the optimum for problem class $N/G/./d(V,V)/\max$
using the node-partitioning-heuristic and returns the objective value, in the list 'belongto' the membership-information for every node is contained .}*/

  double        N_center_greedy(lolaundirected& U,int n,list<int>& belongto);
/*{\Mopl computes the optimum for problem class $N/G/./d(V,V)/\max$
using the greedy-heuristic and returns the objective value, in the list 'belongto' the membership-information for every node is contained .}*/

  list<vector> medPareto_bi(lolaundirected& U,list<edge_segment>& wholesol);
/*{\Mopl computes the optimum for problem class $1/G/./d(V,G)/2-\sum\nolimits_{par}$
}*/

  list<vector> lgraphalg::medPareto_Q(lolaundirected& U,list<edge_segment>& wholesol);
/*{\Mopl computes the optimum for problem class $1/G/./d(V,G)/Q-\sum\nolimits_{par}$
}*/

  list<vector> lgraphalg::medPareto(loladirected& G, list<edge_segment>& wholesol);
/*{\Mopl computes the optimum for problem class $1/G_D/./d(V,G)/Q-\sum\nolimits_{par}$
}*/

  list<vector> lgraphalg::loc_medPareto(loladirected& G, list<edge_segment>& wholesol);
/*{\Mopl computes the optimum for problem class $1/G_D/./d(V,V)/Q-\sum\nolimits_{par}$,
}*/

  list<vector> lgraphalg::loc_medPareto(lolaundirected& U, list<edge_segment>& wholesol);
/*{\Mopl computes the optimum for problem class $1/G/./d(V,V)/Q-\sum\nolimits_{par}$,
}*/

  list<vector> lgraphalg::loc_cenPareto(loladirected& G, list<edge_segment>& wholesol);
/*{\Mopl computes the optimum for problem class $1/G_D/./d(V,V)/Q-\max_{par}$,
}*/

  list<vector> lgraphalg::loc_cenPareto(lolaundirected& U, list<edge_segment>& wholesol);
/*{\Mopl computes the optimum for problem class $1/G/./d(V,V)/Q-\max_{par}$,
}*/

  list<vector> lgraphalg::cenPareto(loladirected& G, list<edge_segment>& wholesol);
/*{\Mopl computes the optimum for problem class $1/G_D/./d(V,G)/Q-\max_{par}$
}*/

  list<vector> lgraphalg::medLexi(lolaundirected& U, list<edge_segment>& wholesol);
/*{\Mopl computes the optimum for problem class $1/G/./d(V,G)/Q-\sum\nolimits_{lex}$
}*/

  list<vector> lgraphalg::medLexi(loladirected& G, list<edge_segment>& wholesol);
/*{\Mopl computes the optimum for problem class $1/G_D/./d(V,G)/Q-\sum\nolimits_{lex}$
}*/

  list<vector> lgraphalg::loc_medLexi(loladirected& G, list<edge_segment>& wholesol);
/*{\Mopl computes the optimum for problem class $1/G_D/./d(V,V)/Q-\sum\nolimits_{lex}$,
}*/

  list<vector> lgraphalg::loc_medLexi(lolaundirected& U, list<edge_segment>& wholesol);
/*{\Mopl computes the optimum for problem class $1/G/./d(V,V)/Q-\sum\nolimits_{lex}$,
}*/

  list<vector> lgraphalg::loc_cenLexi(loladirected& G, list<edge_segment>& wholesol);
/*{\Mopl computes the optimum for problem class $1/G_D/./d(V,V)/Q-\max_{lex}$,
}*/

  list<vector> lgraphalg::loc_cenLexi(lolaundirected& U, list<edge_segment>& wholesol);
/*{\Mopl computes the optimum for problem class $1/G/./d(V,V)/Q-\max_{lex}$,
}*/

};
#endif


