/**********************************************************************
* LoLA (Library of Location Algorithms)                               *
*                                                                     *
* CLASS LINE                                                          *
**********************************************************************/

#ifndef LOLA_LINE_H_
#define LOLA_LINE_H_

#include <LOLA/location.h>

/*{\Manpage {Line} {} {Line}}*/

class Line {
/*{\Mdefinition An instance of the data type $Line$ consists of three doubles and the line-defining locations; $(ax + by - c = 0)$.}*/
	
 private:
   double M;     // slope
   location l1;  // the defining locations
   location l2;
   double A;
   double B;     // ax + by - c = 0, a = 1
   double C;
   
 public:

/*{\Mcreation V }*/
	
   Line();
/*{\Mcreate creates a zero-$Line$.}*/

   Line(location l1, location l2);
/*{\Mcreate creates a $line$ defined by $l1$ and $l2$.}*/
	
   Line(const Line &L);
   ~Line();

/*{\Moperations 2.3 4.6 }*/
   Line& operator = (const Line &L);
/*{\Mbinop assignment operator.}*/

   double& m() { return M; }
/*{\Mop returns a reference to the slope $M$ of $V$.}*/
	
   location& L1() { return l1; }
/*{\Mop returns a reference to the first defining location $L1$.}*/
	
   location& L2() { return l2; }
/*{\Mop returns a reference to the first defining location $L2$.}*/
	
   double& a() { return A; }
/*{\Mop returns a reference to the $x$-coefficient of $V$.}*/
	
   double& b() { return B; }
/*{\Mop returns a reference to the $y$-coefficient of $V$.}*/
	
   double& c() { return C; }
/*{\Mop returns a reference to the constant $c$.}*/

   bool parallel(Line &L);
/*{\Mop returns true if the lines $V$ and $L$ are parallel; otherwise false.}*/
	
   double weighted_distance(location &L);
/*{\Mopl calculates the weighted euclidean distance from $L$ to the line $V$.}*/

   double distance(location &L);
/*{\Mop calculates the unweighted euclidean distance from $L$ to the line $V$.}*/
	
   double lp_distance(location &L, int p);
/*{\Mopl calculates the unweighted $l_p$-distance from $L$ to the line $V$.}*/

   location intersection(Line &L);
/*{\Mop calculates the intersection-location of the two lines.}*/
   
   bool operator < (Line &L);
/*{\Mbinop returns true if L is parallel to $V$ and lies below $V$.}*/
   bool operator > (Line &L);
/*{\Mbinop returns true if $L$ is parallel to $V$ and lies above $V$.}*/
   bool operator == (Line &L);
/*{\Mbinop returns true if the lines are identical.}*/

friend ostream& operator << (ostream &out, Line &L); 
};

#endif
