#ifndef _Loc_Vector_h
#define _Loc_Vector_h

#include <LOLA/lola_error.h>
#include <LOLA/globals.h>


/*{\Manpage {Loc_Vector} {} {Real-Valued Location-Vectors}}*/

class Loc_Vector {
/*{\Mdefinition An instance of the data type $Loc\_Vector$ consists of an
 $n$-dimensional array of reals.}*/

 protected:
   double *data;
   int v_size;
   
 public:

/*{\Mcreation V }*/
   Loc_Vector(int dim=0);
/*{\Mcreate creates a $dim$-dimensional $Loc\_Vector$, default dimension is 0.}*/
   Loc_Vector(double dim,...);
/*{\Mcreate creates an instance $Loc\_Vector$ with the following initialization: $v(dim,arg_1,..,arg_{dim}.)$. All numbers must be doubles.}*/
   Loc_Vector(const Loc_Vector &v);
   ~Loc_Vector() { delete[] data; }
/*{\Moperations 2.3 4.6}*/
   Loc_Vector operator+ (const Loc_Vector &v1);
/*{\Mbinop returns the sum of $V$ and $v1$.}*/
   Loc_Vector operator- (const Loc_Vector &v1);
/*{\Mbinop returns the result of the subtraction of  $v1$ from $V$.}*/
   Loc_Vector& operator+= (const Loc_Vector &v1);
/*{\Mbinop adds $v1$ to $V$.}*/
   Loc_Vector& operator-= (const Loc_Vector &v1);
/*{\Mbinop subtracts $v1$ from $V$.}*/
   Loc_Vector& operator *= (double scalar);
/*{\Mbinop multiplies $V$ by $scalar$.}*/
   Loc_Vector& operator += (double scalar);
/*{\Mbinop adds $scalar$ to $V$.}*/
   Loc_Vector& operator -= (double scalar);
/*{\Mbinop subtracts $scalar$ from $V$.}*/
   Loc_Vector& operator /= (double scalar);
/*{\Mbinop divides $V$ by $scalar$.}*/

   Loc_Vector operator * (double scalar);

/*{\Mbinop returns the product of $V$ and $scalar$.}*/
 Loc_Vector operator + (double scalar);
/*{\Mbinop returns the sum of $V$ and $scalar$.}*/
   Loc_Vector operator - (double scalar);
/*{\Mbinop returns the result of the subtraction of $scalar$ from $V$.}*/
   Loc_Vector operator / (double scalar);
/*{\Mbinop returns the result of the division of $V$ by $scalar$.}*/
   double operator * (const Loc_Vector &v);
/*{\Mtext
\medskip
\begin{tabular}{lll}{p{1.8cm}p{4.54cm}l}
{\it double} & $V * V_2$ & inner product of $V$ with $V_2$.
\end{tabular}
\medskip }*/
   double& operator [] (int idx) { return data[idx]; }
/*{\Marrop returns a reference to the $idx$-th component of $V$.}*/
   double operator [] (int idx) const { return data[idx]; }
   bool operator < (const Loc_Vector &v1);
/*{\Mbinop returns true if $V < v1$ for all elements.}*/
   bool operator <= (const Loc_Vector &v1);
/*{\Mbinop returns true if $V \leq v1$ for all elements.}*/
   bool operator > (const Loc_Vector &v1);
/*{\Mbinop returns true if $V > v1$ for all elements.}*/
   bool operator >= (const Loc_Vector &v1);
/*{\Mbinop returns true if $V \geq v1$ for all elements.}*/
   bool operator != (const Loc_Vector &v1);
/*{\Mbinop returns true if $V \neq v1$ for all elements.}*/
   bool operator == (const Loc_Vector &v1);
/*{\Mbinop returns true if $V = v1$ for all elements.}*/
   Loc_Vector& operator = (const Loc_Vector &v1);
/*{\Mtext
\medskip
\begin{tabular}{lll}{p{1.8cm}p{4.54cm}l}
{\it Loc\_Vector}\& & $V = V_2$ & assignment operator.
\end{tabular}
\medskip }*/
   double norm(int p) const;
/*{\Mop returns the $p$-norm ($p$-metric) of $V$.}*/
   void abs();
/*{\Mop returns the absolute value of $V$.}*/
   int size() const { return v_size; }
/*{\Mop returns the dimension of $V$.}*/
   bool null_chk();
/*{\Mop returns true if one or more components of $V$ are zero.}*/
   Loc_Vector rotate(double phi);
/*{\Mop rotates $V$ by $phi$.}*/
friend ostream& operator << (ostream &stream, const Loc_Vector &v);
/*{\Mtext
\begin{tabular}{lll}{p{1.8cm}l}
{\it ostream}\& & $<<$ (ostream \&stream, Loc\_Vector \&v)
\end{tabular}
\begin{tabular}{p{1.6cm}l}
& writes $V$ componentwise.
\end{tabular}
\medskip }*/

};

/*{\Mimplementation
All operations on a $Loc\_Vector$ take time $O(size())$.}*/

#endif
















