#ifndef LOLA_PAR_PLANAR_LOCATION_H
#define LOLA_PAR_PLANAR_LOCATION_H

#include <LOLA/loc_vector.h>
#include <math.h>


/*{\Manpage {location} {} {Real-Valued Locations with weights}}*/

class location {
/*{\Mdefinition An instance of the data type $location$ consists of two Loc\_Vectors.}*/
 protected:
   Loc_Vector coord;
   Loc_Vector whts;
   
   void test_coords(const location &L) const;
   void test_weights(const location &L) const;

 public:
/*{\Mcreation L }*/ 
   location(int dim=2, int wht=1);
/*{\Mcreate creates a location with a $dim$-dimensional $Loc\_Vector$ for the 
coordinates and a $wht$-dimensional $Loc\_Vector$ for the weights.}*/
   location(double dim,...);
/*{\Mcreate creates an instance $Location$ with the following initialization: $v(dimension,\#weights,coord1,coord2,...,wht1,wht2,...)$. \\ All numbers must be doubles.}*/
   location(const location &L);
   ~location();
   
/*{\Moperations 2.2 4.6 }*/
   double norm(int p) const;
/*{\Mop returns the $p$-norm ($p$-metric) of $L$.}*/
   int dim() const { return coord.size(); }
/*{\Mop returns the dimension of $L$.}*/
   int wht_dim() const { return whts.size(); }
/*{\Mop returns the number of weights of $L$.}*/
   int overall() const { return dim()+wht_dim(); }
/*{\Mop returns $dim() + wht\_dim()$ of $L$.}*/

   Loc_Vector& pos() { return coord; }
/*{\Mop returns a reference to the coordinate $Loc\_Vector$.}*/
   Loc_Vector pos() const { return coord; } 
   Loc_Vector& wht() { return whts; }
/*{\Mop returns a reference to the weights $Loc\_Vector$.}*/
   Loc_Vector wht() const { return whts; }

   void transform();
/*{\Mop transforms the coordinates. \\ Precondition: $L.dim()=2$.}*/
   void retransform();
/*{\Mop retransforms the coordinates. \\ Precondition: $L.dim()=2$.}*/
   double r_angle(const location &L1) const;
/*{\Mop calculates the radian angle between $L$ and $L1$.}*/
   double d_angle(const location &L1) const;
/*{\Mop calculates the degree angle between $L$ and $L1$.}*/
   point loc2point() const;
/*{\Mop transforms $location$ $L$ into type $point$. 
\\$Precondition:$ $dim()=2$.}*/
   vector loc2vector() const;
/*{\Mop transforms $location$ $L$ into type $vector$ (not $Loc\_Vector$). \\$Precondition:$ $dim()>0$.}*/
   segment makesegment(const location& L1) const;
/*{\Mop returns a $segment$ built of $location$ $L$ and $L1$. \\$Precondition:$ $L.dim()=2$.}*/
   
   double& operator [] (int i);
/*{\Marrop returns a reference to the $i$-th coordinate of $L$.}*/
   double operator [] (int i) const;
   double& operator () (int j);
/*{\Mfunop returns a reference to the $j$-th weight of $L$.}*/
   double operator() (int j) const;
   double operator*(const location &L1) const;
/*{\Mbinop returns product of $L$ and $L1$.}*/
   location operator*(const double &scalar) const;
/*{\Mbinop returns product of $L$ with $scalar$.}*/
   location operator/(const double &scalar) const;
/*{\Mbinop returns the result of the division of $L$ by $scalar$.}*/
   location operator+(const location &L1) const;
/*{\Mbinop returns $location$ which coordinates are the sum of of the 
$L1$-coordinates and the $L$-coordinates.}*/
   location operator-(const location &L1) const;
/*{\Mbinop returns $location$ which coordinates are the difference of of the 
$L1$-coordinates and the $L$-coordinates. }*/
friend location operator-(const location &L1);
   int operator==(const location &L1) const;
/*{\Mbinop tests for equality in coordinates $and$ weights of $L$ and $L1$ (true = 1, false = 0).}*/
   int operator!=(const location &L1) const;
/*{\Mbinop tests for inequality in coordinates $and$ weights of $L$ and $L1$ (true = 1, false = 0).}*/
   location& operator=(const location &L1); 
/*{\Mop assignment operator.}*/
   bool operator < (location &L1);
   bool operator > (location &L2);



static void WriteAsPolygonList (ostream &out, const list<location>&L);
  // writes a list of locations as a list of polygons (such as facilities.solution)

friend ostream& operator<<(ostream &out, const location &L);
friend istream& operator>>(istream& In, location& L);
};


// ----------------- global functions -----------------------------

location point2loc (const point &p);


#endif







