/***************************************************************************
* class mixgauge for LOLA (Library of Location Algorithms)
* Purpose: 
* Author : Lothar Schnell
* Date   : Dez-11-1995
***************************************************************************/
#ifndef LOLA_MGAUGE_H
#define LOLA_MGAUGE_H

#include <LEDA/plane_alg.h>
#include <LEDA/line.h>
#include <LEDA/list.h>
#include <LEDA/matrix.h>
#include <LOLA/location.h>
//#include <LOLA/lola_error.h>

/*{\Manpage {mixgauge} {} {Mixed Gauges} }*/

class mixgauge {
/*{\Mdefinition
An instance $G$ of the data type $mixgauge$ is a gauge
in the two-dimensional plane defined by the sequence of its vertices
in counterclockwise order which creates the cones. These cones are numbered
in counterclockwise order. The number of vertices is called the size of $G$.
See also the Gauge-utilities header (gauge\_util.h), which provides
Load, Save, Create and View for Gauges.}*/
 
 protected:
   list<segment> seg_list;
   list<double> alpha_list;
   list<bool> typ_list;

 public:
   
/*{\Mcreation G }*/

   mixgauge(const list<point>& pl, const list<bool>& typ);
/*{\Mcreate introduces a variable \var\ of type \name. \var\ is initialized 
to the mixgauge with vertex sequence $pl$.\\
\precond The vertices in $pl$ are given in counterclockwise order and define a mixgauge. The polyhedral belonging to the mixgauge must be convex.}*/
   mixgauge(); 
/*{\Mcreate introduces a variable \var\ of type \name. \var\ is initialized to the empty mixgauge.}*/

   mixgauge(const mixgauge& G);
   ~mixgauge();

   mixgauge& operator = (const mixgauge& G);

/*{\Moperations 2.5 4.4}*/

   point operator [] (int c);
/*{\Marrop returns the $c$-th extremalpoint of \var\ .}*/


   bool conetype(const int& c);  
/*{\Mop returns the type of the cone $c$.}*/

   list<point> vertices(const point& r) const;  
/*{\Mop returns the vertex sequence of \var\ for the reference point $r$.}*/

   list<point> vertices(const location& rl) const;  
/*{\Mop returns the vertex sequence of \var\ for the reference location 
$rl$.}*/

   list<segment> segments(const point& r); 
/*{\Mop returns the sequence of bounding segments of the cones of \var\ 
in counterclockwise order.}*/

   list<segment> segments(const location& rl); 
/*{\Mop returns the sequence of bounding segments of the cones of \var\ 
in counterclockwise order.}*/

   list<double> alphas();  
/*{\Mop returns the angle sequence of \var\ .}*/

   list<point> conepoints(const point& r, int c);  
/*{\Mop returns the extremal points, creating the cone $c$.}*/

   list<point> conepoints(const location& rl, int c);  
/*{\Mopl returns the extremal points, creating the cone $c$.}*/

   list<segment> coneseg(const point& r, int c);  
/*{\Mop returns the segments, creating the cone $c$.}*/

   list<segment> coneseg(const location& rl, int c);  
/*{\Mop returns the segments, creating the cone $c$.}*/

   int conetest(const point& r, int c, const point& p, bool& unique);  
/*{\Mopl returns 1 if $p$ lies inside the cone $c$, 0 otherwise; if $p$ lies 
on a segment border, unique is false; if $r$ is equal to $p$, 0 is returned and 
unique is false.}*/

   int conetest(const location& rl, int c, const location& l, bool& unique);  
/*{\Mopl returns 1 if $l$ lies inside the cone $c$, 0 otherwise; if $l$ lies 
on a segment border, unique is false; if $rl$ is equal to $l$, 0 is returned and 
unique is false.}*/

   int inCone(const point& r, const point& p, bool& unique);  
/*{\Mopl returns the number of the cone, $p$ lies inside; if $p$ lies on a 
segment border, unique is false; if $r$ is equal to $p$, 0 is returned and unique is false.}*/

   int inCone(const location& rl, const location& l, bool& unique);  
/*{\Mopl returns the number of the cone, $l$ lies inside; if $l$ lies on a segment border, unique is false; if $rl$ is equal to $l$, 0 is returned and unique is false.}*/

   list<point> pl_inCone(const point& r, int c, const list<point>& pl);  
/*{\Mopl returns the list of points which lie inside the cone $c$, returns the empty list if no point is inside the cone.}*/

   list<location> ll_inCone(const location& rl, int c, const list<location>& ll);  
/*{\Mopl returns the list of locations which lie inside the cone $c$, returns the empty list if no location is inside the cone.}*/

   double norm(const point& r, const point& p);  
/*{\Mop returns the norm for the gauge from point $r$ to point $p$.}*/

   double norm(const location& rl, const location& l);  
/*{\Mopl returns the norm for the gauge from location $rl$ to location $l$.}*/

   list<double> lin_describe(const point& r, int c); 
/*{\Mop returns $m$ (in list.head()) and $b$ (in list.tail()) as the linear description ($y=m \cdot x+b$) of the cone $c$;
returns $inf$ (in list.head()) and $x$ (in list.tail()) if segment is vertical.}*/

   list<double> lin_describe(const location& rl, int c);
/*{\Mopl returns $m$ (in list.head()) and $b$ (in list.tail()) as the linear description ($y=m \cdot x+b$) of the cone $c$, returns $inf$ (in list.head()) and $x$ (in list.tail()) if segment is vertical.}*/

   point maxi(const point& r,int i);  
/*{\Mop returns the extremalpoint with the maximal coordinate, $i$=0 for 
$x$-coord and $i$=1 for $y$-coord.}*/

   point mini(const point& r,int i);  
/*{\Mop returns the extremalpoint with the minimal coordinate, $i$=0 for 
$x$-coord and $i$=1 for $y$-coord.}*/

   point maxi(const location& rl,int i);  
/*{\Mop returns the extremalpoint with the maximal coordinate, $i$=0 for 
$x$-coord and $i$=1 for $y$-coord.}*/

   point mini(const location& rl,int i);  
/*{\Mop returns the extremalpoint with the minimal coordinate, $i$=0 
for $x$-coord and $i$=1 for $y$-coord.}*/

   double max_diff(int i); 
/*{\Mop returns the  difference between $max \, i$ and $min \, i$, $i$=0 
for $x$-coord and $i$=1 for $y$-coord.}*/

   mixgauge dual();
/*{\Mop returns the dual gauge of $G$. }*/

   mixgauge rotate(double alpha);  
/*{\Mop returns the gauge created by a rotation of \var\ by angle $alpha$.}*/

   mixgauge join(mixgauge& H);
/*{\Mop returns the union of gauge $G$ and $H$.} */

   mixgauge unit();
/*{\Mop returns the gauge with all segments of length $1$. } */

   mixgauge scale(double scale);
/*{\Mop returns the gauge with all segments scaled with $scale$. } */

   bool symmetrical(); 
/*{\Mop returns true if \var\ is symmetrical, false otherwise. } */

   int size() { return seg_list.size(); }
/*{\Mop returns the size of \var.}*/

   bool empty() { return seg_list.empty(); }
/*{\Mop returns true if \var\ is empty, false otherwise.}*/


   bool operator == (const mixgauge& H);
/*{\Mbinop    Test for equality of $G$ and $H$.}*/

   bool operator != (const mixgauge& H) { return !operator==(H); };
/*{\Mbinop    Test for inequality of $G$ and $H$.}*/


friend ostream& operator << (ostream& out, const mixgauge& g);
friend istream& operator >> (istream& in,  mixgauge& g);

};

#endif 
