/*********************************************************************
* LoLA (Library of Location Algorithms)                              *
*                                                                    *
* CLASS PLANEALG
**********************************************************************/

#ifndef LOLA_PLANEALG_H
#define LOLA_PLANEALG_H

#include <LOLA/restrictions.h>
#include <LOLA/barrier.h>
//#include <LOLA/lines.h>


/*{\Manpage {planealg} {} {Algorithm}}*/

class planealg 
/*{\Mdefinition An instance of the data type $planealg$ consists of the algorithm to solve planar location problems.}*/
{
 protected: 
 list<location> solution;

 public: 
/*{\Mcreation P }*/
/*{\Mtext {\it planealg} P; \hspace{1cm}
creates a variable $P$ of type $planealg$.}*/
/*{\Moperations 2.3 4.7 }*/

  list<location>  alg_solution() {return solution;}

/*{\Mop returns the solution for the current problem.}*/

  void            WriteOpt();
  void            WriteOpt(ostream& out);
/*{\Mop writes the solution for the current problem.}*/

/*{\Mtext
\medskip
{\bf 3.1 Algorithms}
\medskip }*/
/*{\Mtext
\medskip
{\bf 3.1.1 Median-problems}
\medskip }*/

  double          l1_sum(facilities& EX); 
/*{\Mop computes the optimum for problem class $1/P/./l_1/\sum$
and returns the objective value.}*/
  double          l1_sum(facilities& EX, restrictions& R);
/*{\Mopl computes the optimum for problem class $1/P/R=convex/l_1/\sum$
{\it (Konstukrionslinienalgorithmus)} and returns the objective value.}*/
  double          l2sqr_sum(facilities& EX);
/*{\Mop computes the optimum for problem class $1/P/./l_2^2/\sum$ and returns the objective value.}*/
  double          l2sqr_sum(facilities& EX, restrictions& R);
/*{\Mopl computes the optimum for problem class $1/P/R=convex/l_2^2/\sum$
and returns the objective value.}*/
  double          linf_sum(facilities& EX);
/*{\Mop computes the optimum for problem class $1/P/./l_\infty/\sum$
and returns the objective value.}*/
  double          linf_sum(facilities& EX, restrictions& R);
/*{\Mopl computes the optimum for problem class $1/P/R=convex/l_\infty/\sum$
and returns the objective value.}*/
  double          l2_sum(facilities& EX, double epsilon);
/*{\Mopl computes the optimum for problem class $1/P/./l_2/\sum$ (Weiszfeld-Algorithm) and returns the objective value.}*/
  double          Weiszfeld(facilities& EX, double epsilon);
  double          l2_sum(facilities& EX, barrier& R);
/*{\Mopl computes the optimum for problem class $1/P/(R,\infty)/l_2/\sum$ and returns the 
  objective value, where $(R,\infty)$ may be {\bf one} barrier such as a circle, a segment 
  or a polygon.}*/
  double          l2_sum(facilities& EX, barrier& R,bool erase,int choose,double percent,double accel,double init_ss,double epsilon);
/*{\Mopl computes the optimum for problem class $1/P/(R,\infty)/l_2/\sum$ and returns the
  objective value, where $(R,\infty)$ may be {\bf one} barrier such as a circle, a segment 
  or a polygon. Same algorithm like the above one but user can choose from two heuristics(0 
  or 1) and define a percentage(percent) of points to handle, acceleration factor(accel) and 
  inital step size(init\_ss) as well as epsilon for the iterations.}*/

  double          lp_sum(facilities& EX, int p,double epsilon,double delta,int iter_max);
/*{\Mopl computes the optimum for problem class $1/P/./l_p/\sum$ 
(Generalized Weiszfeld-Algorithm) and returns the objective value.}*/

  double          lp_sum(facilities& EX, restrictions& R, int p,double epsilon,double delta,int iter_max);
/*{\Mopl computes the optimum for problem class $1/P/R/l_p/\sum$ 
 and returns the objective value.}*/

  double          l2sqr_qsum(facilities& EX);
/*{\Mopl computes the optimum for problem class $1/P/./l_2^2/Q-\sum$ 
(Q-criterial problem) and returns the objective value.}*/

  double          l1_2sum(facilities& EX);
/*{\Mopl computes the optimum for problem class $1/P/./l_1/2-\sum$
(bi-criterial problem) and returns the objective value.}*/

  double          linf_2sum(facilities& EX);
/*{\Mopl computes the optimum for problem class $1/P/./linf/2-\sum$
(bi-criterial problem) and returns the objective value.}*/

  double          N_l1_sum(facilities& EX, int n, matrix& w);  
/*{\Mopl computes the optimum for problem class $N/P/./l_1/\sum$ 
and returns the objective value.}*/

  double          N_linf_sum(facilities& EX, int n, matrix& w);  
/*{\Mopl computes the optimum for problem class $N/P/./l_\infty/\sum$ 
and returns the objective value.}*/


  double          N_l2sqr_sum(facilities& EX, int n, matrix& w);  
/*{\Mopl computes the optimum for problem class $N/P/./l_2^2/\sum$ 
and returns the objective value.}*/

  double*         N_lp_sum_v1(facilities& EX, int n,int p,matrix& w);  
/*{\Mopl
 computes the optimum for problem class $N/P/./l_p/\sum$ (Version 1) and returns the objective value. }*/

  double*	  N_lp_sum_v2(facilities& EX, int n,int p,matrix& w,double epsilon);
/*{\Mopl
 computes the optimum for problem class $N/P/./l_p/\sum$ (Version 2) and returns the objective value.  }*/

double          L_l2_sum_M3(facilities &EX);
/*{\Mopl computes the optimum for problem class $1L/P/./l_2/\sum$
and returns the objective value.}*/

double          L_l1_sum_M3(facilities &EX);
/*{\Mopl computes the optimum for problem class $1L/P/./l_1/\sum$
and returns the objective value.}*/

double          L_linf_sum_M3(facilities &EX);
/*{\Mopl computes the optimum for problem class $1L/P/./l_\infty/\sum$
and returns the objective value.}*/

double          L_lp_sum_M3(facilities &EX, int p);
/*{\Mopl computes the optimum for problem class $1L/P/./l_p/\sum$
and returns the objective value.}*/

double          L_l2_sum_M2logM(facilities &EX);
/*{\Mopl computes the optimum for problem class $1L/P/./l_2/\sum$
with $O(M^2logM)$ and returns the objective value.}*/

double          L_l2_sum_M2(facilities &EX);
/*{\Mopl computes the optimum for problem class $1L/P/v_i=1/l_2/\sum$
with $O(M^2)$ and returns the objective value.}*/

double          L_RkonvPoly_l2_sum(facilities& EX, restrictions& R);
/*{\Mopl computes the optimum for problem class $1L/P/R=convex/l_2/\sum$
and returns the objective value.}*/

double          L_RkonvPoly_lp_sum(facilities& EX, restrictions& R, int p);
/*{\Mopl computes the optimum for problem class $1L/P/R=convex/l_p/\sum$
and returns the objective value.}*/

double          L_RkonvPoly_l1_sum(facilities& EX, restrictions& R);
/*{\Mopl computes the optimum for problem class $1L/P/R=convex/l_1/\sum$
and returns the objective value.}*/

double          L_RkonvPoly_linf_sum(facilities& EX, restrictions& R);
/*{\Mopl computes the optimum for problem class $1L/P/R=convex/l_{inf}/\sum$
and returns the objective value.}*/


/*{\Mtext
\medskip
{\bf 3.1.2 Center-problems}
\medskip }*/

  double          l1_max(facilities& EX);
/*{\Mopl computes the optimum for problem class $1/P/./l_1/max$
and returns the objective value.}*/
  double l1_max(facilities& EX, restrictions& R);
 /*{\Mopl computes the optimum for problem class $1/P/R=convex/l_1/max$
and returns the objective value.}*/
  double          l1_v1_max(facilities& EX);  
/*{\Mopl computes the optimum for problem class $1/P/v_i=1/l_1/max$  
and returns the objective value.}*/
  double          l2_max(facilities& EX);
/*{\Mopl computes the optimum for problem class $1/P/./l_2/max$
and returns the objective value.}*/
  double          l2_max(facilities& EX, restrictions& R);
/*{\Mopl computes the optimum for problem class 
$1/P/R=\mbox{convex polyhedron},v_i=1/l_2/max$ and returns the objective value.}*/
  double          linf_max(facilities& EX);
/*{\Mopl computes the optimum for problem class $1/P/./l_\infty/max$
and returns the objective value.}*/
  double          linf_max(facilities& EX, restrictions& R);
/*{\Mopl computes the optimum for problem class $1/P/R=convex/l_\infty/max$
and returns the objective value.}*/
  double          linf_v1_max(facilities& EX); 
/*{\Mopl computes the optimum for problem class $1/P/v_i=1/l_\infty/max$ 
and returns the objective value.}*/
  double          elzhearn(facilities& EX);
/*{\Mopl computes the optimum for Elzinga-Hearn-Algorithm
and returns the objective value.}*/
  double          N_linf_max(facilities& EX, int N, matrix& W);
/*{\Mopl computes the optimum for problem class $N/P/./l_\infty/max$
and returns the objective value.}*/
  double          N_l1_max(facilities& EX, int N, matrix& W);
/*{\Mopl computes the optimum for problem class $N/P/./l_1/max$
and returns the objective value.}*/
//  double          N_l1_max_mps(facilities& EX, int N, matrix& W);
//  double          N_linf_max_mps(facilities& EX, int N, matrix& W);

double          L_l2_max_M4(facilities &EX);
/*{\Mopl computes the optimum for problem class $1L/P/./l_2/max$
and returns the objective value.}*/

double          L_l1_max_M4(facilities &EX);
/*{\Mopl computes the optimum for problem class $1L/P/./l_1/max$
and returns the objective value.}*/

double          L_linf_max_M4(facilities &EX);
/*{\Mopl computes the optimum for problem class $N/P/./l_\infty/max$
and returns the objective value.}*/
double          L_l2_max_MlogM(facilities &EX);
/*{\Mopl computes the optimum for problem class $1L/P/v_i=1/l_2/max$
with $O(MlogM)$ and returns the objective value.}*/

double          L_l2_max_M2logM(facilities &EX);
/*{\Mopl computes the optimum for problem class $1L/P/./l_2/max$
with $O(M^2logM)$ and returns the objective value.}*/

double          L_lp_max_M4(facilities &EX, int p);
/*{\Mopl computes the optimum for problem class $1L/P/./l_p/max$
and returns the objective value.}*/
};

#endif

