/***************************************************************************
* class polygauge for LOLA (Library of Location Algorithms)
* Purpose: 
* Author : Lothar Schnell
* Date   : Dez-11-1995
***************************************************************************/
#ifndef LOLA_PGAUGE_H
#define LOLA_PGAUGE_H

#include <LEDA/plane_alg.h>
#include <LOLA/location.h>

/*{\Manpage {polygauge} {} {Polyhedral Gauges} }*/

class polygauge {
/*{\Mdefinition
An instance $G$ of the data type $polygauge$ is a gauge
in the two-dimensional plane defined by the sequence of its vertices
in counterclockwise order which creates the cones. These cones are numbered
in counterclockwise order. The number of vertices is called the size of $G$.
See also the Gauge-utilities header (gauge\_util.h), which provides
Load, Save, Create and View for Gauges.}*/
 
 protected:
   list<segment> seg_list;
   list<double> alpha_list;

 public:
   
/*{\Mcreation G }*/

   polygauge(const list<point>& pl);
/*{\Mcreate introduces a variable \var\ of type \name. \var\ is initialized 
to the polygauge with vertex sequence $pl$.\\
\precond The vertices in $pl$ are given in counterclockwise order and define a polygauge. The polyhedral belonging to the polygauge must be convex.}*/

   polygauge(); 
/*{\Mcreate introduces a variable \var\ of type \name. \var\ is initialized to the empty polygauge.}*/

   polygauge(const polygauge& G);
   ~polygauge();

   polygauge& operator = (const polygauge& G);

/*{\Moperations 2.5 4.4}*/

   point operator [] (int c);
/*{\Marrop returns the $c$-th extremalpoint of \var .}*/


   point origin() {return seg_list.head().source();}  
/*{\Mop returns the origin \var .}*/

   list<point> vertices() const;  
/*{\Mop returns the vertex sequence of \var\ for the reference point 
$origin$.}*/

   list<point> vertices(const point& r) const;  
/*{\Mop returns the vertex sequence of \var\ for the reference point $r$.}*/

   list<point> vertices(const location& rl) const;  
/*{\Mop returns the vertex sequence of \var\ for the reference location $rl$.}*/

   list<segment> segments(const point& r); 
/*{\Mop returns the sequence of bounding segments of the cones of \var\ 
in counterclockwise order for the reference point $r$.}*/

   list<segment> segments(const location& rl); 
/*{\Mop returns the sequence of bounding segments of the cones of \var\ 
in counterclockwise order for the reference location $rl$.}*/

   list<double> alphas();  
/*{\Mop returns the angle sequence of \var\.}*/

   list<point> conepoints(const point& r, int c);  
/*{\Mop returns the extremal points, creating the cone $c$.}*/

   list<point> conepoints(const location& rl, int c);  
/*{\Mopl returns the extremal points, creating the cone $c$.}*/

   list<segment> coneseg(const point& r, int c);  
/*{\Mop returns the segments, creating the cone $c$.}*/

   list<segment> coneseg(const location& rl, int c);  
/*{\Mop returns the segments, creating the cone $c$.}*/

   int conetest(const point& r, int c, const point& p, bool& unique);  
/*{\Mopl returns 1 if $p$ lies inside the cone $c$, 0 otherwise; if $p$ 
lies on a segment border, unique is false; if $r$ is equal to $p$, 0 is 
returned and unique is false.}*/

   int conetest(const location& rl, int c, const location& l, bool& unique);  
/*{\Mopl returns 1 if $l$ lies inside the cone $c$, 0 otherwise; if $l$ 
lies on a segment border, unique is false; if $rl$ is equal to $l$, 0 is returned 
and unique is false.}*/

   int inCone(const point& r, const point& p, bool& unique);  
/*{\Mopl returns the number of the cone, $p$ lies inside; if $p$ lies on a segment border, unique is false; if $r$ is equal $p$, 0 is returned and unique is 
false.}*/

   int inCone(const location& rl, const location& l, bool& unique);  
/*{\Mopl returns the number of the cone, $l$ lies inside; if $l$ lies on a segment border, unique is false; if $rl$ is equal to $l$, 0 is returned and unique 
is false.}*/

   list<point> pl_inCone(const point& r, int c, const list<point>& pl);  
/*{\Mopl returns the list of points $pl$ which lie inside the cone $c$, returns the empty list if no point is inside the cone.}*/

   list<location> ll_inCone(const location& rl, int c, const list<location>& ll);  
/*{\Mopl returns the list of locations of $ll$ which lie inside the cone $c$, returns the empty list if no location is inside the cone.}*/

   double norm(const point& r, const point& p);  
/*{\Mop returns the norm for the gauge from point $r$ to point $p$.}*/

   double norm(const location& rl, const location& l);  
/*{\Mopl returns the norm for the gauge from location $rl$ to location $l$.}*/

   list<double> lin_describe(const point& r, int c); 
/*{\Mop returns $m$ (in list.head()) and $b$ (in list.tail()) as the linear description ($y=m \cdot x+b$) of the cone $c$;
returns $inf$ (in list.head()) and $x$ (in list.tail()) if segment is vertical.}*/

   list<double> lin_describe(const location& rl, int c);
/*{\Mopl returns $m$ (in list.head()) and $b$ (in list.tail()) as the linear description ($y=m \cdot x+b$) of the cone $c$, returns $inf$ (in list.head()) and $x$ (in list.tail()) if segment is vertical.}*/

   double max_dist();  
/*{\Mop returns the maximal euclidean distance from origin of the gauge $G$ 
to the extremalpoints.}*/

   point maxi(const point& r,int i);  
/*{\Mop returns the extremalpoint of $G$ with the maximal coordinate, 
$i$=0 for $x$-coord and $i$=1 for $y$-coord.}*/

   point mini(const point& r,int i);  
/*{\Mop returns the extremalpoint of $G$ with the minimal coordinate, $i$=0 
for $x$-coord and $i$=1 for $y$-coord.}*/

   point maxi(const location& rl,int i);  
/*{\Mop returns the extremalpoint of $G$ with the maximal coordinate, $i$=0 
for $x$-coord and $i$=1 for $y$-coord.}*/

   point mini(const location& rl,int i);  
/*{\Mop returns the extremalpoint of $G$ with the minimal coordinate, $i$=0 
for $x$-coord and $i$=1 for $y$-coord.}*/

   double max_diff(int i); 
/*{\Mop returns the  difference between $max \, i$ and $min \, $, $i$=0 
for $x$-coord and $i$=1 for $y$-coord.}*/

   polygauge dual();
/*{\Mop returns the dual gauge of $G$. }*/

   polygauge rotate(double alpha);  
/*{\Mop returns the gauge created by a rotation of \var\ by angle $alpha$.}*/

   polygauge l1(); 
/*{\Mop returns the $l_1$ gauge. } */

   polygauge linf(); 
/*{\Mop returns the $l_\infty$ gauge. } */

   polygauge unit(); 
/*{\Mop returns the gauge with all segments of length $1$. } */

   polygauge join(polygauge& H); 
/*{\Mop returns the union of gauge $G$ and $H$. } */

   polygauge scale(double scale); 
/*{\Mop returns the gauge with all segments scaled with $scale$. } */

   polygauge translate(const point& r); 
/*{\Mop returns the gauge with all segments translated to $r$. } */

   bool symmetrical(); 
/*{\Mop returns true if \var\ is symmetrical, false otherwise. } */

   int size() { return seg_list.size(); }
/*{\Mop returns the size of \var.}*/

   bool empty() { return seg_list.empty(); }
/*{\Mop returns true if \var\ is empty, false otherwise.}*/


   bool operator == (const polygauge& H);
/*{\Mbinop    Test for equality of $G$ and $H$.}*/

   bool operator != (const polygauge& H) { return !operator==(H); };
/*{\Mbinop    Test for inequality of $G$ and $H$.}*/

friend ostream& operator << (ostream& out, const polygauge& g);
friend istream& operator >> (istream& in, polygauge& g);
};

#endif 
