#ifndef _RESTRICTION_H_
#define _RESTRICTION_H_

#include <LEDA/basic.h>
#include <LEDA/polygon.h>
#include <LEDA/map.h>
#include <LEDA/ray.h>
#include <LOLA/facilities.h>

/*{\Manpage {restriction} {} {Restriction}}*/

/*{\Mdefinition Restriction is one of three restriction-types ($restr\_poly$, 
$restr\_circle$ or $restr\_rect$) and has the following virtual function for 
all of them.
}*/
/*{\Mcreation R }*/
/*{\Mtext {\it restriction R}; \hspace{1cm}
creates a variable $R$ of type $restriction$.}*/
/*{\Moperations 1.6 4.9 }*/
/*{\Mtext
\begin{tabular}{p{1.9cm}lp{8cm}}
{\it int} & type() &
returns the type of the restriction ($restr\_p$, $restr\_c$ or $restr\_r$). \\
[2mm]
{\it bool} & inout() &
returns true if the forbidden region should be inside of $R$, false otherwise. \\[2mm]
{\it void} & in\_forbid() &
changes forbidden region to inside. \\[2mm]
{\it void} & out\_forbid() &
changes forbidden region to outside. \\[2mm]
{\it bool} & inside(point p) &
returns true if $p$ lies inside of $R$, false otherwise. \\[2mm]
{\it bool} & inside(location Loc) &
returns true if $Loc$ lies inside of $R$, false otherwise. \\[2mm]
{\it bool} & inside(segment seg) &
returns true if $seg$ lies inside of $R$, false otherwise. \\[2mm]
\end{tabular}
\begin{tabular}{p{1.9cm}lp{5cm}}
{\it list$<$point$>$} & intersect(ray r) &
returns $R \cap r$ as a list of points. \\[2mm]
{\it list$<$point$>$} & intersection(line l) &
returns $R \cap l$ as a list of points. \\[2mm]
{\it list$<$point$>$} & intersection(segment s) &
returns $R \cap s$ as a list of points. \\[2mm]
{\it list$<$point$>$} & intersection(polygon P) &
returns $R \cap P$ as a list of points. \\[2mm]
{\it list$<$point$>$} & intersection(circle C) &
returns $R \cap C$ as a list of points. \\[2mm]
\end{tabular}
\begin{tabular}{lp{12cm}}
{\it list$<$location$>$} & proj\_l2sqr(location opt) \\
& $\quad$ returns the projected locations for the $l_2^2$ Norm. \\[2mm]
{\it double} & proj\_v1\_l2\_max(location opt, facilities facs, double z\_opt) \\
& $\quad$ returns the objective value for the projected solution in opt for the problem class $1/P/w_m=1/l_2/max$. \\[2mm]
\end{tabular}
}*/
/*{\Mtext
%************
\begin{tabbing}
{\it int} \qquad \=  type() \\
\> \qquad \quad returns the type of the restriction ($restr\_p$, $restr\_c$ or $restr\_r$). \\ [0.8ex]
{\it bool}  \qquad \= inout() \\
\> \qquad \quad returns true if the forbidden region should be inside of $R$, false otherwise. \\ [0.8ex]
{\it void}  \qquad \= in\_forbid() \\
\> \qquad \quad changes forbidden region to inside. \\  [0.8ex]
{\it void}  \qquad \= out\_forbid() \\
\> \qquad \quad changes forbidden region to outside. \\ [0.8ex]
{\it bool}  \qquad \= inside(point p) \\
\> \qquad \quad returns true if $p$ lies inside of $R$, false otherwise. \\ [0.8ex]
{\it bool}  \qquad \= inside(location Loc) \\
\> \qquad \quad returns true if $Loc$ lies inside of $R$, false otherwise. \\ [0.8ex]
{\it bool}  \qquad \=  inside(segment seg) &
\> \qquad \quad returns true if $seg$ lies inside of $R$, false otherwise. \\ [0.8ex]
{\it list$<$point$>$}  \qquad \=  intersect(ray r) \\
\> \qquad \quad returns $R \cap r$ as a list of points. \\ [0.8ex]
{\it list$<$point$>$}  \qquad \=  intersection(line l) \\
\> \qquad \quad returns $R \cap l$ as a list of points. \\ [0.8ex]
{\it list$<$point$>$}  \qquad \=  intersection(segment s) \\
\> \qquad \quad returns $R \cap s$ as a list of points. \\ [0.8ex]
{\it list$<$point$>$}  \qquad \=  intersection(polygon P) \\
\> \qquad \quad returns $R \cap P$ as a list of points. \\ [0.8ex]
{\it list$<$point$>$}  \qquad \=  intersection(circle C) \\
\> \qquad \quad returns $R \cap C$ as a list of points. \\ [0.8ex]
{\it list$<$location$>$}  \qquad \=  proj\_l2sqr(location opt) \\
\> \qquad \quad returns the projected locations for the $l_2^2$ Norm. \\ [0.8ex]
{\it double}  \qquad \=  proj\_v1\_l2\_max(location opt, facilities facs, double z\_opt) \\
\> \qquad \quad returns the objective value for the projected solution in opt for the problem class $1/P/w_m=1/l_2/max$. \\ 
\end{tabbing}


\newpage
{\bf Non-virtual functions} \\[4mm]
\begin{tabular}{lp{12cm}}
{\it bool} & inside\_all(restriction$*$ restrict, list$<$location$>$ opt) \\
& $\quad$ returns true if all locations lie inside the restriction, false otherwise. \\[2mm]
restriction$*$ & inside\_which(list$<$restriction$*>$ restrict, location loc) \\
& $\quad$ returns $restriction$ where $loc$ lies inside. \\[2mm]
restriction$*$ & inside\_which(list$<$restriction$*>$ restrict, segment seg); \\
& $\quad$ returns $restriction$ where $seg$ lies inside. \\[2mm]
restriction$*$ & inside\_which(list$<$restriction$*>$ restrict, list$<$location$>$ opt); \\
& $\quad$ returns $restriction$ where all locations from $opt$ lie inside. \\
\end{tabular}
}*/

enum {restr_p, restr_r, restr_c, restr_e, restr_s}; 
enum {t_forbidden_region,t_congested_region,t_barrier};
enum {t_rectangle,t_circle,t_segment,t_polygon};

struct restriction {
public:
  virtual ~restriction() {}
  virtual int type() = 0;
  virtual int subtype() = 0;
  virtual bool inout() = 0;
  //   virtual double bar_typ() = 0;
  virtual void in_forbid() = 0;
  virtual void out_forbid() = 0;
  virtual bool inside(const point &P) = 0;
  virtual bool inside(const location &Loc) = 0;
  virtual bool inside(const segment &seg) = 0;
  virtual list<point> intersect(point &p, point &q) = 0;
  //   virtual list<point> intersect(ray &r) = 0;
  virtual list<point> intersect(ray &r, point &p) = 0;
  virtual list<point> intersection(const line &L) = 0;
  virtual list<point> intersection(const segment &S) = 0;
  virtual list<point> intersection(const polygon &P) = 0;
  virtual list<point> intersection(const circle &C) = 0;

  virtual double objective(const facilities& EX,const point& p) = 0;

/*  virtual bool on_edge(const point& p) = 0;
  virtual bool on_edge(const location& l) = 0;
  virtual bool visible(const point&,const point&) = 0;
  virtual point l_tang_point(const point&) = 0;
  virtual point r_tang_point(const point&) = 0;*/

  
  virtual list<location> proj_l2sqr(const location &opt) = 0;
  
  virtual double proj_v1_l2_max(location &opt, facilities &facs, double z_opt_tmp);
  virtual double gauge_max(list<location> &, facilities &, double, list<polygauge> &, list<int> &);
  double proj_conv_linf_max(list<location> &opt, facilities &facs, double z);
  list<point> cutGauge(polygauge &G);
};

bool inside_all(restriction* &restrict, list<location> &opt);
restriction* inside_which(list<restriction*> &restrict, location &loc);
restriction* inside_which(list<restriction*> &restrict, segment &seg);
restriction* inside_which(list<restriction*> &restrict, list<location> &opt);

#endif
