#include <LOLA/polygauge.h>
#include <LEDA/matrix.h>

ostream& operator << (ostream& out, const polygauge &g) {
	g.vertices(origin).print(out);
	return out << endl;
} 

istream& operator >> (istream& in, polygauge &g) {
	list<point> L; 
	L.read(in,'\n'); 
	g = polygauge(L); 
	return in;
}

polygauge::polygauge() {
	list<point> pl;
	pl.empty();
}

polygauge::polygauge(const list<point> &pl){
	seg_list.empty();
	alpha_list.empty();
	list_item it;
	for(it = pl.first(); it; it = pl.succ(it)) {
		point q = pl[it];
		seg_list.append(segment(point(0,0),q));
	}
	segment segf=seg_list.head(),s;
	segment segl=seg_list.tail();
	double tmpangle;
	double angletst=-1;
	forall(s, seg_list) { 
		tmpangle=segf.angle(s);
        if (tmpangle <0) tmpangle += 2*LEDA_PI;
        if (tmpangle <= angletst) 
			lola_error("polygauge: must be in counterclockwise order",1701,1);
		alpha_list.append(tmpangle);
		angletst=tmpangle;
	}
	it=alpha_list.succ(alpha_list.first());
	while (it != alpha_list.first()) {
		if (fabs(alpha_list[alpha_list.pred(it)]-alpha_list[it])>=LEDA_PI) 
			lola_error("polygauge: alpha > PI",1702,1);
		it=alpha_list.cyclic_succ(it);
	}
	list<point> C = CONVEX_HULL(pl);
	if (C.size() != size()) lola_error("polygauge: is not convex",1703,1);
}

polygauge::polygauge(const polygauge &G) {
	seg_list = G.seg_list;
	alpha_list = G.alpha_list;
}

polygauge::~polygauge() {
	seg_list.clear();
	alpha_list.clear();
}

polygauge& polygauge::operator = (const polygauge &G) {
	seg_list = G.seg_list;
	alpha_list = G.alpha_list;
	return *this;
}

point polygauge::operator[](int c) {
	if ((c > size())||(c < 1)) lola_error("polygauge: Conenumber is not correct",1705,1);
	list_item s_it=seg_list.first();
	for(int i=1; i<c; ++i) s_it = seg_list.cyclic_succ(s_it);
	return (seg_list[s_it].end());
}

list<double> polygauge::alphas() {
	list<double> result;
	double a;
	forall(a, alpha_list) result.append(a);
	return result;
}

list<segment> polygauge::segments(const point& r) {
	list<segment> result;
	segment s;
	forall(s, seg_list) {
		point q(r.xcoord()+s.xcoord2(),r.ycoord()+s.ycoord2());
		result.append(segment(r,q));
	}
	return result;
}


list<segment> polygauge::segments(const location& rl) {
	point tmp;
	tmp=rl.loc2point();
	return segments(tmp);
}

list<point> polygauge::vertices(const point& r) const{
	list<point> result;
	segment s;
	forall(s, seg_list) {
		point tmp(s.xcoord2()-s.xcoord1()+r.xcoord(),s.ycoord2()-s.ycoord1()+r.ycoord());
		result.append(tmp);
	}
	return result;
}


list<point> polygauge::vertices(const location& rl) const{
	point tmp = rl.loc2point();
	return vertices(tmp);
}

list<point> polygauge::vertices() const{
	point origin(0,0);
	return vertices(origin);
}

list<point> polygauge::conepoints(const point& r,int c) {
	if ((c > size())||(c < 1)) lola_error("polygauge: Conenumber is not correct",1706,1);
	list<point> result;
	list_item s_it=seg_list.first();
	point tmp;
	for(int i=1; i<c; ++i) s_it = seg_list.cyclic_succ(s_it);
	tmp=point(seg_list[s_it].xcoord2()+r.xcoord(),seg_list[s_it].ycoord2()+r.ycoord());
	result.append(tmp);
	s_it = seg_list.cyclic_succ(s_it);
	tmp=point(seg_list[s_it].xcoord2()+r.xcoord(),seg_list[s_it].ycoord2()+r.ycoord());
	result.append(tmp);
	return result;
}

list<point> polygauge::conepoints(const location& rl,int c) {
	if ((c > size())||(c < 1)) lola_error("polygauge: Conenumber is not correct",1707,1);
	point tmp = rl.loc2point();
	return conepoints(tmp,c);
}

list<segment> polygauge::coneseg(const point& r,int c) {
	if ((c > size())||(c < 1)) lola_error("polygauge: Conenumber is not correct",1708,1);
	list<segment> result;
	list_item s_it=seg_list.first();
	point tmp;
	for(int i=1; i<c; ++i) s_it = seg_list.cyclic_succ(s_it);
	tmp=point(seg_list[s_it].xcoord2()+r.xcoord(),seg_list[s_it].ycoord2()+r.ycoord());
	result.append(segment(r,tmp));
	s_it = seg_list.cyclic_succ(s_it);
	tmp=point(seg_list[s_it].xcoord2()+r.xcoord(),seg_list[s_it].ycoord2()+r.ycoord());
	result.append(segment(r,tmp));
	return result;
}

list<segment> polygauge::coneseg(const location& rl,int c) {
	if ((c > size())||(c < 1)) lola_error("polygauge: Conenumber is not correct",1709,1);
	point tmp = rl.loc2point();
	return coneseg(tmp,c);
}


int polygauge::inCone(const point &r, const point &p, bool& unique) {
	int cone=0;
	int c=0;
	if (r==p) {
		unique=false;
		return 0;
	}
	while (!cone) {
		++c;
		cone=conetest(r,c,p,unique);
	}
	return c;
}

int polygauge::inCone(const location &rl, const location &l, bool& unique) {
	point tmp1 = rl.loc2point();
	point tmp2 = l.loc2point();
	return inCone(tmp1,tmp2,unique);
}

int polygauge::conetest(const point &r, int c, const point &p, bool& unique) {
	if ((c > size())||(c < 1)) lola_error("polygauge: Conenumber is not correct",1710,1);
	unique=true;
	if (r==p) {
		unique=false;
		return 0;
	}
	segment s=seg_list.head();
	point segfend(r.xcoord()+s.xcoord2(),r.ycoord()+s.ycoord2());
	segment segf(r,segfend);
	segment tmp(r,p);
	list_item a_it=alpha_list.first();
	double angletst=segf.angle(tmp);
	if (angletst <0) angletst += 2*LEDA_PI;   
	if ((c==size())&&(alpha_list.tail()<angletst)) return 1;
	for(int i=1; i<c; ++i) a_it = alpha_list.cyclic_succ(a_it);
	if ((alpha_list[a_it] == angletst)||(alpha_list[alpha_list.cyclic_succ(a_it)] == angletst)) unique=false;
	if ((alpha_list[a_it] <= angletst)&&(alpha_list[alpha_list.cyclic_succ(a_it)] >= angletst)) return 1;
	else return 0;
}

int polygauge::conetest(const location &rl, int c, const location &l,bool& unique) {
	if ((c > size())||(c < 1)) lola_error("polygauge: Conenumber is not correct",1711,1);
	point tmp1 = rl.loc2point();
	point tmp2 = l.loc2point();
	return conetest(tmp1,c,tmp2,unique);
}

list<point> polygauge::pl_inCone(const point &r, int c, const list<point> &pl) {
	if ((c > size())||(c < 1)) lola_error("polygauge: Conenumber is not correct",1712,1);
	list<point> result;
	point tmp;
	bool unique;
	result.clear();
	forall(tmp,pl) {
		if (conetest(r,c,tmp,unique)==1) result.append(tmp);
	}
	return result;
}

list<location> polygauge::ll_inCone(const location &rl, int c, const list<location> &ll) {
	if ((c > size())||(c < 1)) lola_error("polygauge: Conenumber is not correct",1713,1);

	list<location> result;
	location tmp;
	bool unique;
	result.clear();
	forall(tmp,ll) {
		if (conetest(rl,c,tmp,unique)==1) result.append(tmp);
	}
	return result;
}

double polygauge::norm(const point &r, const point &p) {
	bool unique;
	if (r==p) return 0;
	matrix M(2,2);
	vector b(p.xcoord()-r.xcoord(),p.ycoord()-r.ycoord());
	int c=inCone(r,p,unique);
	list<point> ext=conepoints(r,c);
	M(0,0)=ext.head().xcoord()-r.xcoord();
	M(1,0)=ext.head().ycoord()-r.ycoord();
	M(0,1)=ext.tail().xcoord()-r.xcoord();
	M(1,1)=ext.tail().ycoord()-r.ycoord();
	vector x=M.solve(b);
	return (fabs(x[0])+fabs(x[1]));
}


double polygauge::norm(const location &rl, const location &l) {
	point tmp1 = rl.loc2point();
	point tmp2 = l.loc2point();
	return norm(tmp1,tmp2);
}


list<double> polygauge::lin_describe(const point& r,int c) {
	list<double> result;
	list<point> pltmp=conepoints(r,c);
	line l(pltmp.head(),pltmp.tail());   
	if (l.is_vertical()) {
		result.append(L_INF);  
		result.append(l.x_proj(1));
	}
	else {
		result.append(l.slope());  
		result.append(l.y_abs());
	}
	return result;
}


list<double> polygauge::lin_describe(const location& rl,int c) {
	point tmp=rl.loc2point();
	return lin_describe(tmp,c);}

double polygauge::max_dist() {
	list<point> ext=vertices(point(0,0));
	double result=0;
	point p;
	forall(p,ext) 
		if (p.distance() > result ) result=p.distance();
		return result;
}

point polygauge::maxi(const point& r,int i) {
	segment s;
	double maximum;
	point result(seg_list.head().end());
	if (i==0) {
		maximum=seg_list.head().xcoord2();
		forall(s, seg_list) {
			if (s.xcoord2()>maximum) {
				maximum=s.xcoord2();
				result=s.end();
			}
		}
	} else if (i==1) {
		maximum=seg_list.head().ycoord2();
		forall(s, seg_list) {
			if (s.ycoord2()>maximum) {
				maximum=s.ycoord2();
				result=s.end();
			}
		}
	} else lola_error("polygauge:max i must be 0 or 1",1714,1);

	result=point(result.xcoord()+r.xcoord(),result.ycoord()+r.ycoord());
	return result;
}

point polygauge::mini(const point& r,int i) {
	segment s;
	double minimum;
	point result(seg_list.head().end());
	if (i==0) {
		minimum=seg_list.head().xcoord2();
		forall(s, seg_list) {
			if (s.xcoord2()<minimum) {
				minimum=s.xcoord2();
				result=s.end();
			}
		}
	} else if (i==1) {
		minimum=seg_list.head().ycoord2();
		forall(s, seg_list) {
			if (s.ycoord2()<minimum) {
				minimum=s.ycoord2();
				result=s.end();
			}
		}
	} else lola_error("polygauge:mini i must be 0 or 1",1715,1);

	result=point(result.xcoord()+r.xcoord(),result.ycoord()+r.ycoord());
	return result;
}

point polygauge::maxi(const location& rl,int i) {
	point tmp = rl.loc2point();
	return maxi(tmp,i);
}

point polygauge::mini(const location& rl,int i) {
	point tmp = rl.loc2point();
	return mini(tmp,i);
}

double polygauge::max_diff(int i) {
	if (i==0)  return maxi(point(0,0),0).xcoord()-mini(point(0,0),0).xcoord();
	else if (i==1) return maxi(point(0,0),1).ycoord()-mini(point(0,0),1).ycoord();
	else lola_error("polygauge:max_diff i must be 0 or 1",1716,1);
	return 1;
}

polygauge polygauge::rotate(double alpha) {
	segment tmp;
	segment s;
	list<point> rot_list;
	forall(s,seg_list) {
		tmp=s.rotate(alpha);
		rot_list.append(tmp.end());
	}
	polygauge Gr(rot_list);
	return Gr;
}

polygauge polygauge::dual() {
	segment yseg(point(0,0),point(0,1));
	list<point> ext=vertices(point(0,0));
	list<point> dual_list;
	matrix M(2,2);
	vector b(1,1);
	list_item e_it=ext.succ(ext.first());

	while (e_it != ext.first())
	{
		M(0,0)=ext[ext.pred(e_it)].xcoord();
		M(0,1)=ext[ext.pred(e_it)].ycoord();
		M(1,0)=ext[e_it].xcoord();
		M(1,1)=ext[e_it].ycoord();
		vector p=M.solve(b);
		dual_list.append(point(p[0],p[1]));
		e_it = ext.cyclic_succ(e_it);
	}
	M(0,0)=ext.tail().xcoord();
	M(0,1)=ext.tail().ycoord();
	M(1,0)=ext.head().xcoord();
	M(1,1)=ext.head().ycoord();
	vector p=M.solve(b);
	segment stest(point(0,0),point(p[0],p[1]));
	if (yseg.angle(stest) < 0) dual_list.append(point(p[0],p[1]));
	else dual_list.push(point(p[0],p[1]));
	polygauge Gd(dual_list);
	return Gd;
}

bool polygauge::symmetrical() {
	if ((size()%2) != 0) return false;
	list<point> GP=vertices(point(0,0));
	int i=1;
	list_item it_1=GP.first();
	list_item it_2=GP.first();
	while (i<=(size()/2)) {
		it_2=GP.succ(it_2);
		++i;
	}
	i=1;
	while (i<=(size()/2)) {
		if ((GP[it_1].xcoord()!=(-1)*GP[it_2].xcoord())||(GP[it_1].ycoord()!=(-1)*GP[it_2].ycoord())) return false;
		else {
			it_1=GP.succ(it_1);
			it_2=GP.succ(it_2);
			++i;
		}
	}
	return true;
}

polygauge polygauge::join(polygauge& H) {
	double a;
	int C1=1;
	int C2=1;
	point p;
	point origin(0,0);
	list<point> result;
	list<double> Gal;
	list<double> Hal;
	segment yseg(point(0,0),point(0,1));
	list<double> al = alphas();
	segment stmp = coneseg(origin,1).head();
	double alp = yseg.angle(stmp);
	forall(a,al) Gal.append(a+alp);

	al = H.alphas();
	stmp = H.coneseg(origin,1).head();
	alp = yseg.angle(stmp);
	forall(a,al) Hal.append(a+alp);

	list_item G_it=Gal.first();
	list_item H_it=Hal.first();

	do {
		if (Hal[H_it] == Gal[G_it]) {
			p=coneseg(origin,C1).head().end();
			G_it = Gal.cyclic_succ(G_it);
			H_it = Hal.cyclic_succ(H_it);
			++C1;
			++C2;
		} else if (Gal[G_it] < Hal[H_it])  {
			p=coneseg(origin,C1).head().end();
			G_it = Gal.cyclic_succ(G_it);
			++C1;
		} else if (Hal[H_it] < Gal[G_it]) {
			p=H.coneseg(origin,C2).head().end();
			H_it = Hal.cyclic_succ(H_it);
			++C2;
		}
		stmp = segment(origin,p);
		stmp = segment(origin,stmp.direction(),1.0);
		result.append(stmp.end());
		if (G_it == Gal.last()) {
			a = Gal.pop();
			Gal.push(a+2*LEDA_PI);
		}
		if (H_it == Hal.last()) {
			a = Hal.pop();
			Hal.push(a+2*LEDA_PI);
		}
	} while ((G_it != Gal.first()) || (H_it != Hal.first()));
	polygauge Gu(result);
	return Gu;
}


polygauge polygauge::l1() {
	list<point> pl;
	pl.append(point(0,1));
	pl.append(point(-1,0));
	pl.append(point(0,-1));
	pl.append(point(1,0));
	polygauge Gl1(pl); 
	return Gl1;
}

polygauge polygauge::linf() {
	list<point> pl;
	pl.append(point(-1,1));
	pl.append(point(-1,-1));
	pl.append(point(1,-1));
	pl.append(point(1,1));
	polygauge Glinf(pl); 
	return Glinf;
}

polygauge polygauge::unit() {
	point origin(0,0);
	list<point> result;
	segment stmp;
	forall(stmp,seg_list) {
		stmp = segment(origin,stmp.angle(),1.0);
		result.append(stmp.end());
	}
	polygauge Gu(result);
	return Gu;
}


polygauge polygauge::translate(const point& r) {
	point p;
	segment stmp;
	list<segment> SL;
	polygauge Gt;
	Gt.alpha_list=alpha_list;
	forall(stmp,seg_list) {
		p = point(stmp.xcoord2()+r.xcoord(),stmp.ycoord2()+r.ycoord());
		stmp = segment(r,p);
		SL.append(stmp);
	}
	Gt.seg_list=SL;
	return Gt;
}

polygauge polygauge::scale(double scale) {
	point p;
	point origin(0,0);
	segment stmp;
	list<point> Vert=vertices(origin);
	list<point> result;
	if (scale <= 0) lola_error("polygauge:scale must be greater than 0",1704,1);
	forall(p,Vert) {
		stmp = segment(origin,p);
		stmp = segment(origin,stmp.direction(),stmp.length()*scale);
		result.append(stmp.end());
	}
	polygauge Gs(result);
	return Gs;
}

bool polygauge::operator == (const polygauge &G) {
	if (seg_list.size() != G.seg_list.size()) return false;
	list_item trav1 = G.seg_list.first();
	for (list_item trav = seg_list.first(); trav; trav = seg_list.succ(trav)) {
		if (seg_list.contents(trav) != G.seg_list.contents(trav1)) return false;
		trav1 = G.seg_list.succ(trav);
	}
	return true;
}

