#include <LOLA/facilities.h> 
#include <LEDA/matrix.h>
#include <iomanip.h>
#include "lpsol.h"

int _SORT_DIRECTION=0;  // Indicator for coordinate-direction for sorting lists of locations.


// ------------------------- global functions ------


void trans_all(list<location> &facs) {
	for(list_item trav = facs.first(); trav; trav = facs.succ(trav))
		facs[trav].transform();
}

void retrans_all(list<location> &facs) {
	for(list_item trav = facs.first(); trav; trav = facs.succ(trav))
		facs[trav].retransform();
}

void trans_all(facilities &facs) {
	for(list_item trav = facs.first(); trav; trav = facs.succ(trav))
		facs[trav].transform();
}

void retrans_all(facilities &facs) {
	for(list_item trav = facs.first(); trav; trav = facs.succ(trav))
		facs[trav].retransform();
}

line mid_perpend(location &l1, location &l2) {
	line result;
	list<point> intsecpnts;
	circle circ1(l1[0], l1[1], (l1-l2).norm(2));
	circle circ2(l2[0], l2[1], (l1-l2).norm(2));
	intsecpnts = circ1.intersection(circ2);
	result = line(intsecpnts.head(),intsecpnts.tail());
	return result;
}

int _COMPARE(const location& generic_pointer1, const location& generic_pointer2)
/*********************************************************************
* Purpose   : Support function for LEDA-sort-algorithm on linear lists
* Returns   : integer -1 or 0 or 1 for less, equal, greater
* Influences: %
* Date      : Jul 95
* Author    : Michael Ochs
**********************************************************************/
{
	if (generic_pointer1[_SORT_DIRECTION] < generic_pointer2[_SORT_DIRECTION]) return -1;
	else if (generic_pointer1[_SORT_DIRECTION] == generic_pointer2[_SORT_DIRECTION]) return 0;
	else return 1;
}

void _TRANSFORM_LOCATIONS(location& generic_pointer)
/********************************************************************
* Purpose   : Rotate a location by 45 degree in planar real space
* Returns   : %
* Influences: Changes the coordinates of given location
* Date      : Aug 95
* Author    : Michael Ochs
*********************************************************************/
{
	location temp(2,generic_pointer.wht_dim());
	temp[0]=0.5*(generic_pointer[0]+generic_pointer[1]);
	temp[1]=0.5*(generic_pointer[1]-generic_pointer[0]);
	for(int i=0;i<generic_pointer.wht_dim();i++) 
		temp(i)=generic_pointer(0);
	generic_pointer=temp;
}

void _RE_TRANSFORM_LOCATIONS(location& generic_pointer)
/********************************************************************
* Purpose   : Inverse function to the above one
* Returns   : %
* Influences: Changes the coordinates of given location
* Date      : Aug 95
* Author    : Michael Ochs
*********************************************************************/
{
	location temp(2,generic_pointer.wht_dim());
	temp[0]=generic_pointer[0]-generic_pointer[1];
	temp[1]=generic_pointer[0]+generic_pointer[1];
	for(int i=0;i<generic_pointer.wht_dim();i++) 
		temp(i)=generic_pointer(0);
	generic_pointer=temp;
}

// ------------- member functions
bool facilities::element(location &Loc) {
/************************************************************
* Purpose    : Returns true if Loc is element of facilities; 
*              else false
* Returns    : %
* Influences : %
* Date       : Oct 97
* Author     : Ingo Bartling
* Last Change: Jan 99 (Holger Hennes)
*************************************************************/
	location tmp;
	forall(tmp,*this)
		if (tmp.pos() == Loc.pos()) return true;
	return false;
}

location facilities::get_element(int idx) {
/************************************************************
* Purpose   : returns the location with index idx
* Returns   : %
* Influences: %
* Date      : Oct 97
* Author    : Ingo Bartling
*************************************************************/
	location found;
	if (this->size() > 0 && idx >= 0 && idx < this->size()) {
		list_item current_element = this->first();
		for (int i = 0; i < idx; i++)
		  current_element = this->succ(current_element);
		found = this->contents(current_element);
	}
	return found;
}

int facilities::get_idx(location &L) {
/************************************************************
* Purpose   : Returns the index of L in facilities
* Returns   : %
* Influences: %
* Date      : Oct 97
* Author    : Ingo Bartling
*************************************************************/
	int idx = 0, N = this->size();
	list_item current_element = this->first();
	location compare = this->contents(current_element);
	for (idx = 0; idx < N; idx++)
		if (compare == L) return idx;
		else current_element = this->succ(current_element);
	return idx;
}

void facilities::remove(location &L) {
/************************************************************
* Purpose   : Removes L from facilities
* Returns   : %
* Influences: %
* Date      : Oct 97
* Author    : Ingo Bartling
*************************************************************/
	location trans = L;
	facilities newfacs;
	if (L == this->head()) this->pop();
	else if (L == this->tail()) this->Pop();
	else {
		forall(trans,*this)
			if (trans != L) newfacs.append(trans);
		*this = newfacs;
	}
}

void facilities::remove(int idx) {
/************************************************************
* Purpose    : Removes the idx-location from facilities
* Returns    : %
* Influences : %
* Date       : Oct 97
* Author     : Ingo Bartling
* Last Change: Jan 99 (Holger Hennes)
*************************************************************/
	if (idx >= 0 && idx < this->size()) {
		if (idx == 0) this->pop();
		else if (idx == (this->size())-1) this->Pop();
		else {
			location trans = this->head();
			int cnt = 0;
			facilities newfacs;
			forall(trans,*this) {
				if (cnt != idx) newfacs.append(trans);
				cnt++;
			}
			*this = newfacs;
		}
	}
	else lola_error("facilities::remove(): There is no corresponding location for idx",1305,1); 
}


string &facilities::refname(list_item location)
/***********************************************************************
 * Purpose     : give r/w access to the names of locations
 * Side-effect : pads LocTxt with empty strings if too short
 * Returns     : reference to the name of the location
 * Influences  : %
 * Date        : Aug 1997
 * Author      : Martin C. Mueller
 **********************************************************************/
{
	list_item loc_it, txt_it;

	for(loc_it=first(), txt_it=LocTxt.first();
		loc_it;
		loc_it = succ(loc_it), txt_it = LocTxt.succ(txt_it)) {
			if (!txt_it) { txt_it = LocTxt.append(""); }
			if (loc_it == location) break;
	}
	if (!loc_it) lola_error("facilities::refname(): location isn't an item of this object",1334,1);
	return LocTxt[txt_it];
} // string &facilities::refname(list_item location)

string &facilities::refname(int location)
/***********************************************************************
 * Purpose     : give r/w access to the names of locations
 * Side-effect : pads LocTxt with empty strings if too short
 * Returns     : reference to the name of the location
 * Influences  : %
 * Date        : Aug 1997
 * Author      : Martin C. Mueller
 **********************************************************************/
{
	int pos;
	list_item loc_it, txt_it;

	if (location > size()) lola_error("facilities::refname(): location out of range",1335,1);
	for(pos = 0, loc_it = first(), txt_it = LocTxt.first();
	pos < location;
	pos++, loc_it = succ(loc_it), txt_it = LocTxt.succ(txt_it)) 
	{
		if (!txt_it) { txt_it = LocTxt.append(""); }
	}
	return LocTxt[txt_it];
} // string facilities::refname(int location)

void facilities::set_name(list_item location, const string &name)
/***********************************************************************
 * Purpose   : name a location
 * Returns   : reference to the name of the location
 * Influences: %
 * Date      : Aug 1997
 * Author    : Martin C. Mueller
 **********************************************************************/
{
	this->refname(location) = name;
}

void facilities::set_name(int location, const string &name)
/***********************************************************************
 * Purpose   : name a location
 * Returns   : reference to the name of the location
 * Influences: %
 * Date      : Aug 1997
 * Author    : Martin C. Mueller
 **********************************************************************/
{
	this->refname(location) = name;
}

string facilities::name(list_item location)
/***********************************************************************
 * Purpose   : give read-only access to the names of locations
 * Returns   : reference to the name of the location
 * Influences: %
 * Date      : Aug 97
 * Author    : Martin C. Mueller
 **********************************************************************/
{
	return this->refname(location);
}

string facilities::name(int location)
/***********************************************************************
 * Purpose   : give read-only access to the names of locations
 * Returns   : reference to the name of the location
 * Influences: %
 * Date      : Aug 97
 * Author    : Martin C. Mueller
 **********************************************************************/
{
	return this->refname(location);
}


facilities facilities::collect(int direction,int weight)
/********************************************************************
* Purpose   : Cumulate weights of locations with same coordinate 
*             in direction 'direction'. Member of CLASS FACILITIES.
* Returns   : Pointer to facilities
* Influences: %
* Date      : Jul 95
* Author    : Michael Ochs
*********************************************************************/
{
	list_item  generic_pointer;
	location   generic_loc1;
	location   generic_loc2;
	facilities collected; 

	collected.clear();
	collected.append(head());
	for(generic_pointer=succ(first()); generic_pointer; generic_pointer=succ(generic_pointer))
	{
		if (generic_pointer!=nil)
		{ 
			generic_loc1=contents(generic_pointer);
			generic_loc2=collected.tail();
			if (generic_loc1[direction]==generic_loc2[direction]) 
			{
				generic_loc2(weight) += generic_loc1(weight);
				collected.assign(collected.last(),generic_loc2);
			}
			else
			{
				collected.append(generic_loc1);
			}
		} 
	}
	return collected;
}   

/*
int facilities::weiszfeld_test(list_item generic_pointer)
/********************************************************************
* Purpose   : Test an existing location for optimality 
* Returns   : integer 0 or 1 if weiszfeld_test failed or not
* Influences: %
* Date      : Aug 95
* Author    : Michael Ochs
*********************************************************************
{
 list_item pointer_to_location;
 double    test_sum_0=0;
 double    test_sum_1=0;
 double    test_value=0;
 location  generic_loc;
 location  generic_loc_from_list;

 generic_loc=contents(generic_pointer);
 for(pointer_to_location=first(); pointer_to_location; pointer_to_location=succ(pointer_to_location))
  if (generic_pointer != pointer_to_location)
  { 
   generic_loc_from_list=contents(pointer_to_location);
   test_sum_0+=generic_loc_from_list(0)*(generic_loc[0]-generic_loc_from_list[0])/(generic_loc-generic_loc_from_list).norm(2);
   test_sum_1+=generic_loc_from_list(0)*(generic_loc[1]-generic_loc_from_list[1])/(generic_loc-generic_loc_from_list).norm(2);
  }
 
 test_value=pow((pow(test_sum_0,2.0)+pow(test_sum_1,2.0)),0.5);
 
 return (test_value <= generic_loc(0) ? 0 : 1);
} 
*/

int facilities::weiszfeld_test(location& test_loc)
/********************************************************************
* Purpose   : Test an existing location for optimality 
* Returns   : 0 if non is optimal, else 1 and save opt. loc. in 'test_loc'
* Influences: %
* Date      : Jul 99
* Author    : Holger Hennes
*********************************************************************/
{
	return weiszfeld_test(test_loc, 0);
}

int facilities::weiszfeld_test(location& test_loc, int weight)
/********************************************************************
* Purpose    : Test an existing location for optimality 
* Returns    : 0 if non is optimal, else 1 and save opt. loc. in 'test_loc'
* Influences : %
* Date       : Feb 99
* Author     : Jrg Kalcsics
* Last Change: Jul 99 (Holger Hennes)
*********************************************************************/
{
	double    sum_0=0,sum_1=0,help;
	location  loc_tmp;

	forall(loc_tmp,*this) {
		help=(test_loc-loc_tmp).norm(2);
		if(help>0.001) {
			sum_0+=loc_tmp(0)*(test_loc[0]-loc_tmp[0])/help;
			sum_1+=loc_tmp(0)*(test_loc[1]-loc_tmp[1])/help;
		}
	}
	help=L_sqr(sum_0)+L_sqr(sum_1);

	if(help <= L_sqr(test_loc(weight)))
		return 1;

	return 0;
}

int facilities::generalized_weiszfeld_test(list_item generic_pointer,int p,int weight)
/********************************************************************
* Purpose    : Test an existing location for optimality in p-mteric 
* Returns    : integer 0 or 1 if generalized_weiszfeld_test failed
*              or not
* Influences : %
* Date       : Aug 95
* Author     : Michael Ochs
* Last Update: May 99 (Holger Hennes)
*********************************************************************/
{
	list_item pointer_to_location;
	double    test_sum_0=0;
	double    test_sum_1=0;
	double    test_value=0;
	location  generic_loc(2,_Q());
	location  generic_loc_from_list(2,_Q());

	generic_loc=contents(generic_pointer);
	for(pointer_to_location=first(); pointer_to_location; pointer_to_location=succ(pointer_to_location))
		if (generic_pointer != pointer_to_location)
		{ 
			generic_loc_from_list=contents(pointer_to_location);
			test_sum_0+=generic_loc_from_list(weight)*(pow(fabs(generic_loc[0]-generic_loc_from_list[0]),p-1.0) * L_signum(generic_loc[0]-generic_loc_from_list[0]))/pow((generic_loc-generic_loc_from_list).norm(p),p-1.0);
			test_sum_1+=generic_loc_from_list(weight)*(pow(fabs(generic_loc[1]-generic_loc_from_list[1]),p-1.0) * L_signum(generic_loc[1]-generic_loc_from_list[1]))/pow((generic_loc-generic_loc_from_list).norm(p),p-1.0);
		}
		
	test_value= pow ( (pow( fabs(test_sum_0),(p/(p-1.0)) ) + pow( fabs(test_sum_1),(p/(p-1.0)) )), (p-1.0)/p );

	return (test_value <= generic_loc(weight) ? 1 : 0);
}

location facilities::generalized_weiszfeld_iteration(location old_approx, int p, double delta,int weight)
/********************************************************************
* Purpose    : Compute one iteration step of the generalized 
*             Wieszfeld_Algorithm
* Returns    : new approxiamtion of the optimum as 'location'
* Influences : %
* Date       : Aug 95
* Author     : Michael Ochs
* Last Change: June 99 (Holger Hennes)
*********************************************************************/
{
	location  new_approx(2,1);
	location  temp(2,_Q());
	list_item p_to_loc;
	double    nenner[2];
	double    sum_zaehler=0;
	double    sum_nenner=0;
	double    sub1_1;
	double    sub1_2;
	double    sub2_1;
	double    sub2_2;
	double    sub3_1;
	double    sub3_2;
	double    sub4;

	int       k;

	new_approx(0)=0.0;

	for(k = 0; k < 2; k++) {
		sum_zaehler=0;
		sum_nenner =0;
		for(p_to_loc = first(); p_to_loc; p_to_loc = succ(p_to_loc)) {
			temp = contents(p_to_loc);

//			sub1_1 = pow( (temp[0]-old_approx[0]), 2.0);
			sub1_1 = L_sqr(temp[0]-old_approx[0]);
			sub1_2 = pow( (sub1_1+delta), (p/2.0));

//			sub2_1 = pow( (temp[1]-old_approx[1]), 2.0);
			sub2_1 = L_sqr(temp[1]-old_approx[1]);
			sub2_2 = pow( (sub2_1+delta), (p/2.0));

//			sub3_1 = pow( (temp[k]-old_approx[k]), 2.0);
			sub3_1 = L_sqr(temp[k]-old_approx[k]);
			sub3_2 = pow( (sub3_1+delta), (1-(p/2.0)));

			sub4   = pow( (sub1_2+sub2_2), (1-(1.0/p)));
			nenner[k] = sub4*sub3_2;
     

			sum_zaehler += temp(weight)*temp[k]/nenner[k];
			sum_nenner  += temp(weight)/nenner[k];

		}
		new_approx[k]=sum_zaehler/sum_nenner;
	}

	return new_approx;
}

Loc_Vector facilities::total_weight() {
	Loc_Vector result(contents(first()).wht_dim());
	for (list_item tmp = first(); tmp; tmp = succ(tmp))
		result += contents(tmp).wht();
	return result;
}

void facilities::Write() {
	this->Write(cout);
}

void facilities::Write(ostream &out)
/********************************************************************
* Purpose   : Print all facilities
* Returns   : %
* Influences: ostream
* Date      : Jul 95
* Author    : Michael Ochs
*********************************************************************/
{
	list_item pointer_to_location;
	location dummy;
	for(pointer_to_location=first(); pointer_to_location; pointer_to_location=succ(pointer_to_location))
	{
		dummy=contents(pointer_to_location);
		out << dummy << endl;
	}
}

double facilities::n_median_objective(int p,int weight)
/********************************************************************
* Purpose   : Compute value of a n-median objective function
* Returns   : double
* Influences: %
* Date      : Oct 95
* Author    : Michael Ochs
*********************************************************************/
{
	if (solution.size()==0) lola_error("Solve a problem first.",1301,1);
	
	location  search(_d(),_Q());
	list_item pointer_to_optimum;

	for(pointer_to_optimum=solution.first();pointer_to_optimum;pointer_to_optimum=solution.succ(pointer_to_optimum))
	{
		search=solution.contents(pointer_to_optimum);
		if (search(0)==weight) return median_objective_comp(p,search,weight);
	}
	
	lola_error("Indetermined weight criterion. Check number of weights.",1302,1);
	return 0;
}

double facilities::median_objective(int p,int weight)
/********************************************************************
* Purpose   : Compute value of a median objective function
* Returns   : double
* Influences: %
* Date      : Jul 95
* Author    : Michael Ochs
*********************************************************************/
{
	if (solution.size()==0) lola_error("Solve a problem first.",1303,1);
	location  optimum(_d(),_Q());

	optimum=solution.head();

	return median_objective_comp(p,optimum,weight);
}

double facilities::median_objective_comp(int p,location optimum,int weight)
/********************************************************************
* Purpose   : Compute value of a median objective function for a 
	      special location
* Returns   : double
* Influences: %
* Date      : Dec 95
* Author    : Michael Ochs
*********************************************************************/
{
	list_item pointer_to_location;
	location  temp(_d(),_Q());
	double output=0;

	for(pointer_to_location=first(); pointer_to_location; pointer_to_location=succ(pointer_to_location))
	{
		temp=contents(pointer_to_location);
		output += temp(weight) * (temp.pos()-optimum.pos()).norm(p);
	}
	return output; 
}

void facilities::WriteOpt() {
	 this->WriteOpt(cout);
}
  
void facilities::WriteOpt(ostream &out)
/********************************************************************
* Purpose   : Print the set of optimal solutions
* Returns   : %
* Influences: ostream
* Date      : Jul 95
* Author    : Michael Ochs
*********************************************************************/
{ 
	if (solution.size()==0) lola_error("Solve a problem first.",1304,1);

	location::WriteAsPolygonList(out, solution);
} 

double facilities::max_direction_value(int i)
/********************************************************************
* Purpose   : Compute maximum of all locations in coord-direction i
* Returns   : double (maximum)
* Influences: %
* Date      : Aug 95
* Author    : Michael Ochs
*********************************************************************/
{
	double    maximum;
	location  tmp(_d(),_Q());
	list_item pointer_to_location;
	
	tmp=head();
	maximum=tmp[i];
	for(pointer_to_location=first();pointer_to_location;pointer_to_location=succ(pointer_to_location))
	{ 
		tmp=contents(pointer_to_location);
		if (tmp[i]>maximum) maximum=tmp[i];
	}
	return maximum;
}

double facilities::min_direction_value(int i)
/********************************************************************
* Purpose   : Compute minimum of all location in coord-direction i
* Returns   : double (minimum)
* Influences: %
* Date      : Aug 95
* Author    : Michael Ochs
*********************************************************************/
{ 
	double    minimum;
	location  tmp(_d(),_Q());
	list_item pointer_to_location;

	tmp=head();
	minimum=tmp[i];
	for(pointer_to_location=first();pointer_to_location;pointer_to_location=succ(pointer_to_location))
	{ 
		tmp=contents(pointer_to_location);
		if (tmp[i]<minimum) minimum=tmp[i];
	}
	return minimum;
}

double facilities::diff_value(int i)
/********************************************************************
* Purpose   : Compute difference between minimum and maximimum 
*             of all location in coord-direction i
* Returns   : double (diff)
* Influences: %
* Date      : Sep 95
* Author    : Lothar Schnell
*********************************************************************/
{ 
	return max_direction_value(i)-min_direction_value(i);
}

double facilities::l2sqr_sum(int weight,bool erase)
/********************************************************************
* Purpose    : Compute the optimum for problem class 1|P|.|l2sqr|sum
* Returns    : double (objective value of solution)
* Influences : list<location> solution
* Date       : Jul 95
* Author     : Michael Ochs
* Last Change: June 99
*********************************************************************/
{
	double    zaehler1=0;
	double    zaehler2=0;
	double    nenner=0;
	double    objective=0;
	list_item pointer_to_location;
	location  temp(_d(),_Q());
	location  optimum(2,1);

	if (erase) solution.clear();

	for(pointer_to_location=first(); pointer_to_location ; pointer_to_location=succ(pointer_to_location))
	{
		temp[0]=contents(pointer_to_location)[0];
		temp[1]=contents(pointer_to_location)[1];
		temp(0)=contents(pointer_to_location)(weight);
		zaehler1 += temp[0]*temp(0);
		zaehler2 += temp[1]*temp(0);
		nenner   += temp(0);
	}
	optimum[0]=zaehler1/nenner;
	optimum[1]=zaehler2/nenner;
	optimum(0)=weight;
	solution.append(optimum);
	for(pointer_to_location=first(); pointer_to_location; pointer_to_location=succ(pointer_to_location))
	{
		temp=contents(pointer_to_location);
//		objective += temp(0) * pow((temp.pos()-optimum.pos()).norm(2),2.0);
		objective += temp(0) * L_sqr((temp.pos()-optimum.pos()).norm(2));
	}
	return objective;
}

double facilities::l1_sum(int weight, bool erase)
/********************************************************************
* Purpose    : Compute the optimum for problem class 1|P|.|l1|sum
* Returns    : double (objective value of solution)
* Influences : list<location> solution
* Date       : Aug 95
* Author     : Michael Ochs
* Last Change: Feb 98 (Holger Hennes)
*********************************************************************/
{
	facilities temp0(*this);
	facilities temp1(*this);
	list_item  pointer_to_location;
	list_item  optimum0;
	list_item  optimum1;
	double     allsum=0;
	double     optsum_direction_0=0;
	double     optsum_direction_1=0;
	location   temp(_d(),_Q());
	location   optimum(2,1);

	if (erase) solution.clear();
	_SORT_DIRECTION=0;
	temp0.sort(_COMPARE);
	_SORT_DIRECTION=1;
	temp1.sort(_COMPARE);
	temp0=temp0.collect(0,weight);
	temp1=temp1.collect(1,weight);
	for(pointer_to_location=temp0.first(); pointer_to_location; pointer_to_location=temp0.succ(pointer_to_location))
		allsum += temp0.contents(pointer_to_location)(weight);
	if (!allsum) lola_message("facilities::l1_sum(): All weights are equal zero:The whole plane is optimal",0);
	pointer_to_location=temp0.first();
	do
	{
		temp=temp0.contents(pointer_to_location);
		optsum_direction_0 += temp(weight);
		if (optsum_direction_0 < (0.5*allsum)) pointer_to_location=temp0.succ(pointer_to_location);
	} while (optsum_direction_0 < (0.5*allsum));
	optimum0=pointer_to_location;
	pointer_to_location=temp1.first();
	do
	{
		temp=temp1.contents(pointer_to_location);
		optsum_direction_1 += temp(weight);
		if (optsum_direction_1 < (0.5*allsum)) pointer_to_location=temp1.succ(pointer_to_location);
	} while (optsum_direction_1 < (0.5*allsum));
	optimum1=pointer_to_location; 

	optimum(0)=weight;

	/* Construction of the set of optimal solutions to problem class 1|P|.|l1|sum
	**
	** First we check if it is a single point */
	if ((optsum_direction_0 > (0.5*allsum)) && (optsum_direction_1 > (0.5*allsum)))
	{
		/* Push the coordinates of the optimum to 'list<location> solution' with default weight 1 */ 

		temp=temp0.contents(optimum0);
		optimum[0]=temp[0];
		temp=temp1.contents(optimum1);
		optimum[1]=temp[1];
		solution.append(optimum);
	}

	/* Wasn't ?
	** Maybe it's a line segment parallel to the y-axis ? */ 

	else if ((optsum_direction_0 > (0.5*allsum)) && (optsum_direction_1 = (0.5*allsum)))
	{
		/* Push the coordiantes of the first point of optimal line segment to 'list<location> solution' */

		temp=temp0.contents(optimum0);
		optimum[0]=temp[0];
		temp=temp1.contents(optimum1);
		optimum[1]=temp[1];
		//  optimum(0)=weight;
		solution.append(optimum);

  /* Get the other point of the line segment by taking the successor coordinate in direction 1(y-axis)
  ** and push it to 'list<location> solution'. */
		do {
			optimum1=temp1.succ(optimum1);
			temp=temp1.contents(optimum1);
			optimum[1]=temp[1];
		}
		while (temp(0)==0); 
		solution.append(optimum);
	}

	/* Wasn't ?
	** Maybe it's a line segment parallel to the x-axis ? */

	else if ((optsum_direction_0 = (0.5*allsum)) && (optsum_direction_1 > (0.5*allsum)))
	{
		/* Push the coordiantes of the first point of optimal line segment to 'list<location> solution' */

		temp=temp0.contents(optimum0);
		optimum[0]=temp[0];
		temp=temp1.contents(optimum1);
		optimum[1]=temp[1];
		//  optimum(0)=weight;
		solution.append(optimum);

		/* Get the other point of the line segment by taking the successor coordinate in direction 1(x-axis)
		** and push it to 'list<location> solution'. */
		do {
			optimum0=temp0.succ(optimum0);
			temp=temp0.contents(optimum0);
			optimum[0]=temp[0];
		}
		while (temp(0)==0); 
		solution.append(optimum);
	}
	else

	/* Wasn't ?
	** Then there can be only one: A hatched rectangle ! */

	{ 
		/* Get first point of hatched rectangle and push it to 'list<location> solution' */

		temp=temp0.contents(optimum0);
		optimum[0]=temp[0];
		temp=temp1.contents(optimum1);
		optimum[1]=temp[1];
		//  optimum(0)=weight;
		solution.append(optimum);

		/* Get second point of h.r. on equal y-coordinate (direction 1) and push ...     */

		do {
			optimum0=temp0.succ(optimum0);
			temp=temp0.contents(optimum0);
			optimum[0]=temp[0];
		}
		while (temp(0)==0);
		solution.append(optimum);

		/* Get third point of h.r. on equal x-coordinate (direction 0) and push ...      */
		do {
			optimum1=temp1.succ(optimum1);
			temp=temp1.contents(optimum1);
			optimum[1]=temp[1];
		}
		while (temp(0)==0);
		solution.append(optimum);

		/* Get last point of h.r. on equal y-coordinate (direction 1) and push ...       */

		do {
			optimum0=temp0.pred(optimum0);
			temp=temp0.contents(optimum0);
			optimum[0]=temp[0];
		}
		while (temp(0)==0);
		solution.append(optimum);
	} 

	return median_objective(1,weight);
}

double facilities::linf_sum(int weight,bool erase)
/********************************************************************
* Purpose   : Compute the optimum for problem class 1|P|.|linf|sum
* Returns   : double (objective value of solution)
* Influences: list<location> solution
* Date      : Aug 95
* Author    : Michael Ochs
*********************************************************************/
{
	double objective=0;

	if (erase) solution.clear();
	facilities facs=*this;
	trans_all(facs);
	facs.l1_sum(weight,false);
	solution = facs.solution;
	retrans_all(solution);
	objective=median_objective(L_inf,weight);
	return objective;
}  

/*
double facilities::l2_sum(int weight,double epsilon,bool erase)
/********************************************************************
* Purpose   : Compute the optimum for problem class 1|P|.|l2|sum
	      ( Weiszfeld-Algorithm )
* Returns   : double (objective value of solution)
* Influences: list<location> solution
* Date      : Aug 95
* Author    : Michael Ochs
*********************************************************************
{
 list_item pointer_to_location;
 location  iteration_base(_d(),1);
 location  iteration_new(_d(),1); 
 location  temp(_d(),_Q());
 location  temp_opt(_d(),1);
 double    sum_d0=0;
 double    sum_d1=0;
 double    sum_both=0;
 double    zaehler1=0;
 double    zaehler2=0;
 double    nenner=0;

 if (erase) solution.clear();

 for(pointer_to_location=first(); pointer_to_location; pointer_to_location=succ(pointer_to_location))
 {
  if (weiszfeld_test(pointer_to_location)==0) 
  { 
   temp=contents(pointer_to_location);
   temp_opt(0)=weight;
   temp_opt[0]=temp[0];
   temp_opt[1]=temp[1];
   solution.append(temp_opt);
   return median_objective(2,weight); 
  }  
 }
 
 for(pointer_to_location=first(); pointer_to_location ; pointer_to_location=succ(pointer_to_location))
 {
  temp=contents(pointer_to_location);
  zaehler1 += temp[0]*temp(0);
  zaehler2 += temp[1]*temp(0);
  nenner   += temp(weight);
 }

 temp_opt[0]=zaehler1/nenner;
 temp_opt[1]=zaehler2/nenner;
 
 iteration_new=temp_opt;
 do
   {
    iteration_base=iteration_new;
    for(pointer_to_location=first(); pointer_to_location; pointer_to_location=succ(pointer_to_location))
      {
       temp=contents(pointer_to_location);
       sum_d0   += temp(weight)*temp[0]/(temp.pos()-iteration_base.pos()).norm(2);
       sum_d1   += temp(weight)*temp[1]/(temp.pos()-iteration_base.pos()).norm(2);
       sum_both += temp(weight)/(temp.pos()-iteration_base.pos()).norm(2);
      }
    iteration_new[0]=sum_d0/sum_both;
    iteration_new[1]=sum_d1/sum_both;
   } while ((iteration_new-iteration_base).norm(2) >= epsilon);
 
 iteration_new(0)=weight;
 solution.append(iteration_new);
 return median_objective(2,weight);
}
*/

double facilities::l2_sum(int weight,double epsilon,bool erase)
/********************************************************************
* Purpose    : Compute the optimum for problem class 1|P|.|l2|sum
	      ( Weiszfeld-Algorithm )
* Returns    : double (objective value of solution)
* Influences : list<location> solution
* Date       : Aug 95
* Author     : Michael Ochs
* Last Update: Jrg Kalcsics, Jan 99
*********************************************************************/
{
	int iteration_counter=1,i=0;
	location iteration_base(_d(),1),iteration_new(_d(),1);
	location temp(_d(),_Q()),temp_opt(_d(),1);
	double sum_d0=0,sum_d1=0,sum_both=0,zaehler1=0,zaehler2=0,nenner=0,div;
	array<bool> is_tested(this->size());
	//  is_tested.init(false);
	for (i=0;i<size();i++) is_tested[i]=false;
	if (erase) solution.clear();
	// compute starting point: center-of-gravity
	forall(temp,*this) {
		zaehler1 += temp[0]*temp(weight);
		zaehler2 += temp[1]*temp(weight);
		nenner   += temp(weight);
	}
	temp_opt[0]=zaehler1/nenner;
	temp_opt[1]=zaehler2/nenner;
	temp_opt(0)=0;


	// iteration; if l2(temp,iteration_base)<0.1 then check temp for optimality. If
	// temp is not optimal, continue iteration
	iteration_new=temp_opt;
	do {
		sum_d0=0; sum_d1=0; sum_both=0; i=0;
		iteration_base=iteration_new;
		forall(temp,*this) {
			div=(temp.pos()-iteration_base.pos()).norm(2);
			if(div < 0.1) {
				if(!is_tested[i]) {
					is_tested[i]=true;
					if(weiszfeld_test(temp,weight)==1) {            // temp is optimal
						temp(0)=weight; solution.append(temp);
						return median_objective(2,weight);
					}
				}
			}
			sum_d0   += temp(weight)*temp[0]/div;
			sum_d1   += temp(weight)*temp[1]/div;
			sum_both += temp(weight)/div;
			i++;
		}
		iteration_new[0]=sum_d0/sum_both;
		iteration_new[1]=sum_d1/sum_both;
	}while((iteration_new-iteration_base).norm(2) >= epsilon);

	iteration_new(0)=weight;
	solution.append(iteration_new);
	return median_objective(2,weight);
}

double facilities::Weiszfeld(double epsilon)
/********************************************************************
* Purpose   : Compute the optimum for problem class 1|P|.|l2|sum
*             ( Weiszfeld-Algorithm )
* Returns   : double (objective value of solution)
* Influences: list<location> solution
* Date      : Aug 95
* Author    : Michael Ochs
*********************************************************************/
{
	return l2_sum(0,epsilon,true);
}

double facilities::lp_sum(int p,double epsilon,double delta,int iter_max,int weight,bool erase)
/********************************************************************
* Purpose   : Compute the optimum for problem class 1|P|.|lp|sum
*             ( Generalized Weiszfeld-Algorithm )
* Returns   : double (objective value of solution)
* Influences: list<location> solution
* Date      : Aug 95, Sep 95(Iteration-bug fixed.)
* Author    : Michael Ochs
*********************************************************************/
{
	list_item    pointer_to_location;
	location     iteration_base(_d(),1);
	location     iteration_new(_d(),1); 
	location     temp(_d(),_Q());
	double       zaehler1=0;
	double       zaehler2=0;
	double       nenner=0;
	register int iter_count=0;

	if (erase) solution.clear();

	for(pointer_to_location=first(); pointer_to_location; pointer_to_location=succ(pointer_to_location))
	{
		if (generalized_weiszfeld_test(pointer_to_location,p,weight)==1) 
		{ 
			temp=contents(pointer_to_location);
			temp(0)=weight;
			solution.append(temp);
			return median_objective(p,weight);  
		}  
	}

	for(pointer_to_location=first(); pointer_to_location ; pointer_to_location=succ(pointer_to_location))
	{
		temp=contents(pointer_to_location);
		zaehler1 += temp[0]*temp(weight);
		zaehler2 += temp[1]*temp(weight);
		nenner   += temp(weight);
	}
	iteration_new[0]=zaehler1/nenner;
	iteration_new[1]=zaehler2/nenner;
	
	do
	{
		iteration_base=iteration_new;
		iteration_new=generalized_weiszfeld_iteration(iteration_base,p,delta,weight);
		iter_count++;
		if (iter_count==iter_max) break;
	} while ((iteration_new-iteration_base).norm(p) >= epsilon); 

	iteration_new(0)=weight;
	if (iter_count==iter_max)
	{
		iteration_base(0)=weight;
		double value_new=median_objective_comp(p,iteration_new,weight);
		double value_base=median_objective_comp(p,iteration_base,weight);
		if (value_base<value_new) {
			solution.append(iteration_base);
			return value_base;
		}
		else
		{
			solution.append(iteration_new);
			return value_new;
		}
	}
	else
    {
		solution.append(iteration_new);
		return median_objective(p,weight);
    }
}

double facilities::fn_sum(int n,int p) 
/********************************************************************
* Purpose   : Compute the objective value of X_n regarding all 
*             existing facilities 
* Returns   : double
* Influences: %
* Date      : Oct 1995
* Author    : Michael Ochs
*********************************************************************/
{
	list_item pointer_to_location, pointer_to_optimum;
	double    objective=0; 
	location  Xn(_d(),1);
	location  temp(_d(),_Q());
 
	pointer_to_optimum=solution[n];
	Xn=solution.contents(pointer_to_optimum); 
	for(pointer_to_location=first();pointer_to_location;pointer_to_location=succ(pointer_to_location)) {
		temp=contents(pointer_to_location);
		objective += temp(n)*((temp.pos()-Xn.pos()).norm(p));
	}
	return objective; 
}

double facilities::fex_sum(int p)
/********************************************************************
* Purpose   : Compute the objective value of all solutions regarding
	      all existing facilities 
* Returns   : double
* Influences: %
* Date      : Oct 1995
* Author    : Michael Ochs
*********************************************************************/
{
	register int n;
	double       objective=0; 

	for(n=0;n<solution.length();n++) objective += fn_sum(n,p);
	return objective;
}

double facilities::fnew_sum(matrix& w,int p)
/********************************************************************
* Purpose   : Compute the objective value of the X_ns to each other  
* Returns   : double
* Influences: %
* Date      : Oct 1995
* Author    : Michael Ochs
*********************************************************************/
{
	double       objective=0;
	register int l;
	register int k;
	location     templ(_d(),1);
	location     tempk(_d(),1);

	for(l=0; l < solution.length(); l++)
		for(k=l+1; k < solution.length(); k++) {
			templ=solution.contents(solution[l]);
			tempk=solution.contents(solution[k]);
			objective += w(l,k)*(templ.pos()-tempk.pos()).norm(p);
		}
 
	return objective; 
}

double facilities::l2sqr_qsum(bool erase)
/********************************************************************
* Purpose   : Compute the optimum for problem class 1|P|.|l2sqr|qsum
* Returns   : objective values(double)
* Influences: list<location> solution
* Date      : Mai 96
* Author    : Lothar Schnell
*********************************************************************/
{
	double objective=0;
	point p;
	list<point> inter_sol;
	int qcrit=head().wht_dim();
	register int i;

	if (erase) solution.clear();

	for(i=0;i<qcrit;i++) {
		solution.clear();
		l2sqr_sum(i,false);
		inter_sol.append(solution.head().loc2point());
	}
	solution.clear();
	list<point> C = CONVEX_HULL(inter_sol);
	forall(p,inter_sol) 
		solution.append(point2loc(p));
	return objective; 
}

double facilities::l1_2sum(bool erase)
/********************************************************************
* Purpose   : Compute the optimum for problem class 1|P|.|l1|2-sum
* Returns   : objective values(double)
* Influences: %
* Date      : Mar 98
* Author    : Holger Hennes
*********************************************************************/
{
	double objective=0;
	list<polygauge> LG;
	list<point> poi;
	point temp(1,0); poi.append(temp);
	temp=point(0,1); poi.append(temp);
	temp=point(-1,0); poi.append(temp);
	temp=point(0,-1); poi.append(temp);
	LG.append(polygauge(poi));
	location loc;
	list<int> Ln;
	forall(loc,*this) Ln.append(0);
	objective=gauge_bi_crit_sum(LG,Ln,erase);
	return objective;
}

double facilities::linf_2sum(bool erase)
/********************************************************************
* Purpose   : Compute the optimum for problem class 1|P|.|linf|2-sum
* Returns   : objective values(double)
* Influences: %
* Date      : Mar 98
* Author    : Holger Hennes
*********************************************************************/
{
	double objective=0;
	list<polygauge> LG;
	list<point> poi;
	point temp(1,1); poi.append(temp);
	temp=point(-1,1); poi.append(temp);
	temp=point(-1,-1); poi.append(temp);
	temp=point(1,-1); poi.append(temp);
	LG.append(polygauge(poi));
	location loc;
	list<int> Ln;
	forall(loc,*this) Ln.append(0);
	objective=gauge_bi_crit_sum(LG,Ln,erase);
	return objective;
}


double* facilities::N_lp_sum_v1(int N, int p, matrix& w,bool erase)
/********************************************************************
* Purpose    : Compute the optimum for problem class N|P|.|.|sum
* Returns    : pointer to array of objective values(double)
* Influences : list<location> solution
* Date       : Oct 95
* Author     : Michael Ochs
* Last Change: Feb 98 (Holger Hennes)
*********************************************************************/
{
	double  delta;
	double  lastweight;

	list<location> dummy;
	location TRANS(2,1);
	list_item gen_ptr;

	int Q=head().wht_dim();
	int M1=w.dim1();
	int M2=w.dim2();

	register int i;

	string mesg_txt=string("facilities::N_lp_sum_v1(): Matrix of interdependencies is non-quadratic: We are taking the upperleft submatrix of dimension %i in account",Q);
	if (M1!=M2) lola_message(mesg_txt,1);
	if (M1>M2) M1=M2;
	if ((N>Q) || (N>M1) || (M1>Q)) lola_error("Cannot compute more optima than there is provided data.",1306,1);

	if (erase) solution.clear();

	switch(p) {
	case 1     : for(i=0;i<N;i++) l1_sum(i,false); break;
	case 2     : for(i=0;i<N;i++) l2_sum(i,0.0001,false); break;
	case L_max : 
		{
			facilities facs = *this;
			facs.solution.clear();
			trans_all(facs);
			for(i=0;i<N;i++) facs.l1_sum(i,false);
			solution = facs.solution;
			retrans_all(solution);
		} break;
	default    : for(i=0;i<N;i++) lp_sum(p,0.0001,0.01,500,i,false); 
	}
	dummy.clear();
	for(gen_ptr=solution.first();gen_ptr;gen_ptr=solution.succ(gen_ptr)) {
		TRANS=solution.contents(gen_ptr);
		dummy.append(TRANS);
	}

	solution.clear(); 

	gen_ptr=dummy.first();
	TRANS=dummy.contents(gen_ptr);
	solution.append(TRANS);
	lastweight=TRANS(0);

	for(gen_ptr=dummy.first();gen_ptr;gen_ptr=dummy.succ(gen_ptr)) {
		TRANS=dummy.contents(gen_ptr);
		if (lastweight != TRANS(0)) {
			solution.append(TRANS);
			lastweight=TRANS(0);
		}
	}

	double* obj_values=new double[2];

	double fex=fex_sum(p);
	double fnew=fnew_sum(w,p);
	delta=fnew/fex;

	obj_values[0]=fnew+fex;
	obj_values[1]=delta;

	solution.clear();
	for(gen_ptr=dummy.first();gen_ptr;gen_ptr=dummy.succ(gen_ptr)) {
		TRANS=dummy.contents(gen_ptr);
		solution.append(TRANS);
	}

	return obj_values; 
}

double* facilities::N_lp_sum_v2(int N, int p, matrix& w, double epsilon,bool erase)
/********************************************************************
* Purpose   : Compute the optimum for problem class N|P|.|.|sum
* Returns   : pointer to array of objective values(double)
* Influences: list<location> solution
* Date      : Dec 95
* Author    : Michael Ochs
*********************************************************************/
{
	facilities     Ex_save(*this);
	facilities     Ex_hat(*this);
	list<location> save_solution;
	location       temp(_d(),_Q());
	location       search(_d(),1);

	double*        pre_step_objective;
	double         delta;

	register int   n;
	register int   i;
	register int   j;

	int            num_iter=0;

	bool equal;

	if (erase) solution.clear();

	//Schritt 1
	pre_step_objective=N_lp_sum_v1(N,p,w,false);

	//Schritt 2

	delta=fnew_sum(w,p)/fex_sum(p);

	do {
		//Schritt 3
		if (delta<epsilon) {
			pre_step_objective[0]=fnew_sum(w,p)+fex_sum(p);
			pre_step_objective[1]=delta;
			return pre_step_objective;
		} 

//Schritt 4
		save_solution=solution;
		solution.clear();
		for(n=0; n<N; n++) {
			Ex_hat=Ex_save;
			for(i=0;i<save_solution.length();i++) {
				search=save_solution.contents(save_solution[i]);
				if(search(0) != n) {
					for(j=0; j < _d(); j++) temp[j]=search[j];
					for(j=0; j < _Q(); j++) if(j!=n) temp(j)=w(j,n);
					Ex_hat.append(temp);
				}
			}
			switch(p) {
			case 0    :   Ex_hat.l2sqr_sum(n,true); break;
			case 1    :   Ex_hat.l1_sum(n,true); break;
			case 2    :   Ex_hat.l2_sum(n,0.0001,true); break;
			case L_max:   Ex_hat.linf_sum(n,true); break;
			default   :   Ex_hat.lp_sum(p,0.0001,0.01,500,n,true);
			}
			solution.append(Ex_hat.solution.head());
		}
		if (solution.length()==save_solution.length()) {
			equal=true;
			for(j=0;j<solution.length();j++) {
				if (solution.contents(solution[j]) == save_solution.contents(save_solution[i])) equal = equal & true;
				else equal = false;
			}
		}
		if (equal) return pre_step_objective;
		save_solution=solution;
		num_iter++;
	} while (num_iter < 20);         
	return 0;
}

double facilities::S(int N,int n, matrix& w)
/********************************************************************
* Purpose   : Service routine for N_l2sqr_sum
* Return    : S(n) (double)
* Influences: %
* Date      : Feb 96
* Author    : Michael Ochs
*********************************************************************/
{
	register  int l;
	double    S=0;
	list_item pointer_to_location;
	location  temp(_d(),_Q());

	for(pointer_to_location=first();pointer_to_location;pointer_to_location=succ(pointer_to_location))
    {
		temp=contents(pointer_to_location);
		S += temp(n);
    }
	for(l=0;l<N;l++)
    {
		if (l<n) S += w(l,n);
		if (l>n) S += w(n,l);
    }
	return S;
}

double facilities::N_l2sqr_sum(int N, matrix& w, bool erase)
/********************************************************************
* Purpose    : Compute the optimum for problem class N|P|.|l2sqr|sum
* Return     : objective value (double)
* Influences : list<location> solution
* Date       : Feb 96
* Author     : Michael Ochs
* Last Update: Jul 99 (Holger Hennes)
*********************************************************************/
{
	register  int n;
	register  int l;
	register  int k;
	double    objective=0;
	list_item pointer_to_location;
	location  temp(_d(),_Q());
	location  psh(_d(),1);
	location  psh2(_d(),1);
	matrix    A(N,N);
	vector    X1(N);
	vector    X2(N);
	vector    A1(N);
	vector    A2(N);

	/* Build matrix A (NxN) */
	for(l=0;l<N;l++)
    for(n=0;n<N;n++)
	{
		if (l==n) A(l,n)=S(N,n,w);
		if (l<n) A(l,n)=-w(l,n);
		if (l>n) A(l,n)=-w(n,l);
	}
  /* Build vectors A1,A2 */
	for(n=0;n<N;n++)
    { 
		A1[n]=0;
		A2[n]=0;
		for(pointer_to_location=first();pointer_to_location;pointer_to_location=succ(pointer_to_location))
		{
			temp=contents(pointer_to_location);
			A1[n] += temp(n)*temp[0];
			A2[n] += temp(n)*temp[1];
		}
    }
	/* Compute solutions X1 and X2 */
	X1=A.solve(A1);
	X2=A.solve(A2);
	
	if (erase) solution.clear();

	/* Push the optima into the solution data structure */
	for(n=0;n<N;n++)
    {
		psh[0]=X1[n];
		psh[1]=X2[n];
		psh(0)=n;
		solution.append(psh);
    }

  
	/* Compute the objective function */
	for(pointer_to_location=first();pointer_to_location;pointer_to_location=succ(pointer_to_location))
		for(n=0;n<N;n++)
		{
			temp=contents(pointer_to_location);
			psh=solution.contents(solution[n]);
			objective += temp(n)*( L_sqr(temp[0]-psh[0]) + L_sqr(temp[1]-psh[1]) );
		}
	for(l=0;l<N;l++)
		for(k=l+1;k<N;k++)
		{
			psh =solution.contents(solution[l]);
			psh2=solution.contents(solution[k]);
			double diff1=psh[0]-psh2[0];
			double diff2=psh[1]-psh2[1];
			objective += w(l,k)*( L_sqr(diff1) + L_sqr(diff2) );
		}
			
	return objective;
}

void facilities::write_mps(vector& OBJ,matrix& CONSTRAINTS,vector& RHS,list<char>& EQU,char* filename)
/********************************************************************
* Purpose    : write LP to mps file
* Returns    : -none- 
* Influences : %
* Date       : Aug 96
* Author     : Michael Ochs 
* Last Change: Dec 97 (Karen Schulze)
*********************************************************************/
{
	int    blanks;
	double logbase;
	int    bct;
	int i,j;
	char EQU_Type;
	int numrows=CONSTRAINTS.dim1();
	int numcols=CONSTRAINTS.dim2();
	const int io_flags=ios::showpoint|ios::uppercase|ios::scientific;
	ofstream output(filename,ios::out);
	output << setiosflags(io_flags) << setprecision(1) ;
	output << "NAME PROBLEM             FREE" << endl;
	output << "ROWS" << endl;
	output << " N  OBJ" << endl;
	i=0;
	for(list_item gen_ptr=EQU.first();gen_ptr;gen_ptr=EQU.succ(gen_ptr)) {
		i++;
		EQU_Type=EQU.contents(gen_ptr);
		switch (EQU_Type) {
		case 'N' : output << " N  ROW" << i << endl; break;
		case 'E' : output << " E  ROW" << i << endl; break;
		case 'L' : output << " L  ROW" << i << endl; break;
		case 'G' : output << " G  ROW" << i << endl; break;
		}
	}
	output << "COLUMNS" << endl;
	for(j=0;j<numcols;j++) {
		if (OBJ[j] != 0) {
			output << "    COL" << j+1;
			logbase=log10(j+1);
			blanks=7-(int)(logbase+1);
			      for(bct=0; bct < blanks; bct++) output << " ";
				  if (OBJ[j]<0) output << "OBJ";
				  else output << "OBJ ";
				  logbase=log10(OBJ[j]);
				  blanks=12-(int)(logbase+1);
				  for(bct=0; bct < blanks;  bct++) output << " ";
				  output << setiosflags(io_flags) << setprecision(8) << OBJ[j] << endl;
		}
		for(i=0;i<numrows;i++) {
			if (CONSTRAINTS(i,j)!=0) {
				output << "    COL" << j+1;
				logbase=log10(j+1);
				blanks=7-(int)(logbase+1);
				for(bct=0; bct < blanks; bct++) output << " ";
				output << "ROW" << i+1;
				logbase=log10(i+1);
				blanks=12-(int)(logbase+1);
				if (CONSTRAINTS(i,j) < 0) blanks--;
				for(bct=0; bct < blanks; bct++) output << " ";
				output << setiosflags(io_flags) << setprecision(1) << CONSTRAINTS(i,j) << endl;
			}
		}
	}

	output << "RHS" << endl;
	for(i=0;i<numrows;i++) {
		output << "    RHS1      ROW" << i+1;
		logbase=log10(i+1);
		blanks=12-(int)(logbase+1);
		if (RHS[i] < 0) blanks--;
		for(bct=0; bct < blanks; bct++) output << " ";
		output << setiosflags(io_flags) << setprecision(1) << RHS[i] << endl;
	}
	output << "ENDATA" << endl;
	output.close();
}
      
double facilities::read_sol_mps(char* filename1,char* filename2,int N)
/********************************************************************
* Purpose   : read solution file in mps format generated by cplex
* Returns   : double (obj value)
* Influences: list<location> solution
* Date      : Aug 96
* Author    : Michael Ochs
*********************************************************************/
{ 
	double z1=0;
	double z2=0;
	int i=0;
	location *solutions;
	solutions = new location[N];
	const int maxline = 110;
	char line[maxline];
	solution.clear();
	for(int filect=0;filect<2;filect++) {
		fstream input((filect) ? filename2:filename1,ios::in);
		input.seekg(0,ios::beg);
		int cnt_N = 0;
		double tmp;
		while(input.getline(line,maxline)) {
		/****  Reading the objective value  ****
			****  Going to line 3 and reading the value  ****/
			if (line[11] == 'V' && line[12] == 'A' && line[13] == 'L') {
				char value[45];
				for(i=0;i<45;i++) value[i]=line[16+i];
				double tval=atof(value);
				(filect) ? z2 = tval : z1 = tval;
			}
			/**** Reading the solutions for the columns  *****/
			if (line[10] == 'C' && line[11] == 'O' && line[12] == 'L') {
				char value[15];
				for (i = 0; i < 15; i++) value[i]=line[30+i];
				tmp=atof(value);
				if (cnt_N < N) solutions[cnt_N][filect] = tmp;
				cnt_N++;
			}
		}
	} 
	for (i = 0; i < N; i++) {solutions[i](0)=i;solution.append(solutions[i]);}
	delete[] solutions;
	return (z1+z2);
}           

void facilities::N_l1_sum_gen_LP(int N,matrix& w,char* lp1,char* lp2)
/*********************************************************************
* Purpose   : Generate LP for problem class N|P|.|l1|sum
* Returns   : %
* Influences: %
* Date      : Aug 96
* Author    : Michael Ochs
*********************************************************************/
{
	int M=length();

	int numrows=2+N*M+((N*(N-1))/2);
	int numcols=2*N*M+N+(N*(N-1));

	int i,j,k,l;

	vector     obj(numcols);
	vector     rhs1(numrows);
	vector     rhs2(numrows);
	matrix     cst(numrows,numcols);
	list<char> equ;
	location   temp(2,N);
	list_item  gen_ptr;
	
	equ.clear();

	for(j=0;j<N;j++) {
		gen_ptr=first();

		for(i=0;i<M;i++) {
			temp=contents(gen_ptr);
			cst(j*M+i,j)=1.0;
			cst(j*M+i,2*j*M+N+2*i)=1.0;
			cst(j*M+i,2*j*M+N+2*i+1)=-1.0;
			rhs1[j*M+i]=temp[0];
			rhs2[j*M+i]=temp[1];
			obj[N+2*j*M+2*i]=temp(j);
			obj[N+2*j*M+2*i+1]=temp(j);
			gen_ptr=succ(gen_ptr);
			equ.append('E');
		}
	}

	int rowcnt=0;
	int colcnt=0;

	for(l=0;l<N;l++) 
		for(k=0;k<N;k++) {
			if (k>l) {
				cst(N*M+rowcnt,k)=1.0;
				cst(N*M+rowcnt,l)=-1.0;
				cst(N*M+rowcnt,N+2*N*M+colcnt)=1.0;
				cst(N*M+rowcnt,N+2*N*M+(colcnt+1))=-1.0;
				rhs1[N*M+rowcnt]=0.0;
				rhs2[N*M+rowcnt]=0.0;
				obj[N+2*N*M+colcnt]=w(l,k);
				obj[N+2*N*M+(++colcnt)]=w(l,k);
				rowcnt++;
				colcnt++;
				equ.append('E');
			}
		}
	for(j=0;j<numcols-N;j++) cst(numrows-2,j+N)=1.0;
	rhs1[numrows-2]=0.0;
	rhs2[numrows-2]=0.0;
	equ.append('G');
	for(j=0;j<N;j++) cst(numrows-1,j)=1.0;
	rhs1[numrows-1]=0.0;
	rhs2[numrows-1]=0.0;
	equ.append('N');

	write_mps(obj,cst,rhs1,equ,lp1);
	write_mps(obj,cst,rhs2,equ,lp2);
} 

double facilities::N_l1_sum(int N, matrix& w, bool erase)
/*********************************************************************
* Purpose    : Compute optimum for problem class N|P|.|l1|sum
* Returns    : objective (double)
* Influences : solution
* Date       : Aug 96
* Author     : Michael Ochs
* Last Change: Jan 99 (Holger Hennes)
*********************************************************************/
{
	if (w.dim1() != w.dim2()) lola_error("Unequal matrix dimensions.",1331,1); 
	else
		if (w.dim1() != N) lola_error("Matrix dimensions and number of new locations do not fit.",1332,1); 
		else
			if ((head()).wht_dim() != N) lola_error("Weight dimensions and number of new location do not fit.",1333,1); 

	if (erase) solution.clear();

#ifdef HAVECPLEX

	char*       lp1="data_c1.mps";
	char*       lp2="data_c2.mps"; 
	
	const char* remove_data = "rm -f data_c1.mps data_c2.mps";
	const char* remove_sol  = "rm -f sol_c1mps.txt sol_c2mps.txt";

	char*       sol1="sol_c1mps.txt";
	char*       sol2="sol_c2mps.txt";
	const char* cplexcall   = "cplex < runcplex >/dev/null";

	ofstream output("runcplex",ios::out);
	output << "read " << lp1 << endl;
	output << "optimize" << endl;
	output << "write " << sol1 << endl;
	output << "read " << lp2 << endl;
	output << "optimize" << endl;
	output << "write " << sol2 << endl;
	output << "quit" << endl;
	output.close();

	system(remove_data);
	system(remove_sol);

	N_l1_sum_gen_LP(N,w,lp1,lp2);

	system(cplexcall);

	return read_sol_mps(sol1,sol2,N);

#else
	// Computing the solution with help of lp_solve
	int M=length(),i,j;
	int counter=N*M;
	int rows=counter+(N*(N-1))/2;
	double *objfunc = new double[2*rows+N+1];
	double *rhs1 = new double[rows+1];
	double *rhs2 = new double[rows+1];
	list_item loc_ptr;
	location tmp_loc(2,N);
	rhs1[0]=rhs2[0]=objfunc[0]=0;

	for (j=0;j<N;j++) {
		objfunc[j+1]=0;
		loc_ptr=first();
		for (i=0;i<M;i++) {
			tmp_loc=contents(loc_ptr);
			objfunc[N+2*j*M+2*i+1]=tmp_loc(j);
			objfunc[N+2*j*M+2*i+2]=tmp_loc(j);
			rhs1[j*M+i+1]=tmp_loc[0];
			rhs2[j*M+i+1]=tmp_loc[1];
			loc_ptr=succ(loc_ptr);
		}
		for (i=j+1;i<N;i++) {
			objfunc[N+2*counter+2*(i+j)-1]=w(j,i);
			objfunc[N+2*counter+2*(i+j)]=w(j,i);
			rhs1[counter+i+j]=0;
			rhs2[counter+i+j]=0;
		}
	}
	double *soltnx=new double[N], *soltny=new double[N];
	double objval=N_l1_sum_solve(N,M,rows,objfunc,rhs1,rhs2,soltnx,soltny);
	for (j=0;j<N;j++) {
		location L(2,1.0,soltnx[j],soltny[j],j);
		solution.append(L);
	}
	delete[] soltnx;
	delete[] soltny;
	delete[] objfunc;
	delete[] rhs1;
	delete[] rhs2;
	return objval;
#endif
} //N_l1_sum

double facilities::N_linf_sum(int N,matrix& w,bool erase)
/*********************************************************************
* Purpose    : Compute optimum for problem class N|P|.|linf|sum
* Returns    : objective (double)
* Influences : solution via read_sol_mps
* Date       : Aug 96
* Author     : Michael Ochs
* Last Change: Jan 99 (Holger Hennes)
*********************************************************************/
{
	facilities facs = *this;
	trans_all(facs);
#ifndef HAVECPLEX
	double* obj=N_lp_sum_v1(N,-1,w,true);
	return obj[0];
#else
	facs.N_l1_sum(N,w,erase);
	solution=facs.solution;
	retrans_all(solution);
	return fnew_sum(w,L_max)+fex_sum(L_max);
#endif
}

location facilities::A_plus(double &z) {
   location result(2,1), tmp(2,1);
   result.pos() = head().pos() + (1/head()(0)) * z;
   for(list_item trav = succ(first()); trav ;trav = succ(trav) ) {
      tmp = contents(trav);
      tmp.pos() = tmp.pos() + (1/tmp(0)) * z;
      if (tmp.pos()[0] < result.pos()[0]) result.pos()[0] = tmp.pos()[0];
      if (tmp.pos()[1] < result.pos()[1]) result.pos()[1] = tmp.pos()[1];
   }
   return result;
}

location facilities::A_minus(double &z) {
   location result(2,1), tmp(2,1);
   result.pos() = head().pos() - (1/head()(0)) * z;
   for(list_item trav = succ(first()); trav ;trav = succ(trav) ) {
      tmp = contents(trav);
      tmp.pos() = tmp.pos() - (1/tmp(0)) * z;
      if (tmp.pos()[0] > result.pos()[0]) result.pos()[0] = tmp.pos()[0];
      if (tmp.pos()[1] > result.pos()[1]) result.pos()[1] = tmp.pos()[1];
   }
   return result;
}

double facilities::linf_max(bool erase) {
/********************************************************************
* Purpose   : Compute the optimum for problem class 1|P|.|linf|max
* Returns   : objective value
* Influences: %
* Date      : Oct 95
* Author    : Ingo Bartling
*********************************************************************/
	facilities facs= *this;
	if (erase) solution.clear();
	double z = 0, check = 0;
	location tmp1(2,1), tmp2(2,1);
	for(list_item p = first(); p != last();p = succ(p) ) {
		for(list_item q = succ(p); q; q = succ(q) ) {
			location P = contents(p), Q = contents(q);
			check = fabs((P(0) * Q(0) / (P(0) + Q(0))) * (P[0] - Q[0]));
			if (z < check) z = check;
			check = fabs(((P(0) * Q(0)) / (P(0) + Q(0))) * (P[1] - Q[1]));
			if (z < check) z = check;
		}
	}
	if (!z) {
		location P;
		forall(P,facs) 
			if (P(0)) {
				solution.append(P);
				return z;
			}
		lola_message("facilities::linf_max: No weight is unequal to zero and the whole plane is optimal",1);
	}
	tmp1 = A_minus(z); tmp1(0)=1;
	tmp2 = A_plus(z); tmp2(0)=1;
	if (fabs(tmp1[0]-tmp2[0]) < L_eps) tmp1[0] = tmp2[0];
	if (fabs(tmp1[1]-tmp2[1]) < L_eps) tmp1[1] = tmp2[1];
	if (tmp1[0] == tmp2[0] && tmp1[1] == tmp2[1]) {
		solution.append(tmp1);
		return z;
	}
	solution.append(tmp1);
	solution.append(tmp2);
	return z;
}

double facilities::l1_max(bool erase) {
/********************************************************************
* Purpose    : Compute the optimum for problem class 1|P|.|l1|max
* Returns    : objective value
* Influences : %
* Date       : Oct 95
* Author     : Ingo Bartling 
* Last change: Jul 97 (Holger Hennes)
*********************************************************************/
	facilities facs = *this;
	retrans_all(facs);
	double z = 0;
	if (!erase) z = facs.linf_max(false);
	else z = facs.linf_max();
	solution = facs.solution;
	trans_all(solution);
	return z;
}

double facilities::linf_v1_max(bool erase) {
/********************************************************************
* Purpose   : Compute the optimum for problem class 1|P|v=1|linf|max
* Returns   : objective value
* Influences: %
* Date      : Oct 95
* Author    : Ingo Bartling
*********************************************************************/
	list_item trav;
	double wht_chk = head()(0);
	for(trav = succ(first()); trav ;trav = succ(trav) ) {
		location tmp = contents(trav);
		if (tmp(0) != wht_chk) lola_error("All weights must be equal!",1307,1);
	}
	solution.clear();
	double z = 0, z1 = 0, z2 = 0;
	location a_plus(2,1), tmp1(2,1);
	a_plus.pos() = head().pos();
	for(trav = succ(first()); trav ;trav = succ(trav) ) {
		tmp1 = contents(trav);
		tmp1.pos() = tmp1.pos();
		if (tmp1.pos()[0] < a_plus.pos()[0]) a_plus.pos()[0] = tmp1.pos()[0];
		if (tmp1.pos()[1] < a_plus.pos()[1]) a_plus.pos()[1] = tmp1.pos()[1];
	}
	location a_minus(2,1), tmp2(2,1);
	a_minus.pos() = head().pos();
	for(trav = succ(first()); trav ;trav = succ(trav) ) {
		tmp2 = contents(trav);
		tmp2.pos() = tmp2.pos();
		if (tmp2.pos()[0] > a_minus.pos()[0]) a_minus.pos()[0] = tmp2.pos()[0];
		if (tmp2.pos()[1] > a_minus.pos()[1]) a_minus.pos()[1] = tmp2.pos()[1];
	}
	z1 = (a_minus[0] - a_plus[0]) / 2;
	z2 = (a_minus[1] - a_plus[1]) / 2;
	z = L_MAX(z1, z2);
	tmp1[0] = a_minus[0]-z;
	tmp1[1] = a_plus[0]+z;
	tmp1(0) = 0;
	tmp2[0] = a_minus[1]-z;
	tmp2[1] = a_plus[1]+z;
	tmp2(0) = 0;
	if (tmp1[0] == tmp1[1] && tmp2[0] == tmp2[1]) {
		tmp1[1] = tmp2[0];
		solution.append(tmp1);
		return z;
	}
	if (tmp1[0] == tmp1[1]) {
		tmp1[1] = tmp2[0];
		tmp2[0] = tmp1[0];
	} else if (tmp2[0] == tmp2[1]) {
		tmp2[0] = tmp1[1];
		tmp1[1] = tmp2[1];
	}
	solution.append(tmp1);
	solution.append(tmp2);
	return z;
}

double facilities::l1_v1_max(bool erase) {
/********************************************************************
* Purpose    : Compute the optimum for problem class 1|P|v=1|l1|max
* Returns    : objective value
* Influences : %
* Date       : Oct 95
* Author     : Ingo Bartling 
* Last change: Jul 97 (Holger Hennes)
*********************************************************************/
	facilities facs = *this;
	retrans_all(facs);
	double z = 0;
	if (!erase) z = facs.linf_v1_max(false);
	else z = facs.linf_v1_max();
	solution = facs.solution;
	trans_all(solution);
	return z;
}

double facilities::l2_max(bool erase) {return 0;}
/********************************************************************
* Purpose    : Compute the optimum for problem class 1|P|.|l2|max
* Returns    : objective value
* Influences : %
* Date       : Oct 95
* Author     : Ingo Bartling
* Last Change: Sep 98 (Holger Hennes) -> ausgedient
*********************************************************************/
/*{
}*/

/* Functions for the Elzinga-Hearn-Algorithm */
/* first the type definitions and global variables */
const int  MaxLoc = 1500;
const int True = 1;
const int False = 0;
const double epsi = 0.1E-12;
const double epseq = 0.001;


static location UnCoveredPoint(circle &K, int &Anzlocations, location *Work) {
	int i;
	location AktUnCovered(2,1), Center(2,1);
	double AktDistance, OldDistance;
	OldDistance = 0;
	Center[0] = K.center().xcoord(); Center[1] = K.center().ycoord(); Center(0) = 1;
	AktUnCovered[0] = 0; AktUnCovered[1] = 0; AktUnCovered(0) = 0;
	for (i = 1; i <= Anzlocations; i++) {
		AktDistance = Work[i](0) * (Work[i]-Center).norm(2);
		if (((AktDistance - K.radius()) > epseq) && (AktDistance > OldDistance)) {
			AktUnCovered = Work[i];
			OldDistance  = AktDistance;
		}
	}
	return AktUnCovered;
}

static int IsCovered(location P, circle K) {
	location Center(2,1);
	Center[0] = K.center().xcoord(); Center[1] = K.center().ycoord();
	if (P(0) * (P-Center).norm(2) <= (K.radius() + epseq)) return True;
	return False;
}

static void circleIntersection(circle &K1, circle &K2, location &I1, location &I2) {
	double slope, xAb, yAb, a, b, c, Disc;
	/* first the line with equal distance to both centers */
	if (K2.center().ycoord() != K1.center().ycoord()) {
		slope = -(K2.center().xcoord() - K1.center().xcoord())/(K2.center().ycoord() - K1.center().ycoord());
		yAb   = (L_sqr(K2.center().xcoord()) + L_sqr(K2.center().ycoord()) - L_sqr(K1.center().xcoord()) - L_sqr(K1.center().ycoord()) - L_sqr(K2.radius())  + L_sqr(K1.radius())) / (2 * (K2.center().ycoord() - K1.center().ycoord()));
	}
	else {
		slope = L_INF;
		xAb   = (L_sqr(K2.center().xcoord()) - L_sqr(K1.center().xcoord()) - L_sqr(K2.radius())  + L_sqr(K1.radius())) / (2 * (K2.center().xcoord() - K1.center().xcoord()));
	}
   /* Now the quadratic equation for the intersection points */
	if (slope < L_INF) {
		a = 1 + L_sqr(slope);
		b = 2 * (slope * yAb - K1.center().xcoord() - K1.center().ycoord() * slope);
		c = -2 * K1.center().ycoord() * yAb - L_sqr(K1.radius()) + L_sqr(yAb) + L_sqr(K1.center().xcoord()) + L_sqr(K1.center().ycoord());
	}
	else {
		a = 1;
		b = -2 * K1.center().ycoord();
		c = L_sqr(xAb) + L_sqr(K1.center().xcoord()) + L_sqr(K1.center().ycoord()) - 2 * xAb * K1.center().xcoord() - K1.radius() * K1.radius();
	}
	Disc = L_sqr(b) - 4 * a * c;
	/* Now distinguish the possible cases */
	if (Disc < -epsi) {
		I1[0] = L_INF; I1[1] = L_INF; I1(0) = 0;
		I2[0] = L_INF; I2[0] = L_INF; I2(0) = 0;
	}
	if ((Disc >= -epsi) && (Disc <= epsi)) Disc = 0;
	if (Disc >= 0) {
		if (slope < L_INF) {
			I1[0] = (-b + sqrt(Disc))/(2 * a);
			I2[0] = (-b - sqrt(Disc))/(2 * a);
			I1[1] = I1[0] * slope + yAb;
			I2[1] = I2[0] * slope + yAb;
		}
		else {
			I1[1] = (-b + sqrt(Disc))/(2 * a);
			I2[1] = (-b - sqrt(Disc))/(2 * a);
			I1[0] = xAb;
			I2[0] = xAb;
		}
		I1(0) = 1; I2(0) = 1;
	}
}

static void circleLineIntersection(circle &K1, circle &K2, location &I1, location &I2) {
	double slope, xAb, yAb, a, b, c, Disc;
	/* first the line with equal distance to both centers */
	slope = K2.center().xcoord();
	if (slope < L_INF) yAb   = K2.center().ycoord();
	else xAb   = K2.center().ycoord();
	/* Now the quadratic equation for the intersection points */
	if (slope < L_INF) {
		a = 1 + L_sqr(slope);
		b = 2 * (slope * yAb - K1.center().xcoord() - K1.center().ycoord() * slope);
		c = -2 * K1.center().ycoord() * yAb - L_sqr(K1.radius()) + L_sqr(yAb) + L_sqr(K1.center().xcoord()) + L_sqr(K1.center().ycoord());
	}
	else {
		a = 1;
		b = -2 * K1.center().ycoord();
		c = L_sqr(xAb) + L_sqr(K1.center().xcoord()) + L_sqr(K1.center().ycoord()) - 2 * xAb * K1.center().xcoord() - L_sqr(K1.radius());
	}
	Disc  = L_sqr(b) - 4 * a * c;
	/* Now distinguish the possible cases */
	if (Disc < -epsi) {
		I1[0] = L_INF; I1[1] = L_INF; I1(0) = 0;
		I2[0] = L_INF; I2[1] = L_INF; I2(0) = 0;
	}
	if ((Disc >= -epsi) && (Disc <= epsi))
		Disc = 0;
	if (Disc >= 0) {
		if (slope < L_INF) {
			I1[0] = (-b + sqrt(Disc))/(2 * a);
			I2[0] = (-b - sqrt(Disc))/(2 * a);
			I1[1] = I1[0] * slope + yAb;
			I2[1] = I2[0] * slope + yAb;
		}
		else {
			I1[1] = (-b + sqrt(Disc))/(2 * a);
			I2[1] = (-b - sqrt(Disc))/(2 * a);
			I1[0] = xAb;
			I2[0] = xAb;
		}
		I1(0) = 1; I2(0) = 1;
	}
}

static void LineIntersection(circle &K1, circle &K2, location &I1, location &I2) {
   /* Intersect the two lines */
	if (K1.center().xcoord() != K2.center().xcoord()) {
		if (K1.center().xcoord() ==  L_INF) {
			I1[0]=K1.center().ycoord();
			I1[1]=I1[0] * K2.center().xcoord() + K2.center().ycoord();
		}
		if (K2.center().xcoord() == L_INF) {
			I1[0]=K2.center().ycoord();
			I1[1]=I1[0] * K1.center().xcoord() + K1.center().ycoord();
		}
		if ((K1.center().xcoord() <  L_INF) && (K2.center().xcoord() < L_INF)) {
			I1[0] = (K2.center().ycoord() - K1.center().ycoord()) / (K1.center().xcoord() - K2.center().xcoord());
			I1[1] = I1[0] * K1.center().xcoord() + K1.center().ycoord();
		}
	}
	else I1[0] = L_INF; I2[1] = L_INF;
	I2[0] = I1[0]; I2[1] = I1[1];
}

static void Intersection(circle &K1, circle &K2, location &I1, location &I2) {
	if ((K1.radius() > 0) && (K2.radius() > 0))
		circleIntersection(K1, K2, I1, I2); 
	if ((K1.radius() > 0) && (K2.radius() == 0))
		circleLineIntersection(K1, K2, I1, I2);
	if ((K2.radius() > 0) && (K1.radius() == 0))
		circleLineIntersection(K2, K1, I1, I2);
	if ((K1.radius() == 0) && (K2.radius() == 0))
		LineIntersection(K1, K2, I1, I2);
}

static circle Equicircle(location &P1, location &P2) {
	double f,g;
	f = (P2(0) / P1(0)) ; g = 1 - L_sqr(f);
	circle temp((P1[0] - L_sqr(f) * P2[0]) / g, (P1[1] - L_sqr(f) * P2[1]) / g, fabs(f * (P1-P2).norm(2) / g));
	return temp;
}

static circle EquiLine(location &P1, location &P2) {
	circle temp(L_INF, (P1[0] + P2[0]) / 2, 0.0);
	if (P1[1] != P2[1]) temp = circle(-(P2[0] - P1[0])/(P2[1] - P1[1]), (L_sqr(P2[0]) + L_sqr(P2[1]) - L_sqr(P1[0]) - L_sqr(P1[1]) ) / (2 * (P2[1] - P1[1])), 0.0);
	return temp;
}

static circle Compute3Cover(location &P1, location &P2, location &P3) {
	circle   EquiAB, EquiAC, EquiBC, Actcircle;
	double   AktRadius, R1, R2;
	location AktCenter(2,1), I1(2,1), I2(2,1);
	if (P1(0) != P2(0)) EquiAB = Equicircle(P1,P2);
	else EquiAB = EquiLine(P1,P2);
	if (P1(0) != P3(0)) EquiAC = Equicircle(P1,P3);
	else EquiAC = EquiLine(P1,P3);
	if (P2(0) != P3(0)) EquiBC = Equicircle(P2,P3);
	else EquiBC = EquiLine(P2,P3);
	AktRadius = 100000.0;
	Intersection(EquiAB, EquiAC, I1, I2);
	R1 = P1(0) * (I1-P1).norm(2);
	R2 = P1(0) * (I2-P1).norm(2);
	if (R1 < R2) {
		AktRadius = R1;
		AktCenter = I1;
	}
	else {
		AktRadius = R2;
		AktCenter = I2;
	} 
	Intersection(EquiAB, EquiBC, I1, I2);
	R1 = P1(0) * (I1-P1).norm(2);
	R2 = P1(0) * (I2-P1).norm(2);
	if (R1 < AktRadius) {
		AktRadius = R1;
		AktCenter = I1;
	}   
	if (R2 < AktRadius) {
		AktRadius = R2;
		AktCenter = I2;
	}
	Intersection(EquiAC, EquiBC, I1, I2);
	R1 = P1(0) * (I1-P1).norm(2);
	R2 = P1(0) * (I2-P1).norm(2);
	if (R1 < AktRadius) {
		AktRadius = R1;
		AktCenter = I1;
	}
	if (R2 < AktRadius) {
		AktRadius = R2;
		AktCenter = I2;
	}
	Actcircle = circle(AktCenter[0], AktCenter[1], AktRadius);
	return Actcircle;
}

static circle Compute2Cover(location &P1, location &P2) {
	double a1 = (P1(0) * P1[0] + P2(0) * P2[0]) / (P1(0) + P2(0));
	double a2 = (P1(0) * P1[1] + P2(0) * P2[1]) / (P1(0) + P2(0));
	double a3 = P1(0) * P2(0) * (P1-P2).norm(2) / (P1(0) + P2(0));
	circle temp(a1, a2, a3);
	return temp;
}


static void Swap3(location &A, location &B, location &C) {
   /* This procedure guarentees that the AC circle is not smaller
      than the BC circle by renaming the points if neccessary     */
	circle CircAC, CircBC;
	location temp(2,1);
	CircAC = Compute2Cover(A, C);
	CircBC = Compute2Cover(B, C);
	if (CircBC.radius() > CircAC.radius()) {
		temp = A;
		A = B;
		B = temp;
	}
}

static void Swap4(location &A, location &B, location &C, location &D) {
   /* This procedure guarentees that the AD circle is not smaller
      than the BD and CD circles by renaming the points if neccessary     */
	circle CircAD, CircBD, CircCD;
	location temp(2,1);
	CircAD = Compute2Cover(A,D);
	CircBD = Compute2Cover(B,D);
	CircCD = Compute2Cover(C,D);
	if ((CircBD.radius() > CircAD.radius()) && (CircBD.radius() >= CircCD.radius())) {
		temp = A;
		A = B;
		B = temp;
	}
	if ((CircCD.radius() > CircAD.radius()) && (CircBD.radius() < CircCD.radius())) {
		temp = C;
		C = A;
		A = temp;
	}  
}

static int IsTwoCovered(location P1, location P2, location P3) {
	if (IsCovered(P1,Compute2Cover(P2,P3))) return True;
	if (IsCovered(P2,Compute2Cover(P1,P3))) return True;
	if (IsCovered(P3,Compute2Cover(P1,P2))) return True;
	return False;
}

double facilities::elz_hearn(bool erase) {
/********************************************************************
* Purpose    : Compute the optimum for problem class 1|P|v=1|l2|max
* Returns    : objective value
* Influences : %
* Date       : Oct 95
* Author     : Stefan Nickel (Update: Ingo Bartling)
* Last Update: Jul 99 (Holger Hennes)
*********************************************************************/
	if (erase) solution.clear();
	int Anzlocations = size(), i = 1;
	location Work[MaxLoc], A(2,1), B(2,1), C(2,1), D(2,1);
	for (list_item trav = first(); trav; trav = succ(trav), i++) Work[i] = contents(trav); 
	circle Optcircle;
	int Points = 2, covered = False, coverB, coverC;
	circle Actcircle, CircACD, CircBCD, CircABD;
	A = Work[1];
	B = Work[Anzlocations];
	while (covered != True) {
		switch(Points) {
		case 2:
			Actcircle = Compute2Cover(A,B);
			C = UnCoveredPoint(Actcircle, Anzlocations, Work);
			if (C(0) == 0) covered = True;
			else {
				if ( A.pos() == C.pos() || B.pos() == C.pos() ) {
					if (A.pos() == C.pos()) {
						Points = 2;
						A(0) = C(0);
					}
					if (B.pos() == C.pos()) {
						Points = 2;
						B(0) = C(0);
					}
				}                    
				else {
					Points = 3;
					Swap3(A, B, C);
				}
			}
			break;
		case 3:
			Swap3(A, B, C);
			Actcircle = Compute2Cover(A,C);
			if (IsCovered(B,Actcircle) == True) {
				B = C;
				Points = 2;
			}
			else {
				Actcircle = Compute3Cover(A,B,C);
				D = UnCoveredPoint(Actcircle, Anzlocations, Work);
				if (D(0) == 0) covered = True;
				else {
						       if (A.pos() == D.pos() || B.pos() == D.pos() || C.pos() == D.pos()) {
								   		  if (A.pos() == D.pos()) {
											  Points = 3;
											  A(0) = D(0);
											  Swap3(A, B, C);
										  }
										  if (B.pos() == D.pos()) {
											  Points = 3;
											  B(0) = D(0);
											  Swap3(A, B, C);
										  }
										  if (C.pos() == D.pos()) {
											  Points = 3;
											  C(0) = D(0);
											  Swap3(A, B, C);
										  }
							   }                        
							   else {
								   Points = 4;
								   Swap4(A, B, C, D);
							   }
				}
			}
			break;
		case 4: Swap4(A, B, C, D);
			Actcircle = Compute2Cover(A,D);
			coverB = IsCovered(B,Actcircle);
			coverC = IsCovered(C,Actcircle);
			if ((coverB == True) && (coverC == True)) {
				B = D;
				Points = 2;
			}
			else {
				CircACD = circle(CircACD.center().xcoord(), CircACD.center().ycoord(), L_INF);
				CircACD = circle(CircABD.center().xcoord(), CircABD.center().ycoord(), L_INF);
				CircACD = circle(CircBCD.center().xcoord(), CircBCD.center().ycoord(), L_INF);
				if (coverC == False) {
					if (IsTwoCovered(A,C,D) == False) {
						Actcircle = Compute3Cover(A,C,D);
						if (IsCovered(B,Actcircle) == True)
							CircACD = Actcircle;
					}
				}
				if (coverB == False) {
					if (IsTwoCovered(A,B,D) == False) {
						Actcircle = Compute3Cover(A,B,D);
						if (IsCovered(C,Actcircle) == True)
							CircABD = Actcircle;
					}
				}
				if (IsTwoCovered(B,C,D) == False) {
					Actcircle = Compute3Cover(B,C,D);
					if (IsCovered(A,Actcircle) == True)
						CircBCD = Actcircle;
				}
				if ((CircACD.radius() < CircABD.radius()) && (CircACD.radius() < CircBCD.radius())){
					B = D;
					Actcircle = CircACD;
				} 
				if ((CircABD.radius() < CircACD.radius()) && (CircABD.radius() < CircBCD.radius())) {
					C = D;
					Actcircle = CircABD;
				}
				if ((CircBCD.radius() < CircABD.radius()) && (CircBCD.radius() < CircACD.radius())) {
					A = D;
					Actcircle = CircBCD;
				}
				Points = 3;
			}
			break;
		default :
			cout<<"Wrong number!\n";
      }
   }
   location result(2.0, 1.0, Actcircle.center().xcoord(), Actcircle.center().ycoord(), 0.0);
   solution.append(result);
   return (Actcircle.radius());
}
	
double facilities::N_linf_max(int N, matrix &W, bool erase) {
/********************************************************************
* Purpose    : Compute the optimum for problem class N|P|.|linf|max
* Returns    : objective value
* Influences : %
* Date       : Dec 95
* Author     : Ingo Bartling
* Last Change: Jan 99 (Holger Hennes)
*********************************************************************/
#ifdef HAVECPLEX
    return N_linf_max_mps(N,W,erase);
#else
	double objval=0.0;
	int M=length(),counter=1,i;
	int rows=2*M*N+N*(N-1);
	int cols=N+1;
	double *rhs1 = new double[rows+1];
	double *rhs2 = new double[rows+1];
	double *costs = new double[rows+1];
	list_item loc_ptr;
	location tmp_loc(2,N);

	rhs1[0]=0.0;rhs2[0]=0.0;

    for(i=1;i<=N;i++) {
		loc_ptr=first();
		for(int j=1;j<=M;j++) {
			tmp_loc=contents(loc_ptr);
			rhs1[counter]=tmp_loc[0];
			rhs2[counter]=tmp_loc[1];
			if (tmp_loc(i-1)) costs[counter]=1/tmp_loc(i-1);
			else lola_error("facilities::N_linf_max(): Every weight must be unequal to zero",1336,1);
			counter++;
			rhs1[counter]=-tmp_loc[0];
			rhs2[counter]=-tmp_loc[1];
			costs[counter]=1/tmp_loc(i-1);
			counter++;
			loc_ptr=succ(loc_ptr);
		}
	}
	for(i=0;i<N;i++) {
		for(int j=i+1;j<N;j++) {
			rhs1[counter]=0;
			rhs2[counter]=0;
			if (W(i,j)) costs[counter]=1/W(i,j);
			else lola_error("facilities::N_linf_max(): The entries of the dependency matrix must be unequal to zero",1337,1);
			counter++;
			rhs1[counter]=0;
			rhs2[counter]=0;
			costs[counter]=1/W(i,j);
			counter++;
		}
	}
	double *soltnx=new double[N];
	double *soltny=new double[N];
	objval=N_linf_max_solve(N,M,rows,rhs1,rhs2,costs,soltnx,soltny);
	for (i=0;i<N;i++) {
		location L(2,1.0,soltnx[i],soltny[i],i);
		solution.append(L);
	}
	delete[] costs;
	delete[] rhs1;
	delete[] rhs2;
	delete[] soltnx;
    delete[] soltny;
	return objval;
/*#else
	cerr << "This is a substitute function for the original CPLEX" << endl << "based algorithm, because there is no CPLEX on this system." << endl;
	for (list_item trav = first(); trav; trav = succ(trav)) {
		location tmp = contents(trav);
		if (tmp.wht_dim() != head().wht_dim() || tmp.dim() != head().dim() || tmp.wht_dim() < N)
			lola_error("Contradiction in the number of coordinates or weights.",1308,1);
	}
	if (erase) solution.clear();
	int numb_comb = L_bin_coeff(N,2);
	if (N == 1) numb_comb = 0;
	int m = 2*length()*N + 2*(numb_comb), n = N+1, i = m/2, j = m/2, am = 0;
	list<matrix> S;
	for (int cnt_LP = 0; cnt_LP < 2; cnt_LP++) {
		matrix A(m,n);
		vector b(m), c(n); c[n-1] = -1;
		int vm = 0, cnt_col = 0, z_i = 0, z_j = j;
		for (int cnt_N = 0; cnt_N < N; cnt_N++) {
			for (list_item loctmp = first(); loctmp; loctmp = succ(loctmp)) {
				location loc = contents(loctmp);
				A(z_i,cnt_col) = 1; A(z_i,N) = -1/loc(vm); b[z_i++] = loc[am];
				A(z_j,cnt_col) = 1; A(z_j,N) = 1/loc(vm); b[z_j++] = loc[am];
			}
			vm++; cnt_col++;
		}
		am++;
		for (int cnt_k = 1; cnt_k < N; cnt_k++) {
			for (int cnt_l = 0; cnt_l < cnt_k; cnt_l++) {
				if (W(cnt_l,cnt_k) == 0) {
					cout << "one weight between two new locations is zero!" << endl;
					W(cnt_l,cnt_k) = 1;
				}
				A(z_i,0) = 1; A(z_i,1) = -1; A(z_i,2) = -1/W(cnt_l,cnt_k); b[z_i++] = 0;
				A(z_j,0) = 1; A(z_j,1) = -1; A(z_j,2) = 1/W(cnt_l,cnt_k); b[z_j++] = 0;
			}
		}
		list<matrix> tmp_S = SIMPLEX(A,i,j,0,b,c);
		S.append(tmp_S.head());
	}
	for (int cnt_new = 0; cnt_new < N; cnt_new++) {
		//      location tmp_sol(2.0,1.0,S.head()(0,cnt_new),S.tail()(0,cnt_new),0.0);
		location tmp_sol(2.0,1.0,S.head()(0,cnt_new),S.tail()(0,cnt_new),cnt_new);
		solution.append(tmp_sol);
	}
	double z1 = S.head()(0,n-1), z2 = S.tail()(0,n-1), z = L_MAX(z1,z2);
	return z;
#endif*/
#endif
}

double facilities::N_l1_max(int N, matrix &W, bool erase) {
/********************************************************************
* Purpose   : Compute the optimum for problem class N|P|.|l1|max
* Returns   : objective value
* Influences: %
* Date      : Dec 95
* Author    : Ingo Bartling
*********************************************************************/
	if (erase) solution.clear();
	facilities facs = *this;
	retrans_all(facs);
	double z = facs.N_linf_max(N,W);
	solution = facs.solution;
	trans_all(solution);
	return z;
}


double facilities::N_linf_max_mps(int N, matrix &W, bool erase) {
/********************************************************************
* Purpose   : Compute the optimum for problem class N|P|.|linf|max
* Returns   : objective value
* Influences: %
* Date      : Jul 95
* Author    : Ingo Bartling
*********************************************************************/
#ifdef HAVECPLEX
	int cnt_LP;
	const char *message0 = "rm -f sol1.txt sol2.txt current_problem1.mps current_problem2.mps";
	system(message0);
	for (list_item trav = first(); trav; trav = succ(trav)) {
		location tmp = contents(trav);
		if (tmp.dim() != head().dim())
			lola_error("Contradiction in the number of coordinates of the facilities.",1309,1);
		if (tmp.wht_dim() != head().wht_dim())
			lola_error("Contradiction in the number of the weights of the facilities.",1310,1);
		for (int chk_whts = 0; chk_whts < head().wht_dim(); chk_whts++)
			if (tmp(chk_whts) == 0) 
				lola_error("Every weight must be nonzero!",1308,1);
			if (head().wht_dim() != N)
				lola_error("Number of weights of the facilities is wrong.",1311,1);
	}
	if (erase) solution.clear();
	int numb_comb = L_bin_coeff(N,2), M = length();
	if (N == 1) numb_comb = 0;
	int m = 2*M*N + 2*numb_comb, i = 0, n = 0, cnt_cols = 0;
	const int io_flags = ios::showpoint|ios::uppercase|ios::scientific;
	for (cnt_LP = 0; cnt_LP < 2; cnt_LP++) {
		ofstream output ((cnt_LP) ? "current_problem2.mps" : "current_problem1.mps", ios::out);
		output << "NAME          PROBLEM" << endl;
		output << "ROWS" << endl;
		output << " N  OBJ" << endl;
		// ****  All constraints are "lower or equal" and get the name "ROW1"  ****
		for (i = 0; i < m; i++) output << " G  ROW" << i << endl;
		// ****  Objective and constraints columns  ****
		output << "COLUMNS" << endl;
		for (cnt_cols = 0; cnt_cols < N; cnt_cols++) {
			for (i = 0; i < 2*M; i++) {
				output << "    ";
				output << "COL" << cnt_cols+1;
				if (cnt_cols+1 < 10) output << "      ";
				else
					if (cnt_cols+1 < 100) output << "     ";
					else
						if (cnt_cols+1 < 1000) output << "    ";
						else
							if (cnt_cols+1 < 10000) output << "   ";
							else
								if (cnt_cols+1 < 100000) output << "  ";
								else lola_error("Too much facilities for a mps-file.",1312,1);
								output << "ROW" << i+(cnt_cols*2*M);
								if (i+(cnt_cols*2*M) < 10) output << "      ";
								else
									if (i+(cnt_cols*2*M) < 100) output << "     ";
									else
										if (i+(cnt_cols*2*M) < 1000) output << "    ";
										else 
											if (i+(cnt_cols*2*M) < 10000) output << "   ";
											else
												if (i+(cnt_cols*2*M) < 100000) output << "  ";
												else lola_error("Too much facilities for a mps-file.",1313,1);
												output << "1.0";
												output << "            ";
												output << "ROW" << (i+1)+(cnt_cols*2*M);
												if ((i+1)+(cnt_cols*2*M) < 10) output << "      ";
												else
													if ((i+1)+(cnt_cols*2*M) < 100) output << "     ";
													else
														if ((i+1)+(cnt_cols*2*M) < 1000) output << "    ";
														else
															if ((i+1)+(cnt_cols*2*M) < 10000) output << "   ";
															else
																if ((i+1)+(cnt_cols*2*M) < 100000) output << "  ";
																else lola_error("Too much facilities for a mps-file.",1314,1);
																output << "-1.0" << endl; i++;
			}
			// **** Adding constraints for the relation between new locations  ****
			int z_row = 2*M*N;
			for (int cnt_k = 1; cnt_k < N; cnt_k++) {
				for (int cnt_l = 0; cnt_l < cnt_k; cnt_l++) {
					if (W(cnt_l,cnt_k) < 0) {
						cout << "The weight matrix for the new locations is incorrect!" << endl;
						W(cnt_l,cnt_k) = 1;
					}
					if (cnt_cols%2) {
						output << "    ";
						output << "COL" << cnt_cols+1;
						if (cnt_cols+1 < 10) output << "      ";
						else
							if (cnt_cols+1 < 100) output << "     ";
							else
								if (cnt_cols+1 < 1000) output << "    ";
								else
									if (cnt_cols+1 < 10000) output << "   ";
									else
										if (cnt_cols+1 < 100000) output << "  ";
										else lola_error("Too much facilities for a mps-file.",1315,1);
										output << "ROW" << z_row;
										if (z_row < 10) output << "      ";
										else
											if (z_row < 100) output << "     ";
											else
												if (z_row < 1000) output << "    ";
												else 
													if (z_row < 10000) output << "   ";
													else
														if (z_row < 100000) output << "  ";
														else lola_error("Too much facilities for a mps-file.",1316,1);
														output << "-1.0";
														output << "           ";
														output << "ROW" << ++z_row;
														if (z_row < 10) output << "      ";
														else
															if (z_row < 100) output << "     ";
															else
																if (z_row < 1000) output << "    ";
																else
																	if (z_row < 10000) output << "   ";
																	else
																		if (z_row < 100000) output << "  ";
																		else lola_error("Too much facilities for a mps-file.",1317,1);
																		output << "1.0" << endl; i++; z_row++;
					} else {
						output << "    ";
						output << "COL" << cnt_cols+1;
						if (cnt_cols+1 < 10) output << "      ";
						else
							if (cnt_cols+1 < 100) output << "     ";
							else
								if (cnt_cols+1 < 1000) output << "    ";
								else
									if (cnt_cols+1 < 10000) output << "   ";
									else
										if (cnt_cols+1 < 100000) output << "  ";
										else lola_error("Too much facilities for a mps-file.",1318,1);
										output << "ROW" << z_row;
										if (z_row < 10) output << "      ";
										else
											if (z_row < 100) output << "     ";
											else
												if (z_row < 1000) output << "    ";
												else 
													if (z_row < 10000) output << "   ";
													else
														if (z_row < 100000) output << "  ";
														else lola_error("Too much facilities for a mps-file.",1319,1);
														output << "1.0";
														output << "            ";
														output << "ROW" << ++z_row;
														if (z_row < 10) output << "      ";
														else
															if (z_row < 100) output << "     ";
															else
																if (z_row < 1000) output << "    ";
																else
																	if (z_row < 10000) output << "   ";
																	else
																		if (z_row < 100000) output << "  ";
																		else  lola_error("Too much facilities for a mps-file.",1320,1);
																		output << "-1.0" << endl; i++; z_row++;
				}
			}
		}
    }
	// ****  Buiding the "z-column" ****
	cnt_cols = N+1;
	if (cnt_cols < 10) output << "    COL" << cnt_cols << "      OBJ       1.0" << endl;
	else
		if (cnt_cols < 100) output << "    COL" << cnt_cols << "     OBJ       1.0" << endl;
		else
			if (cnt_cols < 1000) output << "    COL" << cnt_cols << "    OBJ       1.0" << endl;
			else
				if (cnt_cols < 10000) output << "    COL" << cnt_cols << "   OBJ       1.0" << endl;
				else
					if (cnt_cols < 100000) output << "    COL" << cnt_cols << "  OBJ       1.0" << endl;
					else lola_error("Too much facilities for a mps-file.",1321,1);
					i = 0;
					for (n = 0; n < N; n++) {
						for (list_item loctmp = first(); loctmp; loctmp = succ(loctmp)) {
							location loc = contents(loctmp);
							output << "    ";
							output << "COL" << cnt_cols;
							if (cnt_cols < 10) output << "      ";
							else
								if (cnt_cols < 100) output << "     ";
								else
									if (cnt_cols < 1000) output << "    ";
									else
										if (cnt_cols < 10000) output << "   ";
										else
											if (cnt_cols < 100000) output << "  ";
											else lola_error("Too much facilities for a mps-file.",1322,1);
											output << "ROW" << i;
											if (i < 10) output << "      ";
											else
												if (i < 100) output << "     ";
												else
													if (i < 1000) output << "    ";
													else 
														if (i < 10000) output << "   ";
														else
															if (i < 100000) output << "  ";
															else lola_error("Too much facilities for a mps-file.",1323,1);
															output << setiosflags(io_flags) << setprecision(1) << 1/loc(n);
															output << "        ";
															output << "ROW" << i+1;
															if (i+1 < 10) output << "      ";
															else
																if (i+1 < 100) output << "     ";
																else
																	if (i+1 < 1000) output << "    ";
																	else
																		if (i+1 < 10000) output << "   ";
																		else
																			if (i+1 < 100000) output << "  ";
																			else lola_error("Too much facilities for a mps-file.",1324,1);
																			output << setiosflags(io_flags) << setprecision(1) << 1/loc(n) << endl;
																			i += 2;
						}
					}
					int cnt_l = 0, cnt_k = 1;
					for (i = 2*M*N; i < m; i++) {
						output << "    ";
						output << "COL" << cnt_cols;
						if (cnt_cols < 10) output << "      ";
						else
							if (cnt_cols < 100) output << "     ";
							else
								if (cnt_cols < 1000) output << "    ";
								else
									if (cnt_cols < 10000) output << "   ";
									else
										if (cnt_cols < 100000) output << "  ";
										else lola_error("Too much facilities for a mps-file.",1325,1);
										output << "ROW" << i;
										if (i < 10) output << "      ";
										else
											if (i < 100) output << "     ";
											else
												if (i < 1000) output << "    ";
												else 
													if (i < 10000) output << "   ";
													else
														if (i < 100000) output << "  ";
														else lola_error("Too much facilities for a mps-file.",1326,1);
														output << setiosflags(io_flags) << setprecision(1) << 1/W(cnt_l,cnt_k);
														output << "        ";
														output << "ROW" << i+1;
														if (i+1 < 10) output << "      ";
														else
															if (i+1 < 100) output << "     ";
															else
																if (i+1 < 1000) output << "    ";
																else
																	if (i+1 < 10000) output << "   ";
																	else
																		if (i+1 < 100000) output << "  ";
																		else lola_error("Too much facilities for a mps-file.",1327,1);
																		output << setiosflags(io_flags) << setprecision(1) << 1/W(cnt_l,cnt_k) << endl; i++;
																		cnt_l++;
																		if (cnt_l >= cnt_k) {cnt_k++; cnt_l = 0;}
					}
					// ****  Building the RHS  ****
					output << "RHS" << endl;
					i = 0;
					for (n = 0; n < N; n++) {
						for (list_item trav = first(); trav; trav = succ(trav)) {
							location loc = contents(trav); 
							output << "    RHS1      ROW" << i;
							if (i < 10) output << "      ";
							else
								if (i < 100) output << "     ";
								else
									if (i < 1000) output << "    ";
									else
										if (i < 10000) output << "   ";
										else
											if (i < 100000) output << "  ";
											else  lola_error("Too much facilities for a mps-file.",1328,1);
											output << loc[cnt_LP] << endl; i++;
											output << "    RHS1      ROW" << i;
											if (i < 10) output << "      ";
											else
												if (i < 100) output << "     ";
												else
													if (i < 1000) output << "    ";
													else
														if (i < 10000) output << "   ";
														else
															if (i < 100000) output << "  ";
															else  lola_error("Too much facilities for a mps-file.",1329,1);
															output << -loc[cnt_LP] << endl; i++;
						}
					}
					for (i = 2*M*N; i < m; i++) {
						output << "    RHS1      ROW" << i;
						if (i < 10) output << "      ";
						else
							if (i < 100) output << "     ";
							else
								if (i < 1000) output << "    ";
								else
									if (i < 10000) output << "   ";
									else
										if (i < 100000) output << "  ";
										else  lola_error("Too much facilities for a mps-file.",1330,1);
										output << "0.0" << endl;
					}
					output << "ENDATA" << endl;
					output.close();
   }
   ofstream output("cplex.exe",ios::out);
   output << "read current_problem1.mps" << endl;
   output << "optimize" << endl;
   output << "write sol1.txt" << endl;
   output << "read current_problem2.mps" << endl;
   output << "optimize" << endl;
   output << "write sol2.txt" << endl;
   output << "quit" << endl;
   output.close();
   const char *message3 = "cplexmip < cplex.exe";
   system(message3);
   cout.setf(0,ios::floatfield);
   location solutions[N];
   double z = 0, z1 = 0, z2 = 0;
   // ****  Reading the resultfiles sol1.txt, sol2.txt  ****
   const int maxline = 110;
   char line[maxline];
   for (cnt_LP = 0; cnt_LP < 2; cnt_LP++) {
	   fstream input((cnt_LP) ? "sol2.txt" : "sol1.txt", ios::in);
	   input.seekg(0,ios::beg);
	   int potenz = 0, start_idx = 0, integer = 1, cnt_N = 0;
	   double tmp;
	   while(input.getline(line,maxline)) {
		   if (line[10] == 'C' && line[11] == 'O' && line[12] == 'L') {
			   char value[45];
			   for(i=0;i<45;i++) value[i]=line[30+i];
			   tmp = atof(value);
			   if (cnt_N < N) solutions[cnt_N][cnt_LP] = tmp;
			   if (cnt_N == N && !cnt_LP) z1 = tmp;
			   if (cnt_N == N && cnt_LP) z2 = tmp;
			   cnt_N++;
		   }
		   potenz = 0; start_idx = 0; integer = 1; tmp = 0;
	   }
   }
   z = L_MAX(z1,z2);
   for (i = 0; i < N; i++) {solutions[i](0)=i;solution.append(solutions[i]);}
   return z;
#endif
#ifndef HAVECPLEX
   erase=true;  // Just to suppress a compiler warning
   cerr << "This is a substitute function for the original CPLEX" << endl << "based algorithm, because there is no CPLEX on this system." << endl;
   return N_linf_max(N,W);
#endif
}


double facilities::N_l1_max_mps(int N, matrix &W, bool erase) {
/********************************************************************
* Purpose   : Compute the optimum for problem class N|P|.|l1|max
* Returns   : objective value
* Influences: %
* Date      : Dec 95
* Author    : Ingo Bartling
*********************************************************************/
	if (erase) solution.clear();
	facilities facs = *this;
	retrans_all(facs);
	double z = facs.N_linf_max_mps(N,W);
	solution = facs.solution;
	trans_all(solution);
	return z;
}

/******************* Subroutines for gauge_sum **********************/

/*   Funktion zur Berechnung eines Segmentes im 1-kriteriellen Fall :     */

static   segment segment_computation_1 (location Ex,point Extr_pkt,double f_X,int q)
{   
	segment s(Ex[0],Ex[1],Ex[0]+Extr_pkt.xcoord()*f_X/Ex(q),Ex[1]+Extr_pkt.ycoord()*f_X/Ex(q)) ;
	return s;
};     


/*   Halbraum-Test :   */

static  int halfspace_test (vector S,point X,point Y) {
	vector T(X.xcoord()-Y.xcoord(),X.ycoord()-Y.ycoord());
	int Lage;
	if ( S * T > 0 ) Lage=1; 
	else {  
		if ( S * T == 0 ) Lage=0;
		else Lage=-1;
	}
	return Lage;
}

/*   Funktion zur Berechnung eines Segmentes im Q-kriteriellen Fall : */
static segment segment_computation_Q (location Ex,point Extr_pkt,double max_fqX_Wq,double dist_Gamma,double dist_EX)
{
	segment s(Ex[0],Ex[1],Ex[0]+Extr_pkt.xcoord()*ceil((max_fqX_Wq*dist_Gamma+dist_EX)/Extr_pkt.distance()),Ex[1]+Extr_pkt.ycoord()*ceil((max_fqX_Wq*dist_Gamma+dist_EX)/Extr_pkt.distance()));
	return s;
}

   /*   Halbraum-Test 2 :   */

static int halfspace_test_senk (vector S,point X,point Y)
{
	vector T(Y.ycoord()-X.ycoord(),X.xcoord()-Y.xcoord());
	int Lage;
	if ( S * T > 0 ) Lage=1; 
	else {  
		if ( S * T == 0 ) Lage=0;
		else Lage=-1;
	}
	return Lage;
}     

   /*   Berechnung des "senk-Vektors" von S :   */

static vector senk (vector S)
{
	vector T(2);
	T[0]=-S[1];
	T[1]= S[0];
	return T;
}
     
   /*   Berechnung des planaren Graphen :   */

static  GRAPH<point,segment> computation_of_planar_graph(GRAPH<point,segment> planarer_Graph)
{
    node w;
    edge f;    
    int i,j,nr,anz;
    segment s1,s2;
    double steig1,steig2;
    forall_nodes(w,planarer_Graph)
    {
		anz=planarer_Graph.outdeg(w);
		/* out-edges des Knoten w im Gegenuhrzeigersinn sortieren : */
		for (i=0 ; i < anz-1 ; i=i+1)
		{
			nr=i;
			s1=planarer_Graph.inf
				(planarer_Graph.out_edges(w).inf
				(planarer_Graph.out_edges(w).get_item(i)));
			steig1=s1.direction(); 
			if ( steig1 < 0 ) 
			{ 
				steig1=steig1+2*3.1415926536; 
			};
			for (j=i+1 ; j < anz ; j=j+1 )
			{
				s2=planarer_Graph.inf
					(planarer_Graph.out_edges(w).inf
					(planarer_Graph.out_edges(w).get_item(j)));
				steig2=s2.direction(); 
				if ( steig2 < 0 ) 
				{
					steig2=steig2+2*3.1415926536; 
				};
				if ( steig1 > steig2 )
				{
					nr=j;
					steig1=steig2;
				};  
			};
			f=planarer_Graph.out_edges(w).inf
				(planarer_Graph.out_edges(w).get_item(i));
#if __LEDA__>=380
			planarer_Graph.new_edge(
				planarer_Graph.out_edges(w).inf
				(planarer_Graph.out_edges(w).get_item(i)),
				planarer_Graph.target(
				(planarer_Graph.out_edges(w).inf
				(planarer_Graph.out_edges(w).get_item(nr)))),
				planarer_Graph.inf(
				(planarer_Graph.out_edges(w).inf
				(planarer_Graph.out_edges(w).get_item(nr)))),
				LEDA::after                                             );
			planarer_Graph.del_edge(
				planarer_Graph.out_edges(w).inf
				(planarer_Graph.out_edges(w).get_item(i)));
			planarer_Graph.new_edge(
				planarer_Graph.out_edges(w).inf
				(planarer_Graph.out_edges(w).get_item(nr)),
				planarer_Graph.target(f),
				planarer_Graph.inf(f),
				LEDA::after                                             );
#else
			planarer_Graph.new_edge(
				planarer_Graph.out_edges(w).inf
				(planarer_Graph.out_edges(w).get_item(i)),
				planarer_Graph.target(
				(planarer_Graph.out_edges(w).inf
				(planarer_Graph.out_edges(w).get_item(nr)))),
				planarer_Graph.inf(
				(planarer_Graph.out_edges(w).inf
				(planarer_Graph.out_edges(w).get_item(nr)))),
				after                                             );
			planarer_Graph.del_edge(
				planarer_Graph.out_edges(w).inf
				(planarer_Graph.out_edges(w).get_item(i)));
			planarer_Graph.new_edge(
				planarer_Graph.out_edges(w).inf
				(planarer_Graph.out_edges(w).get_item(nr)),
				planarer_Graph.target(f),
				planarer_Graph.inf(f), after);
#endif
			planarer_Graph.del_edge(
				planarer_Graph.out_edges(w).inf
				(planarer_Graph.out_edges(w).get_item(nr)));
		}; 
		/* Bei Kanten mit gleicher Richtung die laengere Kante aussortieren : */
		i=0;
		while ( i < anz )
		{
			s1=planarer_Graph.inf
				(planarer_Graph.out_edges(w).inf
				(planarer_Graph.out_edges(w).get_item(i)));
			steig1=s1.direction(); 
			if ( steig1 < 0 ) 
			{ 
				steig1=steig1+2*3.1415926536;
			};
			s2=planarer_Graph.inf
				(planarer_Graph.out_edges(w).inf
				(planarer_Graph.out_edges(w).get_item((i+1)%anz)));
			steig2=s2.direction(); 
			if ( steig2 < 0 ) 
			{
				steig2=steig2+2*3.1415926536; 
			};
			if ( steig1 == steig2 )
			{
				if ( s1.length() >= s2.length() )
				{
					planarer_Graph.del_edge(planarer_Graph.out_edges(w).inf
						(planarer_Graph.out_edges(w).get_item(i)));
				}
				else
				{
					planarer_Graph.del_edge(planarer_Graph.out_edges(w).inf
						(planarer_Graph.out_edges(w).get_item((i+1)%anz)));
				};
				anz=anz-1;
				i=i-1;
			};
			i=i+1;
		};
    };
    return planarer_Graph;
};


   /*   Funktion zur Bestimmung von Sit(C,X) :   Variante 2 */

static int situation_test (point XL, point X, point XR, double f1_XL, double f1_X, double f1_XR, double f2_XL, double f2_X, double f2_XR, vector S1, vector S2)
{
	int sit;
	if ( (f1_X <= f1_XL) && (f1_X <= f1_XR) &&
		(f2_X <= f2_XL) && (f2_X <= f2_XR)    )
	{                /*  Fall 1  */ 
		if ( (f1_X == f1_XR) && (f2_X < f2_XL) && (f2_X == f2_XR) )
		{              /*  Faelle  1.2.3 , 1.3.2  */ 
			sit=4;
		}
		else
		{
			if ( (f1_X == f1_XL) && (f2_X == f2_XL) && (f2_X < f2_XR) )
			{               /*  Faelle  1.2.4 , 1.3.3  */ 
				sit=3;
			}
			else
			{               /*  Faelle  1.1 , 1.2.1 , 1.2.2  */ 
				sit=5;
			};
		};
	}
	else     
	{               /*  Faelle  2 , 3 , 4 */ 
		if ( halfspace_test_senk(S1,XR,X)==-1 ) { S1=-S1; };
		if ( halfspace_test_senk(S2,XR,X)==-1 ) { S2=-S2; };
		if ( ( (f1_X > f1_XL) || (f1_X > f1_XR) ) &&
			( (f2_X > f2_XL) || (f2_X > f2_XR) )    )
		{                /*  Fall  4  */ 
			if ( senk(S1)*S2 == 0 )
			{             /*  Fall  4.1  */ 
				sit=1;
			}
			else
			{             /*  Fall  4.2.2  */ 
				sit=2;
			};
		}
		else
		{                /*  Faelle  2 , 3  */ 
			if ( (f1_X <= f1_XL) && (f1_X <= f1_XR) )
			{             /*  Fall  2  */ 
			}
			else
			{             /*  Fall  3  */     
				double hilf;
				hilf=f1_X  ; f1_X =f2_X  ; f2_X =hilf ;
				hilf=f1_XL ; f1_XL=f2_XL ; f2_XL=hilf ;
				hilf=f1_XR ; f1_XR=f2_XR ; f2_XR=hilf ;
				vector Hilf(2);
				Hilf=S1 ; S1=S2 ; S2=Hilf ;
			};
			if ( senk(S1)*S2 == 0 )
			{             /*  Fall  2.1   bzw.  3.1    */ 
				sit=1;
			}
			else          /*  Faelle  2.2 , 2.3   bzw.  3.1    */ 
			{
				if ( senk(S1)*S2 > 0 )
				{              /*  Fall  2.2   bzw.  3.2    */ 
					sit=3;
				}
				else           /*  Fall  2.3   bzw.  3.3    */ 
				{
					if ( halfspace_test(senk(S2),XL,X) <= 0 )
					{              /*  Fall  2.3.1    bzw.  3.3.1    */ 
						sit=4;
					}
					else
					{              /*  Fall  2.3.2    bzw.  3.3.2    */ 
						if ( f2_XL > f2_XR )
						{           /*  Fall  2.3.2.2  bzw.  3.3.2.2  */ 
							sit=4;
						}
						else
						{           /*  Fall  2.3.2.3  bzw.  3.2.3.2  */
							sit=3;
						};
					};
				};
			};
		};
	};  
	return sit;
};  


   /*  Punkt in Liste von Punkten ? :   */

static int point_in_pointlist (point P, list<point> Punkteliste)
{
	int enthalten=0;
	point R;
	forall(R,Punkteliste)
	{
		if ( P == R )
		{
			enthalten=1;
			return enthalten;
		};
	};
	return enthalten;
};


double facilities::objective_value_computation(list<polygauge>& Lg,list<int>& Ln,point X, int q)
/********************************************************************
* Purpose   : Compute the objective value for a gauge problem
* Returns   : objective value
* Influences: %
* Date      : Dec 96
* Author    : Ansgar Weissler (updated from Lothar Schnell)
*********************************************************************/
{
	location Ex(head().dim(),head().wht_dim());
	polygauge G;
	double fw=0;
	int m=0;
	forall(Ex,*this) {  
		G=Lg.inf(Lg.get_item(Ln.inf(Ln.get_item(m))));
		fw+=Ex(q)*G.norm(Ex.loc2point(),X);
		++m;
	}
	return fw;
}


vector facilities::subgradient_computation (list<polygauge>& Lg,list<int>& Ln,point X,int q)
/********************************************************************
* Purpose   : Compute the subgradient
* Returns   : objective value
* Influences: %
* Date      : Dec 96
* Author    : Ansgar Weissler (updated from Lothar Schnell)
*********************************************************************/
{
	location Ex(head().dim(),head().wht_dim());
	polygauge G;
	int i;
	point E_i,E_ip1;
	bool unique;
	vector S(2);
	int m=0;
	forall(Ex,*this) {  
		G=Lg.inf(Lg.get_item(Ln.inf(Ln.get_item(m))));
		i=G.inCone(Ex.loc2point(),X,unique);
		E_i=G.conepoints(origin,i).head();
		E_ip1=G.conepoints(origin,i).tail();
		S[0]=S[0] + Ex(q) * (E_ip1.ycoord()-E_i.ycoord()) /
			( (E_ip1.ycoord()-E_i.ycoord()) * E_i.xcoord() -
			(E_ip1.xcoord()-E_i.xcoord()) * E_i.ycoord()   );

		S[1]=S[1] - Ex(q) * (E_ip1.xcoord()-E_i.xcoord()) /
			( (E_ip1.ycoord()-E_i.ycoord()) * E_i.xcoord() -
			(E_ip1.xcoord()-E_i.xcoord()) * E_i.ycoord()   );
		++m;
	}
	return S;
}

list<point> facilities::choice_of_optimal_points_1(list<polygauge>& Lg,list<int>& Ln,list<point>& Liste_der_Gitterpunkte,point X, double f_X, int q)
/********************************************************************
* Purpose   : Computes the function values for the gridpoints
* Returns   : list of gridpoints
* Influences: %
* Date      : Dec 96
* Author    : Ansgar Weissler (updated from Lothar Schnell)
*********************************************************************/
{
	point Gp;
	double fw;  
	double objval=f_X;
	list<point> Optsol;
	vector S=subgradient_computation(Lg,Ln,X,q);
	forall(Gp,Liste_der_Gitterpunkte)   {
		if ( halfspace_test(S,Gp,X) != 1 ) {
			fw=objective_value_computation(Lg,Ln,Gp,q);
			if ( fw < objval ) {
				Optsol.clear();
				Optsol.append(Gp);
				objval=fw;
			} else {
				if ( fw == objval ) Optsol.append(Gp);
			}
		}
	}
	return Optsol;
}

   /*  Paretoloesung im trivialen Fall ermitteln:   */

static list<location> trivial_case (list<point> Punkteliste,int dim)
{
	location Ex(dim,1);
	point Gp;
	list<location> OPT_SOL;
	forall(Gp,Punkteliste)   
	{
		Ex[0]=Gp.xcoord();
		Ex[1]=Gp.ycoord();
		Ex(0)=1;                   
		OPT_SOL.append(Ex);
	};
	return OPT_SOL;
};

// wieso hat LEDA-3.5.1 eigentlich __LEDA__=352?
#if __LEDA__==350 || __LEDA__==352 || __LEDA__==361
void SEGMENT_INTERSECTION(const list<segment>& S, list<point>& P)
/********************************************************************
* Purpose   : Surregate function for the missing LEDA-function 
* Returns   : % 
* Influences: %
* Date      : Nov 97 
* Author    : Stefan Naeher (included by Holger Hennes)
*********************************************************************/
{ 
	GRAPH<point,segment> G;
	SWEEP_SEGMENTS(S,G);
	P.clear();

	node v;
	forall_nodes(v,G) 
		if (G.degree(v) > 1) P.append(G[v]);
}
#endif

double facilities::gauge_sum(list<polygauge>& Lg, list<int>& Ln, bool erase)
/********************************************************************
* Purpose   : Compute the optimum for problem class 1|P|.|gauge|sum
* Returns   : objective value
* Influences: %
* Date      : Dec 96
* Author    : Ansgar Weissler (updated from Lothar Schnell)
*********************************************************************/
{
	if (erase) solution.clear();

/*  Ueberpruefung des Eingabeformates : */

	if ( head().wht_dim() > 1 ) lola_error("False weight-dimension.",1339,1);
	if ( length() !=  Ln.length() ) lola_error("Different number of locations and gauges.",1340,1);
	if ((Ln.inf(Ln.min()) < 0)||(Ln.inf(Ln.max()) > Lg.size()-1)) lola_error("False gauge-number.",1341,1);
	location Ex(head().dim(),head().wht_dim());
	list_item it;
	for(it=first(); it; it=succ(it)) {
		Ex=contents(it);
		if ( Ex(0) <= 0 ) lola_error("Negative entry or 0-row or 0-column in weight-matrix.",1338,1);
	}

	/*   Berechnen von Y = "gewichteter Schwerpunkt von EX" 
	= "Optimalloesung von 1/P/./l_2^2/Summe " :   */
	l2sqr_sum(0,false);
	point Y=solution.head().loc2point();

/*   Berechnen von f(Y) :   */   

	double f_Y=objective_value_computation(Lg,Ln,Y,0);


/*   Berechnen der Segmente s^m_i :   */

	list<segment> Liste_der_Segmente=all_segments_computation(Lg,Ln,Y,f_Y,0);


/*   Berechnen der Gitterpunkte Gp :   */

	list<point> Liste_der_Gitterpunkte;	
	SEGMENT_INTERSECTION(Liste_der_Segmente,Liste_der_Gitterpunkte);


/*   Berechnen der Funktionswerte der Gitterpunkte Gp und optimale Gitterpunkte auswhlen :              */

	list<point> Optsol=choice_of_optimal_points_1(Lg,Ln, Liste_der_Gitterpunkte, Y,f_Y,0);

	if ( Optsol.length() > 1 )  Optsol=CONVEX_HULL(Optsol);
   
	double objval=objective_value_computation(Lg,Ln,Optsol.head(),0);

	point Gp;
	solution.clear();
	forall(Gp,Optsol) {
		Ex=point2loc(Gp); 
		solution.append(Ex);
	}
	return objval;

}

vector facilities::niveauline_vector_computation(list<polygauge>& Lg,list<int>& Ln,point X,int q)
/********************************************************************
* Purpose   : Computes the direction vector for "Niveaulinie"
* Returns   : sum
* Influences: %
* Date      : Dec 96
* Author    : Ansgar Weissler (updated from Lothar Schnell)
*********************************************************************/
{
   vector S=subgradient_computation(Lg,Ln,X,q);
   double s;
   s   = S[0];
   S[0]=-S[1];
   S[1]= s;
   return S;
}

   
list<segment> facilities::all_segments_computation(list<polygauge>& Lg,list<int>& Ln,point X, double f_X, int q)
/********************************************************************
* Purpose   : Computes the segments of all locations
* Returns   : sum
* Influences: %
* Date      : Dec 96
* Author    : Ansgar Weissler (updated from Lothar Schnell)
*********************************************************************/
{
	list_item it;
	location Ex(head().dim(),head().wht_dim());
	int m=0;
	double max_fqX_Wq=0;
	double dist_Gamma=0;
	double dist_EX=0;
	if (q != 0) {
		location Ex_Hilf=contents(last());
		double W;
		for(it=first(); it; it=succ(it)) {
			Ex=contents(it);
			dist_EX+=Ex.loc2point().distance(Ex_Hilf.loc2point());
			Ex_Hilf=Ex;
		}
		double dist_gamma_m;
		for ( m=0 ; m < Ln.length() ; m=m+1 ) {
			polygauge tempg=Lg.inf(Lg.get_item(Ln.inf(Ln.get_item(m))));
			dist_gamma_m=tempg.max_dist();
			if ( dist_gamma_m > dist_Gamma ) dist_Gamma=dist_gamma_m;
		}
		double f_X_W;
		for ( q=0 ; q < head().wht_dim() ; q=q+1 )   {
			W=0;
			for(it=first(); it; it=succ(it)) {
				Ex=contents(it);
				W+=Ex(q);
			}
			f_X_W=objective_value_computation(Lg,Ln,X,q)/W;
			if ( f_X_W > max_fqX_Wq ) max_fqX_Wq=f_X_W;
		}
	}
	polygauge G;
	list<point> Extr_pkte;
	point Extr_pkt;
	list<segment> Liste_der_Segmente;
	segment s;  
	m=0;
	for(it=first(); it; it=succ(it)) {
		Ex=contents(it);
		G=Lg.inf(Lg.get_item(Ln.inf(Ln.get_item(m))));
		Extr_pkte=G.vertices(origin); 
		forall(Extr_pkt,Extr_pkte) {
			if (q == 0) s=segment_computation_1(Ex,Extr_pkt,f_X,q);
			else s=segment_computation_Q(Ex,Extr_pkt,max_fqX_Wq,dist_Gamma,dist_EX);
			Liste_der_Segmente.append(s);
		}
		++m;
	}   
	return Liste_der_Segmente;
}


list<point> facilities::computation_of_lex_solutions(list<polygauge>& Lg,list<int>& Ln,point X,int q1, int q2, GRAPH<point,segment>&  planarer_Graph)
/********************************************************************
* Purpose   : Computes the lexicographical solution
* Returns   : sum
* Influences: %
* Date      : Dec 96
* Author    : Ansgar Weissler (updated from Lothar Schnell)
*********************************************************************/
{
	list<point> Lex_sol;
	double fw_1;
	double fw_2;
	double objval_1=objective_value_computation(Lg,Ln,X,q1);
	double objval_12=objective_value_computation(Lg,Ln,X,q2);
	vector S=subgradient_computation(Lg,Ln,X,q1);
	node v;
	point Gp;
	forall_nodes(v,planarer_Graph)
	{
		Gp=planarer_Graph.inf(v);
		if ( halfspace_test(S,Gp,X) != 1 )
		{
			fw_1=objective_value_computation(Lg,Ln,Gp,q1);
			if ( fw_1 < objval_1 )
			{  
				Lex_sol.clear();
				Lex_sol.append(Gp);
				objval_1=fw_1;
				objval_12=objective_value_computation(Lg,Ln,Gp,q2);
			}                
			else
			{
				if ( fw_1 == objval_1 )
				{
					fw_2=objective_value_computation(Lg,Ln,Gp,q2);
					if ( fw_2 < objval_12 ) 
					{  
						Lex_sol.clear();
						Lex_sol.append(Gp);
						objval_12=fw_2;
					}
					else
					{               
						if ( fw_2 == objval_12 )
						{
							Lex_sol.append(Gp);
						};
					};
				};
			};
		};
	};
	return Lex_sol;
};


list<location> facilities::non_trivial_case(list<polygauge>& Lg,list<int>& Ln,list<point>& Lex_sol_12, list<point>& Lex_sol_21, GRAPH<point,segment>&  planarer_Graph)
/********************************************************************
* Purpose   : Computes the pareto solution for a non trivial case
* Returns   : list<location>
* Influences: %
* Date      : Dec 96
* Author    : Ansgar Weissler (updated from Lothar Schnell)
*********************************************************************/
{
	list<location> OPT_SOL;
	location Ex(head().dim(),1);
	point X,XR,XL,Y;
	vector S1(2),S2(2);
	node u,v,w,pred_u,v_hilf;
	edge e;
	double f1_X,f1_XR,f1_XL,f2_X,f2_XR,f2_XL;
	double f2_u,min_f2_in_cell;
	int sit1,sit2;
	int s=0; /* Situationszaehler */
	X=Lex_sol_12.head();
	/* Knoten mit Information X suchen : */
	v=planarer_Graph.first_node();
	while ( X != planarer_Graph.inf(v) )
	{
		v=planarer_Graph.succ_node(v);
	};
	/* Vorgaenger ersatzweise setzen : */  
	w=planarer_Graph.adj_nodes(v).head();
	/* Solange Ende der Kette noch nicht erreicht : */
	segment stmp1,stmp2;
	while ( point_in_pointlist(X,Lex_sol_21) == 0 )
	{
		s=s+1;
		f1_X =objective_value_computation(Lg,Ln,X ,0);
		f2_X =objective_value_computation(Lg,Ln,X ,1);
		/* Kante "aktueller Knoten == Vorgaengerknoten" suchen : */
		e=planarer_Graph.first_adj_edge(v);

		stmp1=segment(planarer_Graph.inf(planarer_Graph.target(e)), planarer_Graph.inf(v));
		stmp2= segment(planarer_Graph.inf(w), planarer_Graph.inf(v));
		while ( stmp1.angle(stmp2) != 0.0 )
		{
			e=planarer_Graph.cyclic_adj_succ(e);
			stmp1=segment(planarer_Graph.inf(planarer_Graph.target(e)), planarer_Graph.inf(v));
			stmp2= segment(planarer_Graph.inf(w), planarer_Graph.inf(v));
		};
 
	/* Begrenzungsknoten der Zelle setzen : */
		XR=planarer_Graph.inf(planarer_Graph.target(e)); 
		XL=planarer_Graph.inf(planarer_Graph.target
				 (planarer_Graph.cyclic_adj_succ(e)));  
		f1_XR=objective_value_computation(Lg,Ln,XR,0);
		f2_XR=objective_value_computation(Lg,Ln,XR,1);
		f1_XL=objective_value_computation(Lg,Ln,XL,0);
		f2_XL=objective_value_computation(Lg,Ln,XL,1);
	/* innerern Punkt Y bestimmen : */
		Y=point((XL.xcoord()+X.xcoord()+XR.xcoord())/3,
		(XL.ycoord()+X.ycoord()+XR.ycoord())/3);
	/* Steigung der beiden Niveaulinien ermitteln : */
		S1=niveauline_vector_computation(Lg,Ln,Y,0);
		S2=niveauline_vector_computation(Lg,Ln,Y,1);
	/* Situationen ermitteln : */
		sit2=situation_test(XL,X,XR,f1_XL,f1_X,f1_XR,f2_XL,f2_X,f2_XR,S1,S2);
		do
		{
			/* Gegen den Uhrzeigersinn eine Zelle weiterruecken : */
			e=planarer_Graph.cyclic_adj_succ(e);
			sit1=sit2;
			XR=XL;
			f1_XR=f1_XL;
			f2_XR=f2_XL;
			XL=planarer_Graph.inf(planarer_Graph.target
				(planarer_Graph.cyclic_adj_succ(e)));  
			f1_XL=objective_value_computation(Lg,Ln,XL,0);
			f2_XL=objective_value_computation(Lg,Ln,XL,1);
			Y=point((XL.xcoord()+X.xcoord()+XR.xcoord())/3,
				(XL.ycoord()+X.ycoord()+XR.ycoord())/3);
			S1=niveauline_vector_computation(Lg,Ln,Y,0);
			S2=niveauline_vector_computation(Lg,Ln,Y,1);
			sit2=situation_test(XL,X,XR,f1_XL,f1_X,f1_XR,f2_XL,f2_X,f2_XR,S1,S2);
		}   
		while ( ! (    ( ( sit1==1 ) || ( sit2==1 )  )
		   ||  ( ( sit1==2 ) && ( sit2==2 )  )  
		   ||  ( ( sit1==2 ) && ( sit2==4 )  )  
		   ||  ( ( sit1==3 ) && ( sit2==2 )  )  
		   ||  ( ( sit1==3 ) && ( sit2==4 )  ) ) );  
		if ( ( sit1==1 ) || ( sit2==1 ) )
		{
	  /* Zelle gefunden - Die Extremalpunkte der Kette werden im Gegenuhrzeiger-
			      sinn angegeben, der Extremalpunkt mit dem kleinsten
			      Funktionswert bzgl. der Funktion f_2 wird zum aktuellen                              Ende der Kette . */
			if ( sit1==1 )
			{
				e=planarer_Graph.cyclic_adj_pred(e);
			};
			min_f2_in_cell=objective_value_computation(Lg,Ln,planarer_Graph.inf(v),1);
			u=planarer_Graph.target(e);
			pred_u=v;
			v_hilf=v;       
			do
			{
				e=planarer_Graph.first_adj_edge(u);
				while ( planarer_Graph.target(e) != pred_u )
				{
					e=planarer_Graph.cyclic_adj_pred(e);
				};
				f2_u=objective_value_computation(Lg,Ln,planarer_Graph.inf(u),1);
				if ( f2_u < min_f2_in_cell )
				{
					min_f2_in_cell=f2_u;
					v=u;
					w=pred_u; 
				};
				X=planarer_Graph.inf(u);
				Ex[0]=X.xcoord();
				Ex[1]=X.ycoord();
				Ex(0)=s;                   
	    /*  Ex=Gp.point2loc();  */
				OPT_SOL.append(Ex);
				pred_u=u;
				u=planarer_Graph.target(
				planarer_Graph.cyclic_adj_pred(e));
			}
			while ( pred_u != v_hilf );  // ok, da Nachabfrage
		}
		else
		{
	  /* Facette gefunden : */
			Ex[0]=X.xcoord();
			Ex[1]=X.ycoord();
			Ex(0)=s;                   
	  /*  Ex=Gp.point2loc();  */
			OPT_SOL.append(Ex);
			Ex[0]=XR.xcoord();
			Ex[1]=XR.ycoord();
			Ex(0)=s;                   
	  /*  Ex=Gp.point2loc();  */
			OPT_SOL.append(Ex);
			w=v;
	  /* aktuelles Ende der Kette : */
			v=planarer_Graph.target(e);
		};
		X=planarer_Graph.inf(v);
    };
	return OPT_SOL;  
};


double facilities::gauge_bi_crit_sum(list<polygauge>& Lg, list<int>& Ln, bool erase)
/********************************************************************
* Purpose   : Compute the optimum for problem class 1|P|.|gauge|2-sum_par
* Returns   : number of situations of pareto set
* Influences: %
* Date      : Dec 96
* Author    : Ansgar Weissler (updated from Lothar Schnell)
*********************************************************************/
{   
	if (erase) solution.clear();
	double f_Y=0;
	polygauge G;
	point Extr_pkt;
	list<point> Extr_pkte;

/*  Ueberpruefung des Eingabeformates : */

	if ( head().wht_dim() != 2 ) lola_error("False weight-dimension.",1343,1);
	if ( length() !=  Ln.length() ) lola_error("Different number of locations and gauges.",1344,1);
	if ((Ln.inf(Ln.min()) < 0)||(Ln.inf(Ln.max()) > Lg.size()-1)) lola_error("False gauge-number.",1345,1);
	double W1=0;
	double W2=0;
	location Ex(head().dim(),head().wht_dim());
	list_item it;
	for(it=first(); it; it=succ(it)) {
		Ex=contents(it);
		if (((Ex(0)< 0)||(Ex(1)< 0))||((Ex(0)==0)&&(Ex(1)==0)))  lola_error("Negative entry or 0-row or 0-column in weight-matrix.",1346,1);
		W1+=Ex(0);
		W2+=Ex(1);
	}
	if ((W1==0)||(W2==0)) lola_error("Negative entry or 0-row or 0-column in weight-matrix.",1342,1);

	/*   Berechnen von Y = "Schwerpunkt von EX" 
	= "Optimalloesung von 1/P/./l_2^2/Summe " :   */

	l2sqr_sum(0,false);
	point Y=solution.head().loc2point();

	/*   Berechnen der Segmente s^m_i :   */
	list<segment> Liste_der_Segmente=all_segments_computation(Lg,Ln,Y,f_Y,1);


	/*   Berechnen des planaren Graphen G_p=(GP,F) und seiner Einbettung:  */

	GRAPH<point,segment> planarer_Graph;
	SWEEP_SEGMENTS(Liste_der_Segmente,planarer_Graph);
	planarer_Graph.insert_reverse_edges();       
	/*  Fehler in LEDA-Prozedur SWEEP-SEGMENTS korrigieren :  */     
	edge d;
	forall_edges(d,planarer_Graph) 
		planarer_Graph.assign(d,segment(planarer_Graph.inf(planarer_Graph.source(d)),planarer_Graph.inf(planarer_Graph.target(d))));

	/*  Adjazente Facetten in die richtige Reihenfolge bringen :  */
	planarer_Graph=computation_of_planar_graph(planarer_Graph);

	/*  Berechnen der beiden lexikographischen Loesungen X_12 und X_21 :  */

	list<point> Lex_sol_12=computation_of_lex_solutions(Lg,Ln,Y,0,1,planarer_Graph);
	list<point> Lex_sol_21=computation_of_lex_solutions(Lg,Ln,Y,1,0,planarer_Graph);

	/*  Trivialer Fall oder nicht-trivialer Fall :  */
	solution.clear();
	if (point_in_pointlist(Lex_sol_12.head(),Lex_sol_21)==1) solution=trivial_case(Lex_sol_12,head().dim());
	else solution=non_trivial_case(Lg,Ln,Lex_sol_12,Lex_sol_21,planarer_Graph);

	return solution.tail()(0);
} 

/******************* Subroutines for gauge_max **********************/


void sort_points(list<point> &LP) 
{
	point vertex1,vertex2;
	if (LP.length()<=1) return;
	else {
		double sumx=0, sumy=0;
		forall(vertex1,LP) {
			sumx+=vertex1.xcoord();
			sumy+=vertex1.ycoord();
		}
		point own_origin(sumx/LP.length(),sumy/LP.length());
		list<point> PL, hilfPL;
		PL.append(LP.head());
		hilfPL=PL;
		for(list_item it1=LP.succ(LP.first()); it1; it1=LP.succ(it1)) {
			bool done=false;
			vertex1=LP.contents(it1);
			for(list_item it2=PL.first(); it2 && !done; it2=PL.succ(it2)) {
				vertex2=PL.contents(it2);
				if (own_origin.angle(vertex1,vertex2)>LEDA_PI) {
					PL.insert(vertex1,it2,LEDA::before);
					done=true;
				}
			}
			if (!done) PL.append(vertex1);
		}
		LP=PL;
	}
}

polygon poly_intersection(const polygon& Pol1, polygon& Pol2)
{
	point vertex,in_vertex;
	bool inside=true;

	forall_vertices(vertex,Pol1) {
		if (!Pol2.contains(vertex)) inside =false;
	}
	if (inside) { return Pol1;}
	inside = true;
	forall_vertices(vertex,Pol2) {
		if (!Pol1.contains(vertex)) inside =false;
	}
	if (inside) { return Pol2;}
	GRAPH<point,segment> G;
	list<segment> SG=Pol1.edges();
	segment seg;
	forall_segments(seg,Pol2)
		SG.append(seg);
	list<point> PL=Pol1.vertices();
	forall_vertices(in_vertex,Pol2)
		PL.append(in_vertex);
	list<point> LP,RP;
	SEGMENT_INTERSECTION(SG,LP);
	if (!LP.empty()) {
		forall(vertex,LP) {
			bool equal=false;
			forall(in_vertex,PL) {
				if (vertex==in_vertex)
					equal=true;
			}
			if (!equal) {
				RP.append(vertex);
			}
			else {
				if (Pol1.contains(vertex))
					if (Pol2.contains(vertex))
						RP.append(vertex);
			}
		}
	}
    	else RP=LP;
	sort_points(RP);
	if (!RP.empty()) {
		polygon result(RP);
		return result;
	}
	polygon result;
	return result;
}


double facilities::intervallschacht(list<double>& niveauliste,list<int>& Ln, list<polygauge>& Lg, double zmax, double zoben)
{
	list<double> hilfliste;
	double ztest=(zoben+zmax)/2;
	double tempe;
	niveauliste.sort();
	if(niveauliste.length()%2==1)
	{
		int med=(niveauliste.length()-1)/2;
		ztest=niveauliste.contents(niveauliste.get_item(med));
	}
	else
	{
		int med=(niveauliste.length()-2)/2;
		ztest=(niveauliste.contents(niveauliste.get_item(med))+niveauliste.contents(niveauliste.get_item(med+1)))/2;
	}

	int test = this->test1(ztest, Ln, Lg);
	if(test==0)
	{
		forall(tempe, niveauliste)
		{
			if(tempe>ztest)  hilfliste.append(tempe);
		}
		hilfliste.push(ztest);
	}
	else
	{
		forall(tempe, niveauliste)
		{
			if (tempe<ztest)    hilfliste.append(tempe);
		}
		hilfliste.append(ztest);
	}
	niveauliste=hilfliste;
	return ztest;
}

double gauge_median(list<double>& niveauliste)
{
	double t7;
	int med=(niveauliste.length()-1)/2;
	if(niveauliste.length()<120) 
		niveauliste.sort(); /*kurze Liste wird schneller komplett sortiert*/
	else
    {
		int l7=0;
		int r7=niveauliste.length()-1;
		while (r7>l7)
		{
			double v7=niveauliste.contents(niveauliste.get_item(r7));
			int i7=l7;
			int j7=r7;
			while (j7>i7)
			{
				while (niveauliste.contents(niveauliste.get_item(i7))<v7&&j7>i7)
				{
					i7=i7+1;
				}
				while (niveauliste.contents(niveauliste.get_item(j7))>=v7&&j7>i7)
				{
					j7=j7-1;
				}
				t7=niveauliste.contents(niveauliste.get_item(i7));
#if __LEDA__>=380
				niveauliste.insert(niveauliste.contents(niveauliste.get_item(j7)),niveauliste.get_item(i7), LEDA::after);
				niveauliste.del_item(niveauliste.get_item(i7));
				niveauliste.insert(t7, niveauliste.get_item(j7), LEDA::after);
				niveauliste.del_item(niveauliste.get_item(j7));
			}
			niveauliste.insert(niveauliste.contents(niveauliste.get_item(i7)), niveauliste.get_item(j7), LEDA::after);
			niveauliste.del_item(niveauliste.get_item(j7));
			niveauliste.insert(niveauliste.contents(niveauliste.get_item(r7)), niveauliste.get_item(i7), LEDA::after);
			niveauliste.del_item(niveauliste.get_item(i7));
			niveauliste.insert(t7, niveauliste.get_item(r7), LEDA::after);
#else
				niveauliste.insert(niveauliste.contents(niveauliste.get_item(j7)),niveauliste.get_item(i7), after);
				niveauliste.del_item(niveauliste.get_item(i7));
				niveauliste.insert(t7, niveauliste.get_item(j7), after);
				niveauliste.del_item(niveauliste.get_item(j7));
			}
			niveauliste.insert(niveauliste.contents(niveauliste.get_item(i7)), niveauliste.get_item(j7), after);
			niveauliste.del_item(niveauliste.get_item(j7));
			niveauliste.insert(niveauliste.contents(niveauliste.get_item(r7)), niveauliste.get_item(i7), after);
			niveauliste.del_item(niveauliste.get_item(i7));
			niveauliste.insert(t7, niveauliste.get_item(r7), after);
#endif
			niveauliste.del_item(niveauliste.get_item(r7));
			if (i7>=med)    r7=i7-1;
			if (i7<=med)    l7=i7+1;
		}
    }
	double ztest;
	if(niveauliste.length()%2==1)
    {
		med=(niveauliste.length()-1)/2;
		ztest=niveauliste.contents(niveauliste.get_item(med));
    }
	else
    {
		med=(niveauliste.length()-2)/2;
		ztest=(niveauliste.contents(niveauliste.get_item(med))+niveauliste.contents(niveauliste.get_item(med+1)))/2;
    }
	return ztest;
}/*Ende von median*/

void facilities::doppel(list<point>& OPTLIST)
{
	point PP;
	location loc(2, 1);
	int hilf5;
	list<location> OPTIMUM;
	forall(PP, OPTLIST)
    {
		hilf5=0;
		location optimum=(point2loc(PP));
		forall(loc, OPTIMUM)
		{
			if(optimum[0]>=loc[0])
			{
				if(optimum[0]-loc[0]<=100*L_eps)
				{
					if(optimum[1]>=loc[1])
					{
						if(optimum[1]-loc[1]<=100*L_eps)   hilf5=hilf5+1;
					}
					else
					{
						if(loc[1]-optimum[1]<=100*L_eps)   hilf5=hilf5+1;
					}
				}
			}
			else
			{
				if(loc[0]-optimum[0]<=100*L_eps)
				{
					if(optimum[1]>=loc[1])
					{
						if(optimum[1]-loc[1]<=100*L_eps)   hilf5=hilf5+1;
					}
					else
					{
						if(loc[1]-optimum[1]<=100*L_eps)   hilf5=hilf5+1;
					}
				}
			}
		}
		if (hilf5==0)  OPTIMUM.append(optimum);
    }
	solution = OPTIMUM;
	return;
}/*doppel*/

double facilities::opt1(double& z, list<int>& Ln, list<polygauge>& Lg,int& hilf3,list<point>& OPTLIST)
{

	point tmp;
	hilf3=0;
	list<point> L;
	int gnumi;
	location tmpex=this->head();
	gnumi=Ln.contents(Ln.get_item(0));
	polygauge gaugei=Lg.contents(Lg.get_item(gnumi));
	list<point> L3=gaugei.vertices(origin);                     /*Eckpunktliste*/
	forall(tmp, L3)
    {
		double xE=(z+L_eps)/tmpex(0)*tmp.xcoord()+tmpex[0];
		double yE=(z+L_eps)/tmpex(0)*tmp.ycoord()+tmpex[1];
		point p(xE,yE);
		L.push(p);
    }
	polygon P(L);

	for (int i=1;i<this->length();i++)
    {
		L.clear();
		tmpex=this->contents(this->get_item(i));
		gnumi=Ln.contents(Ln.get_item(i));
		gaugei=Lg.contents(Lg.get_item(gnumi));
		L3=gaugei.vertices(origin);
		forall(tmp,L3)
		{
			double xE=(z+L_eps)/tmpex(0)*tmp.xcoord()+tmpex[0];
			double yE=(z+L_eps)/tmpex(0)*tmp.ycoord()+tmpex[1];
			point p(xE,yE);
			L.push(p);
		}
		polygon Q(L);
		if(!P.empty())
		{
			polygon S=poly_intersection(P,Q);
	   /*Schnitt von Q und bisherigem Durchschnitt*/
			P=S;
		}
	}
	if(P.empty())
	{
		hilf3=1;
		double zmax=z;
	}
	else
	{
		OPTLIST=P.vertices();
		this->doppel(OPTLIST);
	}
	return z;
}       /*opt1*/

polygon facilities::opt11 (double& z, list<int>& Ln, list<polygauge>& Lg)
{
	point tmp;

	list<point> L;
	int gnumi;
	location tmpex=this->head();
	gnumi=Ln.contents(Ln.get_item(0));
	polygauge gaugei=Lg.contents(Lg.get_item(gnumi));
	list<point> L3=gaugei.vertices(origin);
	/*Eckpunktliste*/
	forall(tmp, L3)
   	{
		double xE=z/tmpex(0)*tmp.xcoord()+tmpex[0];
		double yE=z/tmpex(0)*tmp.ycoord()+tmpex[1];
		point p(xE,yE);
		L.push(p);
   	}
	polygon P(L);
	for (int i=1;i<this->length();i++)
  	{
		L.clear();
		tmpex=this->contents(this->get_item(i));
		gnumi=Ln.contents(Ln.get_item(i));
		gaugei=Lg.contents(Lg.get_item(gnumi));
		L3=gaugei.vertices(origin);
		forall(tmp,L3)
		{
			double xE=z/tmpex(0)*tmp.xcoord()+tmpex[0];
			double yE=z/tmpex(0)*tmp.ycoord()+tmpex[1];
			point p(xE,yE);
			L.push(p);
		}
		polygon Q(L);
                        polygon S=poly_intersection(P,Q);
           /*Schnitt von Q und bisherigem Durchschnitt*/
                        P=S;
    	}
	return P;
}/*opt11*//*Niveaumenge zum Niveau ztest, wenn ztest > zmax;*/


int facilities::test1(double& ztest, list<int>& Ln, list<polygauge>& Lg, double& zoben)
{

	point tmp;

	list<point> L;
	int gnumi;
	location tmpex=this->head();
	gnumi=Ln.contents(Ln.get_item(0));
	polygauge gaugei=Lg.contents(Lg.get_item(gnumi));
	list<point> L3=gaugei.vertices(origin);                     /*Eckpunktliste*/
	forall(tmp, L3)
	{
		double xE=(ztest-L_eps)/tmpex(0)*tmp.xcoord()+tmpex[0];
		double yE=(ztest-L_eps)/tmpex(0)*tmp.ycoord()+tmpex[1];
		point p(xE,yE);
		L.push(p);
	}
	polygon P(L);
	for (int i=1;i<this->length();i++)
	{
		L.clear();
		tmpex=this->contents(this->get_item(i));
		gnumi=Ln.contents(Ln.get_item(i));
		gaugei=Lg.contents(Lg.get_item(gnumi));
		list<point> L3=gaugei.vertices(origin);
		forall(tmp,L3)
		{
			double xE=(ztest-L_eps)/tmpex(0)*tmp.xcoord()+tmpex[0];
			double yE=(ztest-L_eps)/tmpex(0)*tmp.ycoord()+tmpex[1];
			point p(xE,yE);
			L.push(p);
		}
		polygon Q(L);
		if(!P.empty())
		{
                	polygon S=poly_intersection(P,Q);
           /*Schnitt von Q und bisherigem Durchschnitt*/
                        P=S;
		}
	}
	int test;
	if(P.empty())
	{
		test=0;
		double zmax=ztest;
	}
	else
	{
		test=1;
		double zoben=ztest;
	}
	return test;
}       /*test1*/

double facilities::gauge_max(list<polygauge>& Lg, list<int>& Ln ,bool erase)
/********************************************************************
* Purpose   : Compute the optimum for problem class 1|P|.|gauge|max
* Returns   : objective value
* Influences: %
* Date      : Sep 97
* Author    : Ralf Leipe (updated from Holger Hennes)
*********************************************************************/
{

	double kgb;
	double tempe;
	location tmpex1;

	int schnell=500;           /*beschraenkt die Niveaumenge zunaechst auf Laenge 500*/
	/*bei kleinen Problemen lohnt es sich nicht*/
	if (this->length()<6)   schnell=5000;
	if (this->length()>101)  schnell=1000;
	list<double> niveauliste;
	list<double> niveauhilfe;
	int jnum,lnum,gnumi,gnumk,gnuml,tmpex1num,onum;
	double z=0;
	for (int i=0;i<this->length();i++)
	{
		location tmpex=this->contents(this->get_item(i));           
		/*kritische Situation a*/
		/*Standort i, Ecke j*/
		gnumi=Ln.contents(Ln.get_item(i));
		polygauge gaugei=polygauge(Lg.contents(Lg.get_item(gnumi)));
		list<point> Listei=gaugei.vertices(origin);
		for (jnum=0;jnum<Listei.length();jnum++)
		{
			point j=Listei.contents(Listei.get_item(jnum));
			for (int k=0;k<this->length();k++)
			{
				if(k!=i)
					tmpex1=this->contents(this->get_item(k));   /*Kugel k f3r Facette */
				gnumk=Ln.contents(Ln.get_item(k));
				polygauge gaugek=polygauge(Lg.contents(Lg.get_item(gnumk)));
				list<point> Listek=gaugek.vertices(origin);
				for (lnum=0;lnum<Listek.length();lnum++)
					/*Facette von Ecke l und l+1*/
				{
					point l=Listek.contents(Listek.get_item(lnum));
					point m;
					if (l==Listek.tail())  m=Listek.head();
					else  m=Listek.contents(Listek.get_item(lnum+1));
					if (l.xcoord()==m.xcoord())     /* senkrechte Facette*/
					{
						if((j.xcoord()/tmpex(0))-(l.xcoord()/tmpex1(0))==0)  z=0;
						else  z=((tmpex1[0]-tmpex[0])/((j.xcoord()/tmpex(0))-(l.xcoord()/tmpex1(0))));
					}      /*Ecke = Facette setzen*/
					else                                /* sonstige Facette*/
					{
						double c1=j.xcoord()/tmpex(0);
						double c2=j.ycoord()/tmpex(0);
						double d1=tmpex[0];
						double d2=tmpex[1];
						double a=(l.ycoord()-m.ycoord())/(l.xcoord()-m.xcoord());
						double b=tmpex1[1]-a*tmpex1[0];
						double d=(l.ycoord()-a*l.xcoord())/tmpex1(0);
						if(c2-a*c1-d==0)   z=0;
						else  z=(a*d1+b-d2)/(c2-a*c1-d);   /*Ecke=Facette setzen*/
					}
					if(z>0)
					{
						if(niveauliste.length()<schnell)  niveauliste.push(z);
//						else  niveauhilfe.push(z);
					}
				}
			}
		}
	}
	if (niveauliste.length()==2)  niveauliste.push(niveauliste.head());
	/*Megiddo-Verfahren*/
	double ztest;
	while (niveauliste.length()>1)
	{
		list<double> hilfliste;
		ztest = gauge_median(niveauliste);
		int test=test1 (ztest, Ln, Lg); 
					   /*Entscheidungskriterium*/
		if (test==0)
			/*ztest ist unzulaessig*/
		{
			forall(tempe, niveauliste)
			{
				if (tempe>ztest)  hilfliste.append(tempe);		  
			}
	//		hilfliste.push(ztest);
		}
		else
		{
			forall(tempe, niveauliste)
			{
				if (tempe<ztest)  hilfliste.append(tempe);
			}
			hilfliste.append(ztest);
		}
		if (hilfliste.length()==2)
		{
			if(hilfliste.head()!=ztest)
			{
				kgb=hilfliste.head();
				test = test1(kgb, Ln, Lg);
				if (test==0)  hilfliste.Pop();
				else   hilfliste.pop();
			}
			else
			{
				kgb=hilfliste.tail();
				test = test1(kgb, Ln, Lg);
				if (test==0)  hilfliste.pop();
				else   hilfliste.Pop();
			}
			niveauliste=hilfliste;
		}
		else niveauliste=hilfliste;
	}
    z=niveauliste.head();
    double zoben=1.5e8;
    if(niveauliste.length()!=1)   test1(z, Ln, Lg, zoben);
    int hilf3;
    list<point> OPTLIST;
    opt1 (z, Ln, Lg, hilf3,OPTLIST); 
		  /*Niveaumenge zum Niveau z*/
	if (hilf3==1) 	/*algo1a*/
	{   
				      /*kritische Situationen vom Typ b*/
		double zmax = z;
		double a,b,d,e,g,c1,c2,d1,d2,bo,doo,ao;
		list<double> hilfliste;

		z=0;
		for (int i=0;i<this->length()-2;i++)
		{
			location tmpex=this->contents(this->get_item(i));
			gnumi=Ln.contents(Ln.get_item(i));
			polygauge gaugei=polygauge(Lg.contents(Lg.get_item(gnumi)));
			list<point> Listei=gaugei.vertices(origin);
			for (jnum=0;jnum<Listei.length();jnum++)
			{
				point j=Listei.contents(Listei.get_item(jnum));
				for(tmpex1num=i+1;tmpex1num<this->length()-1;tmpex1num++)
				{
					location tmpex1=this->contents(this->get_item(tmpex1num)); /*Kugel k fuer Facette*/
					gnumk=Ln.contents(Ln.get_item(tmpex1num));
					polygauge gaugek=polygauge(Lg.contents(Lg.get_item(gnumk)));
					list<point> Listek=gaugek.vertices(origin);
					for(lnum=0;lnum<Listek.length();lnum++)
					{
						point l=Listek.contents(Listek.get_item(lnum)); /*Facette von l, l+1*/
						for (int oo=tmpex1num+1;oo<this->length();oo++)
						{
							location tmpex2=this->contents(this->get_item(oo));
							gnuml=Ln.contents(Ln.get_item(oo));
							polygauge gaugel=polygauge(Lg.contents(Lg.get_item(gnuml)));
							list<point> Listeo=gaugel.vertices(origin);
							for (onum=0;onum<Listeo.length();onum++)
							{
								point o=Listeo.contents(Listeo.get_item(onum));
								point m,n3,o3;
								if (j==Listei.tail())  m=Listei.head();
								else  m=Listei.contents(Listei.get_item(jnum+1));
								if (l==Listek.tail())  n3=Listek.head();
								else  n3=Listek.contents(Listek.get_item(lnum+1));

								if (o==Listeo.tail())  o3=Listeo.head();
								else  o3=Listeo.contents(Listeo.get_item(onum+1));
								if (j.xcoord()==m.xcoord())
								{
									if (l.xcoord()!=n3.xcoord())
									{
										if (o.xcoord()==o3.xcoord())
										{
											z=0;
										}
										else
										{
											a=(l.ycoord()-n3.ycoord())/(l.xcoord()-n3.xcoord());
											b=tmpex1[1]-a*tmpex1[0];
											d=(l.ycoord()-a*l.xcoord())/tmpex1(0);
											c1=j.xcoord()/tmpex(0);
											c2=a*c1+d;
											d1=tmpex[0];
											d2=a*tmpex[0]+b;
											ao=(o.ycoord()-o3.ycoord())/(o.xcoord()-o3.xcoord());
											bo=tmpex2[1]-ao*tmpex2[0];
											doo=(o.ycoord()-ao*o.xcoord())/tmpex2(0);

											if(c2-ao*c1-doo==0)   z=0;
											else  z=(ao*d1+bo-d2)/(c2-ao*c1-doo);
										}  /*eine senkrechte Facette fij*/
									}
								}
								else
								{
									if (l.xcoord()==n3.xcoord())
									{
										if (o.xcoord()==o3.xcoord())
										{
											z=0;
										}
										else
										{
											a=(j.ycoord()-m.ycoord())/(j.xcoord()-m.xcoord());
											b=tmpex[1]-a*tmpex[0];
											d=(j.ycoord()-a*j.xcoord())/tmpex(0);
											c1=l.xcoord()/tmpex1(0);
											c2=a*c1+d;
											d1=tmpex1[0];
											d2=a*tmpex1[0]+b;
											ao=(o.ycoord()-o3.ycoord())/(o.xcoord()-o3.xcoord());
											bo=tmpex2[1]-ao*tmpex2[0];
											doo=(o.ycoord()-ao*o.xcoord())/tmpex2(0);

											if(c2-ao*c1-doo==0)  z=0;
											else
											{
												z=(ao*d1+bo-d2)/(c2-ao*c1-doo);
											}
										}    /*andere Facette fkl senkrecht*/
									}
									else
									{
										a=(j.ycoord()-m.ycoord())/(j.xcoord()-m.xcoord()); 
										b=tmpex[1]-a*tmpex[0];
										d=(j.ycoord()-a*j.xcoord())/tmpex(0);
										e=(l.ycoord()-n3.ycoord())/(l.xcoord()-n3.xcoord());
										double f1=tmpex1[1]-e*tmpex1[0];
										g=(l.ycoord()-e*l.xcoord())/tmpex1(0);
										if(a==e)
										{
											z=0;
										}
										else
										{
											c1=(g-d)/(a-e);
											c2=a*c1+d;
											d1=(f1-b)/(a-e);
											d2=a*d1+b;
											if (o.xcoord()==o3.xcoord())
											{
												if (((o.xcoord()/tmpex2(0)-c1))==0)  z =0;
												else  z=((d1-tmpex2[0])/(o.xcoord()/tmpex2(0)-c1));
											}
											else
											{
												ao=(o.ycoord()-o3.ycoord())/(o.xcoord()-o3.xcoord());
												bo=tmpex2[1]-ao*tmpex2[0];

												doo=(o.ycoord()-ao*o.xcoord())/tmpex2(0);
												if(c2-ao*c1-doo==0)  z=0;
												else  z=(ao*d1+bo-d2)/(c2-ao*c1-doo);
											}  /*keine Facette senkrecht*/
										}
									}
								}
								if(z>=zmax)
								{
									if(z<=zoben)
									{
										niveauliste.push(z);
									}
								}
							}
						}
					}
				}
			}
		}
		if (niveauliste.length()==2)  niveauliste.push(niveauliste.head());
		/*Megiddo-Verfahren*/
		while (niveauliste.length()>1300)  
		{                                
			ztest = this->intervallschacht(niveauliste, Ln, Lg, zmax, zoben);
		}
		while (niveauliste.length()>1)
	    {
			hilfliste.clear();
			ztest = gauge_median(niveauliste);
			int test = test1 (ztest, Ln, Lg);            
			if (test==0)                             
			{
				forall(tempe, niveauliste)
				{
					if (tempe>ztest)  hilfliste.append(tempe);
				}
				hilfliste.push(ztest);
			}
			else
			{
				forall(tempe, niveauliste)
				{
					if (tempe<ztest)    hilfliste.append(tempe);
				}
				hilfliste.append(ztest);
			}
			if (hilfliste.length()==2)  
			{                        
				if(hilfliste.head()!=ztest)
				{
					kgb=hilfliste.head();
					test = test1(kgb, Ln, Lg);
					if (test==0)  hilfliste.Pop();
					else   hilfliste.pop();
				}
				else
				{
					kgb=hilfliste.tail();
					test = test1(kgb, Ln, Lg);
					if (test==0)  hilfliste.pop();
					else   hilfliste.Pop();
				}
				niveauliste=hilfliste;
			}
			else  niveauliste=hilfliste;
		}
		z=niveauliste.head();
		int in = 0;
		list<point> OPTLIST;
		opt1 (z, Ln, Lg, in, OPTLIST);                       
		zmax=z;  
	}
	return z;
}/*algo1a*/
