#include <LOLA/location.h>
#include <stdarg.h>

location::location(int n,int m) {
  coord = Loc_Vector(n);
  whts = Loc_Vector(m);
}

location::location(double dim,...) {
   va_list arguments;
   va_start (arguments, dim);
   int d = (int)dim;
   int w = (int)va_arg(arguments, double);
   coord = Loc_Vector(d);
   whts = Loc_Vector(w);
   for (int i = 0; i < d; i++)
     coord[i] = va_arg(arguments, double);
   if (w > 0)
       for (int i = 0; i < w; i++)
	   whts[i] = va_arg(arguments, double);
   va_end(arguments);
}

location::location(const location &L) {
   if (L.dim()>0 && L.wht_dim()>0) {
      coord = Loc_Vector(L.dim());
      whts = Loc_Vector(L.wht_dim());
      coord = L.coord;
      whts = L.whts;
   }
}

location::~location() {}

double location::norm(int p) const {         
   if (p > 0) return (*this).pos().norm(p);
   if (p == L_inf || p == L_max) {
      double Max = 0;
      for (int i = 0; i < (*this).dim(); i++)
	if (fabs((*this)[i]) > Max) Max = fabs((*this)[i]);
      return Max;
   }
   lola_error("Bad p-Norm argument in location::norm(int p).",1201,1);
   return 0;
} 

void location::transform() {
   Loc_Vector tmp(2);
   tmp[0] = 0.5 * ((*this)[0] + (*this)[1]);
   tmp[1] = 0.5 * ((*this)[1] - (*this)[0]);
   (*this).pos() = tmp;
}

void location::retransform() {
   Loc_Vector tmp(2);
   tmp[0] = (*this)[0] - (*this)[1];
   tmp[1] = (*this)[0] + (*this)[1];
   (*this).pos() = tmp;
}

double location::r_angle(const location &L) const {
   double result = 0;
   if (coord.norm(2) == 0 || L.coord.norm(2) == 0) {
      lola_error("Zero length location in location::r_angle().",1202,1);
      return result;
   }
   result = ((*this).pos() * L.pos()) / (coord.norm(2) * L.coord.norm(2));
   result = acos(result);
   return result;
}

double location::d_angle(const location &L) const {
   double result = 0;
   result = r_angle(L);
   result = 180.0 * result/LEDA_PI;
   return result;
}

point location::loc2point() const {
   if (dim()!=2) lola_error("Conversion to only possible for dimension 2.",1203,1);
   location generic_pointer(*this); 
   point p(generic_pointer[0],generic_pointer[1]);
   return p;
}

vector location::loc2vector() const {
   if (dim()==0) lola_error("Coversion to empty vector impossible.",1204,1);
   location generic_pointer(*this);
   vector v(dim());
   for(int i=0;i<dim();i++) v[i]=generic_pointer[i];
   return v;
}

segment location::makesegment(const location &l1) const {
   point   p=loc2point();
   point   q=l1.loc2point();
   segment s(p,q);
   return s;
}

void location::test_coords(const location &test) const { 
   if (dim() != test.dim()) lola_error("Incompatible dimension.",1205,1);
}

void location::test_weights(const location &test) const {
   if (wht_dim() != test.wht_dim()) lola_error("Incompatible number of weights.",1206,1);
}

location& location::operator=(const location &L) {
   if (*this == L) return *this;
   coord = L.coord;
   whts = L.whts;
   return *this;
}

double& location::operator[] (int cindex) {
   if (cindex < 0 || cindex >= dim()) lola_error("Index out of range.",1207,1);
   return coord[cindex];
}

double location::operator[] (int cindex) const {
   if (cindex < 0 || cindex >= dim()) lola_error("Index out of range.",1208,1);
   double result = pos()[cindex];
   return result;
}

double& location::operator()(int windex) {
   if (windex < 0 || windex >= wht_dim()) lola_error("Index out of range.",1209,1);
   return whts[windex];
}

double location::operator()(int windex) const {
   if (windex < 0 || windex >= wht_dim()) lola_error("Index out of range.",1210,1);
   double result = wht()[windex];
   return result;
}

double location::operator*(const location &multiplier) const {
   test_coords(multiplier);
   double output=0;
   output = pos() * multiplier.pos();
   return output;
}

location location::operator*(const double &scale) const {
   location output(*this);
   output.pos() *= scale;
   return output; 
} 

location location::operator/(const double &divider) const {
   if (divider==0) lola_error("Division by zero.",1211,1);
   location output(*this);
   output.pos() /= divider;
   return output; 
} 

location location::operator+(const location &summand) const {
   test_coords(summand);
   location output(*this);
   output.pos() += summand.pos(); 
   return output;
}

location location::operator-(const location &summand) const {
   test_coords(summand);
   location output(*this);
   output.pos() -= summand.pos(); 
   return output;
}

location operator-(const location& invert) {
   location zerolocation(invert.dim(),invert.wht_dim());
   return zerolocation - invert;
}

int location::operator==(const location &compare) const {
   if ((*this).pos() != compare.pos() || (*this).wht() != compare.wht()) return 0;
   return 1;
}

int location::operator!=(const location &compare) const {
   return !(*this == compare);
}

bool location::operator < (location &L) {
   if (coord.size() == 2) {
      if (coord[0] < L.coord[0]) return true;
      if (coord[0] == L.coord[0] && coord[1] < L.coord[1]) return true;
   }
   return false;
}

bool location::operator > (location &L) {
   if (coord.size() == 2) {
      if (coord[0] > L.coord[0]) return true;
      if (coord[0] == L.coord[0] && coord[1] > L.coord[1]) return true;
   }
   return false;
}

ostream& operator<<(ostream& os, const location &output) {
   int i;
   for(i = 0; i < output.dim(); i++) os << output[i] << " ";
   os << " ";
   if (output.wht_dim() > 0)
       for(i = 0; i < output.wht_dim(); i++) os <<  output(i) << " ";
   return os;
}

istream& operator >> (istream& is, location &input) {
   int i;
   for(i = 0; i < input.dim(); i++) is >> input[i];
   for(i = 0; i < input.wht_dim(); i++) is >> input(i);
   return is;
}

void location::WriteAsPolygonList (ostream &out, const list<location>&L) 
/********************************************************************
* Purpose   : produce text description of a list of locations meant as
*             a list of polygons (consecutive locations belong to the
*             same polygon if having the same first weight), such as
*             returned by most of the planar algorithms (planealg.solution
*             etc.)
* Returns   : %
* Influences: planealg::WriteOpt, gaugealg::WriteOpt, facilities::WriteOpt,
*             restrictions::WriteOpt
* Date      : July 97
* Author    : MCM
*********************************************************************/
{
  list_item it, next_it;
  int i;
  location curr;

  out << "begin {polygonlist}" << endl;
  for(it = L.first(); it; it = L.succ(it)) {
    if (L.contents(it).wht_dim() == 0) {
      lola_error("Can't happen: L in location::WriteAsPolygonList is not a polygon list?",1212,1);
    }
    // peek if this polygon has more than one vertex
    if ((next_it = L.succ(it)) &&
	((L.contents(next_it))(0) == (L.contents(it))(0))) {
      out << "begin {polygon}" << endl;
      while (1) {
	curr = L.contents(it);
	// I cannot use the ostream<<-operator here, since it would
	// also output the weights
	for(i = 0; i < curr.dim(); i++) {
	  out << curr[i] << " ";
	}
	out << endl;
	if (!next_it) break;
	if ((L.contents(next_it))(0) != curr(0)) break;
	it = next_it;
	next_it = L.succ(next_it);
      }
      out << "end {polygon}" << endl;
      continue; // it has already been advanced
    } else { // only one point
      curr = L.contents(it);
      // I cannot use the ostream<<-operator here, since it would
      // also output the weights
      for(i = 0; i < curr.dim(); i++) {
	out << curr[i] << " ";
      }
      out << endl;
    }
  }
  out << "end {polygonlist}" << endl;
} /* void WriteAsPolygonList (ostream &out, const list<location>&L) */

// ----------------- global functions -----------------------------

location point2loc (const point &p) {
   location result(2.0, 1.0, 0.0, 0.0, 1.0);
   result[0] = p.xcoord();
   result[1] = p.ycoord();
   return result;
}


