// Definition of class circle_barrier

#include <LOLA/circle_barrier.h>

circle_barrier::circle_barrier() {
  _radius = 0;
}

circle_barrier::circle_barrier(const point& p,
			       double r) {
  if (r >= 0) {
    _radius = r;
    _center = p;
  } else circle_barrier();
}

circle_barrier::circle_barrier(double x,
			       double y,
			       double r) {
  if (r >= 0) {
    _center = point(x,y);
    _radius = r;
  } else circle_barrier();
}

circle_barrier::circle_barrier(const point& a,
			       const point& b,
			       const point& c) {
  circle temp(a,b,c);
  _center = point(temp.center());
  _radius = temp.radius();
}

circle_barrier::circle_barrier(const circle_barrier& c) {
  _radius = c._radius;
  _center = c._center;
}

circle_barrier::~circle_barrier() {
  _radius = 0; 
}

circle_barrier& circle_barrier::operator=(const circle_barrier& c) {
  if (this == &c) return *this;
  _center = c._center;
  _radius = c._radius;
  return *this;
}

bool circle_barrier::inside(const point& p) {
  // in LEDA a point is inside a circle even when it is on its edge;
  // we do not want that; so we define our own check for a point being inside of
  // a circle or not 
  if (_center.distance(p) < _radius) return true;
  return false;
}

bool circle_barrier::inside(const location& l) {
  // same functionality as the method above but works on locations which are converted to
  // a point and then passes the point to the above method
  return inside(l.loc2point());
}

bool circle_barrier::on_edge(const point& p) {
  // set up and initialize the return value
  bool retval = false;
  // if the distance from p to the center of the circle is equal to the radius of the 
  // circle the point is located on the edge of the circle
  if (p.distance(_center) == _radius)
    retval = true;
  return retval;
}

bool circle_barrier::on_edge(const location& l) {
  // same functionality as the method above but works on locations which are converted to
  // a point and then passes the point to the above method  
  return on_edge(l.loc2point());
}

list<point> circle_barrier::intersection(const line& l) {
  // set up a LEDA circle ...
  circle temp(_center,_radius);
  // compute the list of points of intersection of the circle and the given line and return it
  return temp.intersection(l);
}

list<point> circle_barrier::intersection(const segment& s) {
  // set up a LEDA circle ...
  circle temp(_center,_radius);
  // compute the list of points of intersection of the circle and the given segment and return it  
  return temp.intersection(s);
}

bool circle_barrier::visible(const point& p1,const point& p2) {
  // initialize and set return value to false (non visible) 
  bool retval = false;
  // make a segment from the two given points
  segment s(p1,p2);
  // get the list of points of intersection of the segment and the circle
  list<point> IsectS=intersection(s);
  // get the number of points of intersection
  int num_of_isct=IsectS.length();  

  // The following conditionals are only for purpose of getting around the defective 
  // intersection method of LEDA's circle and LEDA's segments.
  // I.e.: If a point is located on the edge of a circle LEDA sometimes returns only one point
  // of intersection instead of two. On the other hand LEDA sometimes return one point instead
  // of no points in case of one point is located on the edge of the circle and the other point
  // IS visible.
  // This is the way I abandoned this really BAD behavior...
  if (num_of_isct == 0) retval = true;
  if (num_of_isct == 1) {
    if ((!spccmp(IsectS.head(),p1)) &&
	(!spccmp(IsectS.head(),p2)))    
      retval = false;                  
    if (on_edge(p1) &&
	spccmp(IsectS.head(),p1))
      retval = true;
    if (on_edge(p2) &&
	spccmp(IsectS.head(),p2))
      retval = true;
  }

  // If both points are on the edge of the circle and are not equal they CANNOT be visible...
  if ((on_edge(p1) && on_edge(p2)) && !spccmp(p1,p2))
    retval = false;

 
  // return the decision made by this method concerning visibilty of the points
  // regarding the given circle
  return retval;
}

double circle_barrier::myabs(double d) {
  // there is no double-valued abs in C++, but I need one; so I define my own
  if (d < 0) return -d;
  return d;
}

bool circle_barrier::spccmp(const point& p1,const point& p2) {
  // this is a special method for checking two given points for being equal;
  // they are defined to be equal if there coordinatewise difference is not bigger than
  // a certain value; this method uses "myabs"
  bool retval = false;
  if ( (myabs(p1.xcoord()-p2.xcoord()) < 0.1) &&
      ((myabs(p1.ycoord()-p2.ycoord()) < 0.1)) )
    retval = true;
  return retval;
}

point circle_barrier::r_tang_point(const point p) {
  if (on_edge(p)) return p;

  circle temp(_center,_radius);
  
  return temp.right_tangent(p).target();
}

point circle_barrier::l_tang_point(const point p) {
  if (on_edge(p)) return p;
  circle temp(_center,_radius);
  return temp.left_tangent(p).target();
}

list<point> circle_barrier::t_points(const point& p) {
  list<point> retval;
  retval.clear();
  retval.append(l_tang_point(p));
  retval.append(r_tang_point(p));
  return retval;
}

double circle_barrier::LSP2(const point& p1,const point& p2) {
  // initialize and set return value to zero
  double retval = 0;
  // check the two points for visibility regarding the circle
  if (visible(p1,p2)) retval = p1.distance(p2);
  else { // if the points are not visible ...
    // construct a directed halfspace separation of the plane by the line through p1 and the 
    // center of the circle
    halfspace H(p1,_center);
    // find out in which (left/right) halfspace p2 is
    int which = H.in_which(p2);
    // is it in halfspace 1 do the following:
    if (which == 1) { // right side
      // get right point of tangency to circle from p1
      point rt=r_tang_point(p1);
      // get left point of tangency to circle from p2      
      point lt=l_tang_point(p2);
      // retval = Euclidiean distance from p1 to right point of tangency from p1
      retval = p1.distance(rt);
      // add Euclidean distance from p1 to left point of tangency from p2 to retval
      retval += p2.distance(lt);
      // compute the Euclidean distance from left to right point of tangency through the 
      // interior of the circle
      double incirc=rt.distance(lt);
      // add up 2r times the arcsin(interior distance normed by 2r)
      retval += 2 * _radius * asin( (incirc / (2 * _radius)) );
    } else { // left side
      // the following operations are analogous to the above ones, but operate in the 
      // left halfspace and not in the right one
      point lt=l_tang_point(p1);
      point rt=r_tang_point(p2);
      retval += p1.distance(lt);
      retval += p2.distance(rt);
      double incirc=rt.distance(lt);
      retval += 2 * _radius * asin( (incirc / (2 * _radius)) );
    }
  }
  return retval;
}

double circle_barrier::l_2(const location& L,const point& p) {
  // initialze and set up the return value
  double retval = 0;
  // convert location L to a point l ...
  point l(L.loc2point());
  if (inside(l)) return 1000000000.0;
  // ... and check l and p for visibility ... if l and p are visible, that's it ... 
  if (visible(l,p)) retval = l.distance(p);
  // if not ... 
  else {
    // set up and initialize the halfspace description container variable
    int which = 0;
    halfspace H(l,_center);
    which = H.in_which(p);
    // The next conditional is just as the corresponding conditional in the method LSP2 ...
    // exactly the same Well, I do not see any valid argument that this code is redundant...
    // because we save one method here... otherwise there would be a method which could be
    // called from l_2 and LSP2 ... and there would be one more count of message flow
    // and method-to-method coupling making the class more complex. 
    if (which == 1) { // right side
      point rt=r_tang_point(l);
      point lt=l_tang_point(p);
      retval += l.distance(rt);
      retval += p.distance(lt);
      double incirc=rt.distance(lt);
      retval += 2 * _radius * asin( (incirc / (2 * _radius)) );
    } else { // left side
      point lt=l_tang_point(l);
      point rt=r_tang_point(p);
      retval += l.distance(lt);
      retval += p.distance(rt);
      double incirc=rt.distance(lt);
      retval += 2 * _radius * asin( (incirc / (2 * _radius)) );
    }
  }
  // return the l_2 distance in R^2 \ circle
  return retval;
}

double circle_barrier::objective(const facilities& EX,const point& p) {
  // set up and initialize the return value
  double retval = 0;
  if (_center.distance(p) > _radius) {
    // a pointer for traversing the facilities
    list_item trav;
    // traverse the facilities and add up the weighted Euclidean distance from each facility to
    // the given point p 
    for(trav=EX.first();trav;trav=EX.succ(trav)) {
		retval += EX.contents(trav)(0) * l_2(EX.contents(trav),p);
	}
    // return the objective value of p in R^2 \ circle
  } else retval = 10000000000000.0;
  return retval;
}

ostream& operator << (ostream& os,const circle_barrier& B) {
  os << B._center << " " << B._radius;
  return os;
}

double circle_barrier::left_path(const point& p1,const point& p2) {
  return 0;
}

double circle_barrier::right_path(const point& p1,const point& p2) {
  return 0;
}

// ----
// End of definition of class circle_barrier
// ----