#include <LOLA/halfspace.h>
#include <LOLA/lola_error.h>

halfspace::halfspace()
/*****************************************************
* Purpose   : Empty constructor
* Returns   : %
* Influences: _supp1,_supp2,_halfspace,_xfact,_yfact,_bvalu,_xpara,
              _ypara,_m,_dir_on_para
* Date      : Aug 97
* Author    : Michael Ochs
******************************************************/ 
{
  _supp1 = point();
  _supp2 = point();
  _halfspace = 0;
  _xfact = 0;
  _yfact = 0;
  _bvalu = 0;
  _m = 0;
  _xpara = false;
  _ypara = false;
  _dir_on_para = 0;
}

halfspace::halfspace(const point& s1,const point& s2,int leri) 
/*****************************************************
* Purpose   : point-point-int constructor
* Returns   : %
* Influences: _supp1,_supp2,_halfspace,_xfact,_yfact,_bvalu,_xpara,
              _ypara,_m,_dir_on_para
* Date      : Aug 97
* Author    : Michael Ochs
******************************************************/ 
{
  _xfact = 0;
  _yfact = 0;
  _bvalu = 0;
  _m = 0;
  _xpara = false;
  _ypara = false;
  _dir_on_para = 0;
  if (leri < 0) _halfspace = -1;
  else _halfspace = 1;
  compute(s1,s2);
}

halfspace::halfspace(const point& s1,const point& s2) 
/*****************************************************
* Purpose   : point-point constructor
* Returns   : %
* Influences: _supp1,_supp2,_halfspace,_xfact,_yfact,_bvalu,_xpara,
              _ypara,_m,_dir_on_para
* Date      : Aug 97
* Author    : Michael Ochs
******************************************************/ 
{
  _xfact = 0;
  _yfact = 0;
  _bvalu = 0;
  _m = 0;
  _xpara = false;
  _ypara = false;
  _dir_on_para = 0;
  _halfspace = 1;
  compute(s1,s2);
}

halfspace::halfspace(const halfspace& H)
/*****************************************************
* Purpose   : Copy constructor
* Returns   : %
* Influences: _supp1,_supp2,_halfspace,_xfact,_yfact,_bvalu,_xpara,
              _ypara,_m,_dir_on_para
* Date      : Aug 97
* Author    : Michael Ochs
******************************************************/ 
{
  _supp1 = point(H._supp1);
  _supp2 = point(H._supp2);
  _halfspace = H._halfspace;
  _xfact = H._xfact;
  _yfact = H._yfact;
  _bvalu = H._bvalu;
  _xpara = H._xpara;
  _ypara = H._ypara;
  _m     = H._m;
  _dir_on_para = H._dir_on_para;
}

halfspace::~halfspace()
/*****************************************************
* Purpose   : Destructor
* Returns   : %
* Influences: _supp1,_supp2,_halfspace,_xfact,_yfact,_bvalu,_xpara,
              _ypara,_m,_dir_on_para
* Date      : Aug 97
* Author    : Michael Ochs
******************************************************/ 
{
  _halfspace = 0;
  _xfact = _yfact = _bvalu = 0;
  _xpara = _ypara = false;
  _dir_on_para = 0;
}

void halfspace::compute(const point& s1,const point& s2) 
/*****************************************************
* Purpose   : Compute directed halfspace separation of plane 
* Returns   : (void)
* Influences: _supp1,_supp2,_xfact,_yfact,_bvalu,_xpara,
              _ypara,_m,_dir_on_para
* Date      : Aug 97
* Author    : Michael Ochs
******************************************************/ 
{
  double s1x=s1.xcoord();
  double s1y=s1.ycoord();
  double s2x=s2.xcoord();
  double s2y=s2.ycoord();
  _xpara=false;
  _ypara=false;
  if (s1x==s2x) _ypara=true;
  if (s1y==s2y) _xpara=true;
  if (_ypara && _xpara) {
    lola_error("halfspace::compute(): Not a hyperplane",1803,1);  
  }
  _supp1 = point(s1);
  _supp2 = point(s2);
  if (_ypara) {
    _yfact=0;
    _xfact=1;
    _bvalu=s1x;
    _m=1000000000.0;
    if (s1y < s2y) _dir_on_para=1;
    if (s1y > s2y) _dir_on_para=-1;
  } else
    if (_xpara) {
      _yfact=1;
      _xfact=0;
      _bvalu=s1y;
      _m=0;
    if (s1x < s2x) _dir_on_para=1;
    if (s1x > s2x) _dir_on_para=-1;      
    } else {
      _xfact=s1y-s2y;
      _yfact=s2x-s1x;
      _bvalu=(_xfact*s1x)+(_yfact*s1y);
      _m=(s2y-s1y)/(s2x-s1x);
    }
}

bool halfspace::check_point(const point& p) 
/*****************************************************
* Purpose   : Check whether point is in defined halfspace
* Returns   : (bool) 
* Influences: %
* Date      : Aug 97
* Author    : Michael Ochs
******************************************************/ 
{
  bool retval = false;
  double cmpr=(_xfact*p.xcoord()+_yfact*p.ycoord()) - _bvalu;
  if (_xpara) {
    if ((cmpr < 0) && (_halfspace > 0)) retval = true;
    if ((cmpr > 0) && (_halfspace < 0)) retval = true;
  } else if (_ypara) {
    if ((cmpr < 0) && (_halfspace < 0)) retval = true;
    if ((cmpr > 0) && (_halfspace > 0)) retval = true;
  } else if (_m < 0) {
    if ((cmpr < 0) && (_halfspace > 0)) retval = true;
    if ((cmpr > 0) && (_halfspace < 0)) retval = true;
  } else if (_m > 0) {
    if ((cmpr < 0) && (_halfspace > 0)) retval = true;
    if ((cmpr > 0) && (_halfspace < 0)) retval = true;
  }
  if ((_xpara || _ypara) && (_dir_on_para < 0)) 
    if (retval) retval = false;
    else retval = true;
  return retval;
}

int halfspace::in_which(const point& p) 
/*****************************************************
* Purpose   : Check in which halfspace a point is (1:right,-1:left)
* Returns   : (int)
* Influences: %
* Date      : Aug 97
* Author    : Michael Ochs
******************************************************/ 
{
  if(check_point(p)) return _halfspace;
  else if(_halfspace==-1) return 1;
  else return -1;
}

string halfspace::say_which(const point& p) 
/*****************************************************
* Purpose   : Check in which halfspace a point is and give
              verbose output
* Returns   : (string)
* Influences: %
* Date      : Aug 97
* Author    : Michael Ochs
******************************************************/ 
{
  string lft("left");
  string rgt("right");
  int wh=in_which(p);
  if (wh==1) return rgt;
  else return lft;
}

ostream& operator << (ostream& os,const halfspace& H) {
  os << "- " << H._supp1 << " - " << H._supp2 << " -";
  return os;
}