#! /usr/local/bin/perl5 -w
# Purpose: compute compilation dependencies for source files and update 
#          the Makefile accordingly. Uses cpp to compute the dependencies
# Features:  - only dependencies with LOLA in their path name are considered
#            - the first line of existing dependencies is preserved upon update
#            - silently ignores not existing Makefile
#            - observes EMACS file variables, i.e. inserts new dependencies
#              above
# Calling convention: - see &usage
# Influences: targets "depend" and "depend.template" in all LOLA Makefiles
# Returns: always 0
# Date: 18.7.1997
# Author: MCM

require 5.001; # I actually don't know if it works with Perl 4, but I sure don't support it

use Getopt::Long;
use English;

my ($Head) = q{$Id: makedependency.pl,v 1.3 1998/06/25 08:26:13 hennes Exp $ };
my ($makefilename) = "Makefile";
my ($CFLAGS) = "-E -M";
my ($progname) = $0;
$progname =~ s%^[^/]+/%%; # chop the path off

my ($opt_version,$opt_help,$verbose);

&usage unless (GetOptions("--version"=>\$opt_version,
			   "-h|--help"=>\$opt_help,
			   "--makefile=s"=>\$makefilename,
			   "--verbose"=>\$verbose)
	       );

&usage if (($#ARGV == -1) || $opt_help);
if ($opt_version) {
    print "This is $progname, Version $Head\n";
    exit(0);
}

my ($GCC) = shift @ARGV;
my ($rc,@cpp,$cpp_start);
my ($found); # whether there already existed a dependency line
my ($make,$make_foot); # space for the new makefile

while ($#ARGV > -1) {
    $_ = shift @ARGV;
    if (/^-(?!-)/) { $CFLAGS .= " $_"; next; }
    if ($_  eq "--") { $_ = shift @ARGV; }
    if ($verbose) { print "Processing $_ :"; }
    $cpp = qx/$GCC $CFLAGS $_/;
    $rc = $? >> 8;
    if ($rc) { 
	warn("Something went wrong, $GCC $CFLAGS -E -M $_ gave return code $rc. Skipping $_.");
	next;
    }
    if ($cpp =~ /^\s*$/) { 
	warn("Something went wrong, $GCC $CFLAGS -E -M $_ gave no output. Skipping $_.");
	next;
    }
    /^(\w+)/; # find base name (assuming to be in pwd)
    $obj = "$1.o";
    unless ($cpp =~ /^$obj\s*:\s*$_/) {
	warn("Something went wrong, $GCC $CFLAGS -E -M $_ output doesn't look like a make dependency line. Skipping $_.");
	next;
    }
    # save start of the output
    $cpp_start = $MATCH; $cpp = $POSTMATCH;
    # delete everything which hasn't "LOLA" in the name
    $cpp =~ s/ (?![^ ]+LOLA|\\)[^ ]+/ /gso;
    # delete empty lines
    $cpp =~ s/^\s*\\\n//gmo;
    # delete multiple space
    $cpp =~ s/ +/ /go;
    # delete a dangling backslash
    $cpp =~ s/\s*\\\n\s*$//o;
    if ($verbose) { 
	my ($i) = 0;
	
	while ($cpp =~ /[^ ]+LOLA[^ ]+/go) { $i++; }
	print " $i dependencies";
    }
    
    next if ($cpp =~ /^\s*$/o); # no dependency, no update

    # Now let's update the $makefilename
    unless (open(MAKE,"<$makefilename")) { next; } # silently ignore not existing $makefilename
    $found = 0; $make = ""; $make_foot = "";
    while (<MAKE>) {
	if (/^#\s*Local\s+Variables/o) { # End of file
	    $make_foot = $_;
	    while ($_ = <MAKE>) { $make_foot .= $_; }
	    last;
	}
	if (/^$obj\s*:/) { # we found the dependency for the file
	    $found = 1;
	    chomp;
	    $dep = $_;
	    # so we read it in completely
	    while (/\\$/) { $_ = <MAKE>; } # and ignore all but the first line
	    $dep =~ s/\s*\\$//;
	    # and substitute by the new depencdency
	    $_ = $dep;
	    chomp $cpp;
	    unless ($cpp =~ /^\s*$/) { $_ .= " \\\n".$cpp."\n"; }
	}
	$make .= $_;
    }
    # if there wasn't a dependency already, append one
    unless ($found) {  
	$make .= $cpp_start; chomp $cpp;
	unless ($cpp =~ /^\s*$/o) { $make .= " \\\n".$cpp."\n\n"; }
    }
    close MAKE;
    unless (open(MAKE,">$makefilename")) { warn("Couldn't write to $makefilename, skipping update for $obj."); }
    print MAKE $make;
    print MAKE $make_foot;
    close MAKE;
    if ($verbose) { print " updated in $makefilename\n"; }
}
exit(0);

sub usage {
    print <<USAGE;
$progname [options] <compiler> [compiler-option|sourcefile]

    compiler       : name of the compiler to use (e.g. gcc or g++)
    compile-option : has to start with -, used if calling the compiler
                     (-- marks the next argument as a sourcefile)
    source-file    : sourcefile, which the compiler can process

Options:
    You have to use -- to seperate the options to $progname from
    the rest of the command line, if present.
    -h
    --help         : print this message
    --version      : print the version of $progname
    --verbose      : report progress
    --makefilename=<filename>
                   : update dependencies in makefile <filename> (instead
                     of Makefile)

$progname computes the files, which each of the given sourcefiles 
depends on (i.e. has a \#include for) and updates the Makefile, if present.
To do so, it calls the compiler with the options $CFLAGS plus the options
given as argument (obviously the -I options are crucial, others are usually
ignored but note that $progname does no checks whatsoever on the
options). compiler-options accumulate along the command line, i.e. each 
sourcefile feels all the options preceding it on the command line.
While updating the Makefile, the first line of dependencies is preserved, if
present. Only dependencies, which have "LOLA" in the path are considered.

Intended for use with gcc/g++ in a Makefile like this:
depend:
	    $progname \$(CC) \$(CFLAGS) \$(SRC)

$Head
USAGE
    exit(0);
}
